@@ -837,6 +837,9 @@ namespace arena
                 mprf("---- Turn #%d ----", turns);
 #endif
 
+                if (crawl_state.terminal_resized)
+                    show_fight_banner();
+
                 // Check the consistency of our book-keeping every 100 turns.
                 if ((turns++ % 100) == 0)
                     count_foes();
@@ -1019,8 +1022,6 @@ namespace arena
             virtual void _render() override {};
             virtual void _allocate_region() override {
                 // XX sometimes this gets called spuriously?
-                show_fight_banner();
-                viewwindow();
                 update_screen();
                 display_message_window();
             };
@@ -1480,9 +1481,9 @@ static void _choose_arena_teams(newgame_def& choice,
     prompt.cprintf("  Sigmund v Jessica\n");
     prompt.cprintf("  99 orc v the Royal Jelly\n");
     prompt.cprintf("  20-headed hydra v 10 kobold ; scimitar ego:flaming");
-    vbox->add_child(make_shared<Text>(move(prompt)));
+    vbox->add_child(make_shared<Text>(std::move(prompt)));
 
-    auto popup = make_shared<ui::Popup>(move(vbox));
+    auto popup = make_shared<ui::Popup>(std::move(vbox));
 
     bool done = false, cancel = false;
     popup->on_hotkey_event([&](const KeyEvent& ev) {
@@ -1493,7 +1494,7 @@ static void _choose_arena_teams(newgame_def& choice,
         return done;
     });
 
-    ui::run_layout(move(popup), done, teams_input);
+    ui::run_layout(std::move(popup), done, teams_input);
 
     if (cancel || crawl_state.seen_hups)
     {
