@@ -22,7 +22,6 @@
 
 #define ART_FUNC_H
 
-#include "act-iter.h"      // For autumn katana
 #include "areas.h"         // For silenced() and invalidate_agrid()
 #include "attack.h"        // For attack_strength_punctuation()
 #include "beam.h"          // For Lajatang of Order's silver damage
@@ -36,7 +35,6 @@
 #include "fineff.h"        // For the Storm Queen's Shield
 #include "god-conduct.h"   // did_god_conduct
 #include "mgen-data.h"     // For Sceptre of Asmodeus
-#include "melee-attack.h"  // For autumn katana
 #include "message.h"
 #include "monster.h"
 #include "mon-death.h"     // For demon axe's SAME_ATTITUDE
@@ -53,6 +51,7 @@
 #include "spl-miscast.h"   // For Spellbinder and plutonium sword miscasts
 #include "spl-monench.h"   // For Zhor's aura
 #include "spl-summoning.h" // For Zonguldrok animating dead
+#include "spl-transloc.h"  // For Autumn Katana's Manifold Assault
 #include "tag-version.h"
 #include "terrain.h"       // For storm bow
 #include "unwind.h"        // For autumn katana
@@ -568,7 +567,17 @@ static void _DEMON_AXE_melee_effects(item_def* /*item*/, actor* attacker,
                 .set_summoned(mons, 6, SPELL_SUMMON_DEMON));
         }
         else if (!you.allies_forbidden())
-            cast_summon_demon(50 + random2(100));
+        {
+            monster_type type = (one_chance_in(3) ? random_demon_by_tier(3)
+                                                  : random_demon_by_tier(4));
+            if (create_monster(
+                    mgen_data(type, BEH_COPY, you.pos(), MHITYOU, MG_FORCE_BEH | MG_AUTOFOE)
+                    .set_summoned(&you, 6, SPELL_SUMMON_DEMON, GOD_NO_GOD)))
+            {
+                mpr("A gate to Pandemonium opens briefly!");
+            }
+        }
+
     }
 }
 
@@ -729,6 +738,17 @@ static void _EOS_unequip(item_def */*item*/, bool */*show_msgs*/)
     invalidate_agrid(true);
 }
 
+///////////////////////////////////////////////////
+static void _BRILLIANCE_equip(item_def */*item*/, bool */*show_msgs*/, bool /*unmeld*/)
+{
+    invalidate_agrid(true);
+}
+
+static void _BRILLIANCE_unequip(item_def */*item*/, bool */*show_msgs*/)
+{
+    invalidate_agrid(true);
+}
+
 ///////////////////////////////////////////////////
 static void _SHADOWS_equip(item_def */*item*/, bool */*show_msgs*/, bool /*unmeld*/)
 {
@@ -863,7 +883,7 @@ static void _WOE_melee_effects(item_def* /*weapon*/, actor* attacker,
     if (!mondied)
         defender->hurt(attacker, defender->stat_hp());
 
-    if (defender->as_monster()->can_bleed())
+    if (defender->as_monster()->has_blood())
     {
         blood_spray(defender->pos(), defender->as_monster()->type,
                     random_range(5, 10));
@@ -875,10 +895,6 @@ static void _WOE_melee_effects(item_def* /*weapon*/, actor* attacker,
 
 static void _DAMNATION_launch(bolt* beam)
 {
-    ASSERT(beam->item
-           && beam->item->base_type == OBJ_MISSILES
-           && !is_artefact(*(beam->item)));
-    beam->item->props[DAMNATION_BOLT_KEY].get_bool() = true;
 
     beam->name    = "damnation bolt";
     beam->colour  = LIGHTRED;
@@ -908,12 +924,16 @@ static void _DAMNATION_launch(bolt* beam)
 static int _calc_elemental_staff_damage(beam_type flavour,
                                         actor* defender)
 {
-    const int base_bonus_dam = 10 + random2(15);
+    int preac = 10 + random2(15);
 
-    if (flavour == BEAM_NONE) // earth
-        return defender->apply_ac(base_bonus_dam);
+    if (flavour == BEAM_FIRE || flavour == BEAM_COLD)
+        preac = div_rand_round(preac * 5, 4);
 
-    return resist_adjust_damage(defender, flavour, base_bonus_dam);
+    const ac_type ac_check = flavour == BEAM_ELECTRICITY ? ac_type::half
+                                                         : ac_type::normal;
+    const int postac = defender->apply_ac(preac, 0, ac_check);
+
+    return resist_adjust_damage(defender, flavour, postac);
 }
 
 static void _ELEMENTAL_STAFF_melee_effects(item_def*, actor* attacker,
@@ -942,10 +962,12 @@ static void _ELEMENTAL_STAFF_melee_effects(item_def*, actor* attacker,
         flavour = BEAM_ELECTRICITY;
         break;
     default:
+        // XXX TODO removeme
         dprf("Bad damage type for elemental staff; defaulting to earth");
         // fallthrough to earth
     case 3:
         verb = "crush";
+        flavour = BEAM_MMISSILE;
         break;
     }
 
@@ -1068,10 +1090,10 @@ static void _FORCE_LANCE_melee_effects(item_def* /*weapon*/, actor* attacker,
                                        actor* defender, bool mondied, int dam)
 {
     if (mondied || !dam || !one_chance_in(3)) return;
-    // max power around a !!! hit (ie ~2d11 on collide from a 34+ damage hit)
-    // no real justification for this, just vibes
-    const int collide_power = min(100, dam * 3);
-    defender->knockback(*attacker, 1, collide_power, "blow");
+    // max power on a !!! hit (ie 36+ damage), but try to make some damage
+    // quite likely to beat AC on any collision.
+    const int collide_damage = 7 + roll_dice(3, div_rand_round(min(36, dam), 4));
+    defender->knockback(*attacker, 1, collide_damage, "blow");
 }
 
 ///////////////////////////////////////////////////
@@ -1340,7 +1362,7 @@ static void _LEECH_equip(item_def */*item*/, bool *show_msgs, bool /*unmeld*/)
 static void _LEECH_melee_effects(item_def* /*item*/, actor* attacker,
                                  actor* defender, bool mondied, int dam)
 {
-    if (attacker->is_player() && defender->can_bleed()
+    if (attacker->is_player() && defender->has_blood()
         && mondied && x_chance_in_y(dam, 729))
     {
         simple_monster_message(*(defender->as_monster()),
@@ -1470,7 +1492,8 @@ static int _harvest_corpses()
         for (stack_iterator si(*ri, true); si; ++si)
         {
             item_def &item = *si;
-            if (item.base_type != OBJ_CORPSES)
+            // Don't encourage hoarding old skeletons. Full corpses only.
+            if (!item.is_type(OBJ_CORPSES, CORPSE_BODY))
                 continue;
 
             // forbid harvesting orcs under Beogh
@@ -1671,50 +1694,17 @@ static void _AUTUMN_KATANA_melee_effects(item_def* /*weapon*/, actor* attacker,
 
     unwind_bool nonrecursive_space(_slicing, true);
 
-    vector<actor *> targets;
-    for (actor_near_iterator ai(attacker, LOS_NO_TRANS); ai; ++ai)
-    {
-        if (defender->pos() == ai->pos())
-            continue;
-
-        if (mons_aligned(attacker, *ai))
-            continue;
-        if (attacker->is_player()
-            && (ai->wont_attack()
-                || mons_attitude(*ai->as_monster()) == ATT_NEUTRAL))
-        {
-            continue;
-        }
-        if (ai->is_monster() && (mons_is_firewood(*ai->as_monster())
-                                 || mons_is_projectile(*ai->as_monster())))
-        {
-            continue;
-        }
-        targets.emplace_back(*ai);
-    }
-
-    if (targets.empty())
+    // If attempting to cast manifold assault would abort (likely because of no
+    // valid targets in range), do nothing
+    if (cast_manifold_assault(*attacker, 0, false, false, defender) == spret::abort)
         return;
 
     mprf("%s slice%s through the folds of space itself!",
          attacker->name(DESC_THE).c_str(),
          attacker->is_player() ? "" : "s");
 
-    // Save the time taken by the player before slicing, was set by the
-    // melee_attack that called us or left over from the player's previous turn
-    unwind_var<int> initial_time(you.time_taken);
-
-    shuffle_array(targets);
-    const size_t max_targets = 4;
-    for (size_t i = 0; i < max_targets && i < targets.size(); i++)
-    {
-        melee_attack atk(attacker, targets[i]);
-        atk.is_projected = true;
-        atk.attack();
-
-        if (!attacker->alive())
-            break;
-    }
+    // Casting with 100 power = up to 4 targets hit
+    cast_manifold_assault(*attacker, 100, false, true, defender);
 }
 
 ///////////////////////////////////////////////////
@@ -1833,3 +1823,34 @@ static void _ASMODEUS_melee_effects(item_def* /*weapon*/, actor* attacker,
         }
     }
 }
+
+////////////////////////////////////////////////////
+
+static void _DOOM_KNIGHT_melee_effects(item_def* /*item*/, actor* attacker,
+                                        actor* defender, bool mondied, int /*dam*/)
+{
+    if (!mondied)
+    {
+        int bonus_dam = random2avg((1 + defender->stat_maxhp() / 10), 3);
+        mprf("%s %s%s",
+            defender->name(DESC_THE).c_str(),
+            defender->conj_verb("convulse").c_str(),
+            attack_strength_punctuation(bonus_dam).c_str());
+        defender->hurt(attacker, bonus_dam);
+    }
+}
+
+///////////////////////////////////////////////////
+static void _CHARLATANS_ORB_equip(item_def */*item*/, bool */*show_msgs*/, bool /*unmeld*/)
+{
+    invalidate_agrid(true);
+    calc_hp(true);
+    calc_mp(true);
+}
+
+static void _CHARLATANS_ORB_unequip(item_def */*item*/, bool */*show_msgs*/)
+{
+    invalidate_agrid(true);
+    calc_hp(true);
+    calc_mp(true);
+}
