@@ -36,6 +36,13 @@ enum beam_type                  // bolt::flavour
     BEAM_UNRAVELLED_MAGIC,
     BEAM_LIGHT,
     BEAM_DEVASTATION,
+    BEAM_UMBRAL_TORCHLIGHT,
+    BEAM_FOUL_FLAME,
+    BEAM_CRYSTALLIZING,
+    BEAM_WARPING,
+    BEAM_QAZLAL,
+    BEAM_HAEMOCLASM,
+    BEAM_BLOODRITE,
 
     // Enchantments
     BEAM_SLOW,
@@ -85,18 +92,19 @@ enum beam_type                  // bolt::flavour
     BEAM_VAMPIRIC_DRAINING,
     BEAM_CONCENTRATE_VENOM,
     BEAM_ENFEEBLE,
-    BEAM_NECROTISE,
+    BEAM_SOUL_SPLINTER,
     BEAM_VITRIFY,
     BEAM_VITRIFYING_GAZE,
     BEAM_WEAKNESS,
     BEAM_CURSE_OF_AGONY,
-    BEAM_LAST_ENCHANTMENT = BEAM_CURSE_OF_AGONY,
+    BEAM_RIMEBLIGHT,
+    BEAM_SHADOW_TORPOR,
+    BEAM_LAST_ENCHANTMENT = BEAM_SHADOW_TORPOR,
 
     BEAM_MEPHITIC,
     BEAM_AIR,
     BEAM_PETRIFYING_CLOUD,
     BEAM_ENSNARE,
-    BEAM_CRYSTAL,
     BEAM_DEATH_RATTLE,
     BEAM_LAST_REAL = BEAM_DEATH_RATTLE,
 
