@@ -5,6 +5,7 @@
 
 #include "item-name.h"
 #include "player.h"
+#include "state.h"
 #include "stringutil.h"
 #include "tag-version.h"
 #include "travel.h"
@@ -60,6 +61,7 @@ static const branch_type logical_branch_order[] = {
     BRANCH_DESOLATION,
     BRANCH_GAUNTLET,
     BRANCH_ARENA,
+    BRANCH_CRUCIBLE,
 };
 COMPILE_CHECK(ARRAYSZ(logical_branch_order) == NUM_BRANCHES);
 
@@ -87,6 +89,7 @@ static const branch_type danger_branch_order[] = {
     BRANCH_CRYPT,
     BRANCH_DESOLATION,
     BRANCH_ABYSS,
+    BRANCH_CRUCIBLE,
     BRANCH_WIZLAB,
     BRANCH_SLIME,
     BRANCH_DEPTHS,
@@ -169,6 +172,15 @@ vector<branch_type> random_choose_disabled_branches()
     for (int i=0; i < number_of_branch_swap_pairs; i++)
         disabled_branch.push_back(swap_branches[i][random_choose(0,1)]);
 
+    // Descent mode disables some other branches for dungeon structure reasons
+    if (crawl_state.game_is_descent())
+    {
+        disabled_branch.push_back(BRANCH_TEMPLE);
+        disabled_branch.push_back(BRANCH_TOMB);
+        you.props[DESCENT_WATER_BRANCH_KEY] = random_choose(BRANCH_SWAMP, BRANCH_SHOALS);
+        you.props[DESCENT_POIS_BRANCH_KEY] = random_choose(BRANCH_SPIDER, BRANCH_SNAKE);
+    }
+
     return disabled_branch;
 }
 
@@ -233,6 +245,11 @@ bool is_connected_branch(level_id place)
     return is_connected_branch(place.branch);
 }
 
+bool branch_has_rune(branch_type branch)
+{
+    return !branches[branch].runes.empty();
+}
+
 branch_type branch_by_abbrevname(const string &branch, branch_type err)
 {
     for (branch_iterator it; it; ++it)
@@ -295,6 +312,11 @@ branch_type parent_branch(branch_type branch)
     return branches[branch].parent_branch;
 }
 
+vector<branch_type> descent_parents(branch_type branch)
+{
+    return branches[branch].descent_parents;
+}
+
 /**
  * Describe the ambient noise level in this branch.
  *
