@@ -27,6 +27,7 @@
 #include "melee-attack.h"
 #include "message.h"
 #include "mon-behv.h"
+#include "mon-clone.h"
 #include "mon-death.h"
 #include "mon-place.h"
 #include "nearby-danger.h" // Compass (for random_walk, CloudGenerator)
@@ -34,13 +35,16 @@
 #include "religion.h"
 #include "shout.h"
 #include "spl-clouds.h" // explode_blastmotes_at
+#include "spl-damage.h" // dazzle_target
 #include "spl-util.h"
 #include "state.h"
 #include "stringutil.h"
 #include "tag-version.h"
 #include "terrain.h"
 #include "rltiles/tiledef-main.h"
+#include "traps.h"
 #include "unwind.h"
+#include "xom.h"
 
 cloud_struct* cloud_at(coord_def pos)
 {
@@ -178,7 +182,7 @@ static const cloud_data clouds[] = {
     // CLOUD_PETRIFY,
     { "calcifying dust",  nullptr,              // terse, verbose name
       WHITE,                                    // colour
-      { TILE_CLOUD_PETRIFY, CTVARY_RANDOM },    // tile
+      { TILE_CLOUD_PETRIFY, CTVARY_DUR },       // tile
       BEAM_PETRIFYING_CLOUD, {},                // beam_effect & damage
       true,                                     // opacity
     },
@@ -318,6 +322,16 @@ static const cloud_data clouds[] = {
       ETC_ELECTRICITY,                                   // colour
       { TILE_CLOUD_ELECTRICITY, CTVARY_RANDOM },        // tile
     },
+    // CLOUD_FAINT_MIASMA,
+    { "faint pestilence", nullptr,                // terse, verbose name
+      DARKGREY,                                      // colour
+      { TILE_CLOUD_FAINT_MIASMA, CTVARY_RANDOM },    // tile
+    },
+    // CLOUD_MAGNETISED_DUST,
+    { "magnetised fragments", nullptr,                  // terse, verbose name
+      ETC_ELECTRICITY,                                 // colour
+      { TILE_CLOUD_MAGNETISED_DUST, CTVARY_RANDOM },   // tile
+    },
 };
 COMPILE_CHECK(ARRAYSZ(clouds) == NUM_CLOUD_TYPES);
 
@@ -424,7 +438,7 @@ static int _spread_cloud(const cloud_struct &cloud)
             continue;
         }
 
-        if (cloud.type == CLOUD_INK && !feat_is_watery(env.grid(*ai)))
+        if (cloud.type == CLOUD_INK && !feat_is_water(env.grid(*ai)))
             continue;
 
         int newdecay = cloud.decay / 2 + 1;
@@ -494,7 +508,7 @@ static void _cloud_interacts_with_terrain(const cloud_struct &cloud)
     {
         const coord_def p(*ai);
         if (in_bounds(p)
-            && feat_is_watery(env.grid(p))
+            && feat_is_water(env.grid(p))
             && !cell_is_solid(p)
             && !cloud_at(p)
             && one_chance_in(14))
@@ -529,7 +543,7 @@ static int _cloud_dissipation_rate(const cloud_struct &cloud)
     }
 
     // Ink cloud shouldn't appear outside of water.
-    if (cloud.type == CLOUD_INK && !feat_is_watery(env.grid(cloud.pos)))
+    if (cloud.type == CLOUD_INK && !feat_is_water(env.grid(cloud.pos)))
         return cloud.decay;
 
     return dissipate;
@@ -548,8 +562,25 @@ static void _dissipate_cloud(cloud_struct& cloud)
         cloud.decay       -= _spread_cloud(cloud);
     }
 
+    // Faint miasma becomes proper miasma after the first tick (regardless of
+    // duration), but will not spawn beneath the player.
+    // XXX: This is currently very player-centric, but the player is also the
+    //      only possible source of this at present.
+    if (cloud.type == CLOUD_FAINT_MIASMA)
+    {
+        if (cloud.decay > 1)
+            cloud.decay = 1;
+        else if (cloud.pos != you.pos())
+        {
+            cloud.decay = random_range(50, 90);
+            cloud.type = CLOUD_MIASMA;
+        }
+        else
+            delete_cloud(cloud.pos);
+    }
+
     // Check for total dissipation and handle accordingly.
-    if (cloud.decay < 1)
+    if (cloud.decay < 1 && cloud.type != CLOUD_FAINT_MIASMA)
         delete_cloud(cloud.pos);
 }
 
@@ -625,28 +656,15 @@ static void _maybe_leave_water(const coord_def pos)
 {
     ASSERT_IN_BOUNDS(pos);
 
-    // Rain clouds can occasionally leave shallow water or deepen it.
-    if (!one_chance_in(5))
+    // Rain clouds can occasionally leave shallow water.
+    if (env.grid(pos) != DNGN_FLOOR || !one_chance_in(5))
         return;
 
-    dungeon_feature_type feat = env.grid(pos);
-
-    if (env.grid(pos) == DNGN_FLOOR)
-        feat = DNGN_SHALLOW_WATER;
-    else if (env.grid(pos) == DNGN_SHALLOW_WATER && you.pos() != pos
-             && one_chance_in(3) && !crawl_state.game_is_sprint())
-    {
-        // Don't drown the player!
-        feat = DNGN_DEEP_WATER;
-    }
-
-    if (env.grid(pos) != feat)
-    {
-        if (you.pos() == pos && you.ground_level())
-            mpr("The rain has left you waist-deep in water!");
-        temp_change_terrain(pos, feat, random_range(500, 1000),
-                            TERRAIN_CHANGE_FLOOD);
-    }
+    if (you.pos() == pos && you.ground_level())
+        mpr("The rain has left you waist-deep in water!");
+    temp_change_terrain(pos, DNGN_SHALLOW_WATER,
+                        random_range(500, 1000),
+                        TERRAIN_CHANGE_FLOOD);
 }
 
 void delete_cloud(coord_def p)
@@ -754,7 +772,7 @@ void place_cloud(cloud_type cl_type, const coord_def& ctarget, int cl_range,
     if (is_sanctuary(ctarget) && !is_harmless_cloud(cl_type))
         return;
 
-    if (cl_type == CLOUD_INK && !feat_is_watery(env.grid(ctarget)))
+    if (cl_type == CLOUD_INK && !feat_is_water(env.grid(ctarget)))
         return;
 
     if (env.level_state & LSTATE_STILL_WINDS
@@ -766,7 +784,7 @@ void place_cloud(cloud_type cl_type, const coord_def& ctarget, int cl_range,
 
     const monster * const mons = monster_at(ctarget);
 
-    // Fedhas protects plants from damaging clouds.
+    // Fedhas et al protect their chosen ones from damaging clouds.
     // XX demonic guardians? This logic mostly doesn't apply because protected
     // monsters are also cloud immune, mostly
     if (god_protects(agent, mons)
@@ -978,7 +996,7 @@ bool actor_cloud_immune(const actor &act, const cloud_struct &cloud)
     const bool player = act.is_player();
 
     if (!player
-        && (god_protects(act.as_monster())
+        && (god_protects(*act.as_monster())
             || testbits(act.as_monster()->flags, MF_DEMONIC_GUARDIAN))
         && (cloud.whose == KC_YOU || cloud.whose == KC_FRIENDLY)
         && (act.as_monster()->friendly() || act.as_monster()->neutral())
@@ -1157,8 +1175,7 @@ static bool _actor_apply_cloud_side_effects(actor *act,
     case CLOUD_CHAOS:
         if (coinflip())
         {
-            // TODO: Not have this in melee_attack
-            melee_attack::chaos_affect_actor(act);
+            chaos_affects_actor(act, cloud.agent());
             return true;
         }
         break;
@@ -1447,10 +1464,6 @@ static bool _mons_avoids_cloud(const monster* mons, const cloud_struct& cloud,
 
     switch (cloud.type)
     {
-    case CLOUD_MIASMA:
-        // Even the dumbest monsters will avoid miasma if they can.
-        return true;
-
     case CLOUD_BLASTMOTES:
         // As with traps, make friendly monsters not walk into blastmotes.
         return mons->attitude == ATT_FRIENDLY
@@ -1458,25 +1471,7 @@ static bool _mons_avoids_cloud(const monster* mons, const cloud_struct& cloud,
             || mons->attitude == ATT_GOOD_NEUTRAL;
 
     case CLOUD_RAIN:
-        // Fiery monsters dislike the rain.
-        if (mons->is_fiery() && extra_careful)
-            return true;
-
-        // We don't care about what's underneath the rain cloud if we can fly.
-        if (mons->airborne())
-            return false;
-
-        // These don't care about deep water.
-        if (monster_habitable_grid(mons, DNGN_DEEP_WATER))
-            return false;
-
-        // This position could become deep water, and they might drown.
-        if (env.grid(cloud.pos) == DNGN_SHALLOW_WATER
-            && mons_intel(*mons) > I_BRAINLESS)
-        {
-            return true;
-        }
-        break;
+        return !mons->is_fiery() || !extra_careful;
 
     default:
     {
@@ -1490,8 +1485,9 @@ static bool _mons_avoids_cloud(const monster* mons, const cloud_struct& cloud,
         const int base_damage = _cloud_damage_calc(dam_info.random,
                                                    max(1, dam_info.random / 9),
                                                    dam_info.base, false);
-        // Add in an arbitrary proxy for poison damage from poison clouds.
-        const int bonus_dam = cloud.type == CLOUD_POISON ? roll_dice(3, 4) : 0;
+        // Add in an arbitrary proxy for poison damage from poison/miasma clouds.
+        const int bonus_dam = cloud.type == CLOUD_POISON ? roll_dice(3, 4)
+                              : cloud.type == CLOUD_MIASMA ? roll_dice(3, 5) : 0;
         const int damage = resist_adjust_damage(mons,
                                                 clouds[cloud.type].beam_effect,
                                                 base_damage + bonus_dam);
@@ -1912,3 +1908,229 @@ bool cloud_is_removed(cloud_type cloud)
         return false;
     }
 }
+
+static bool _is_chaos_polyable(const actor &defender)
+{
+    if (!defender.can_safely_mutate())
+        return false;  // no polymorphing undead
+
+    const monster* mon = defender.as_monster();
+    if (!mon)
+        return true;
+
+    return !mons_is_firewood(*mon) && !mons_invuln_will(*mon);
+}
+
+static bool _is_chaos_slowable(const actor &defender)
+{
+    const monster* mon = defender.as_monster();
+    if (!mon)
+        return true;
+
+    return !mons_is_firewood(*mon);
+}
+
+struct chaos_effect
+{
+    string name;
+    int chance;
+    function<bool(const actor& def)> valid;
+    beam_type flavour;
+    function<bool(actor* victim, actor* source)> misc_effect;
+};
+
+// TODO: Unite this with _chaos_beam_flavour in beam.cc.
+// For now, update that when you update this.
+static const vector<chaos_effect> chaos_effects = {
+    {
+        "clone", 1, [](const actor &d) {
+            return d.is_monster() && mons_clonable(d.as_monster(), true);
+        },
+        BEAM_NONE, [](actor* victim, actor* /*source*/) {
+            ASSERT(victim->is_monster());
+            monster *clone = clone_mons(victim->as_monster());
+            if (!clone)
+                return false;
+
+            const bool obvious_effect = you.can_see(*victim) && you.can_see(*clone);
+
+            if (one_chance_in(3))
+                clone->attitude = coinflip() ? ATT_FRIENDLY : ATT_NEUTRAL;
+
+            // The player shouldn't get new permanent followers from cloning.
+            if (clone->attitude == ATT_FRIENDLY && !clone->is_summoned())
+                clone->mark_summoned(6, true, MON_SUMM_CLONE);
+            else
+                clone->flags |= (MF_NO_REWARD | MF_HARD_RESET);
+
+            // Monsters being cloned is interesting.
+            xom_is_stimulated(clone->friendly() ? 12 : 25);
+            return obvious_effect;
+        },
+    },
+    {
+        "polymorph", 2, _is_chaos_polyable, BEAM_POLYMORPH,
+    },
+    {
+        "rage", 5, [](const actor &victim) {
+            return victim.can_go_berserk() && !victim.clarity();
+        }, BEAM_NONE, [](actor* victim, actor* source) {
+            if (victim->is_monster())
+            {
+                monster* mon = victim->as_monster();
+                ASSERT(mon);
+                if (mon->can_go_frenzy()) {
+                    mon->go_frenzy(source);
+                }
+            }
+            else
+                victim->go_berserk(false);
+
+            return you.can_see(*victim);
+        },
+    },
+    { "hasting", 12, _is_chaos_slowable, BEAM_HASTE },
+    { "mighting", 12, [](const actor &victim) {
+        return !victim.is_monster()
+               || (mons_has_attacks(*(victim.as_monster()))
+                   && !victim.as_monster()->has_ench(ENCH_MIGHT));
+    }, BEAM_MIGHT },
+    { "resistance", 10, [](const actor &victim) {
+        return victim.res_fire() < 3 && victim.res_cold() < 3 &&
+               victim.res_elec() < 3 && victim.res_poison() < 3 &&
+               victim.res_acid() < 3; }, BEAM_RESISTANCE, },
+    { "slowing", 10, _is_chaos_slowable, BEAM_SLOW },
+    { "confusing", 12, [](const actor &victim) {
+        return !(victim.clarity()
+               || (victim.is_monster()
+               && mons_is_conjured(victim.as_monster()->type))); },
+               BEAM_CONFUSION },
+    { "weakening", 10, [](const actor & victim) {
+        return !victim.is_monster()
+               || mons_has_attacks(*(victim.as_monster()));
+    }, BEAM_WEAKNESS, },
+    { "will-halving", 10, [](const actor &victim) {
+       return !victim.is_monster()
+              || mons_invuln_will(*(victim.as_monster()));
+    }, BEAM_VULNERABILITY, },
+    { "blinking", 3, nullptr, BEAM_BLINK },
+    { "corroding", 5, [](const actor &victim) {
+        return victim.res_acid() < 3; },
+        BEAM_NONE, [](actor* victim, actor* /*source*/) {
+           victim->corrode_equipment();
+           return you.can_see(*victim);
+       },
+    },
+    { "vitrifying", 5, nullptr, BEAM_VITRIFY, },
+    { "ensnaring", 3, [](const actor &victim) {
+        return !victim.is_web_immune(); },
+        BEAM_NONE, [](actor* victim, actor* /*source*/) {
+           ensnare(victim);
+           return you.can_see(*victim);
+       },
+    },
+    {
+        "minipara", 3, [](const actor &victim) {
+            return !victim.is_monster()
+                    || !mons_is_firewood(*victim.as_monster());
+        }, BEAM_NONE, [](actor* victim, actor* source) {
+            victim->paralyse(source, 1);
+            return you.can_see(*victim);
+        },
+    },
+    {
+        "sleep", 3, [](const actor &victim) {
+            return victim.can_sleep();
+        }, BEAM_SLEEP,
+    },
+    {
+        "petrify", 3, [](const actor &victim) {
+            return _is_chaos_slowable(victim) && !victim.res_petrify();
+        }, BEAM_PETRIFY,
+    },
+    {
+        "blinding", 5, [](const actor &victim) {
+            return victim.can_be_dazzled();
+        }, BEAM_NONE, [](actor* victim, actor* source) {
+            if (victim->is_player())
+                blind_player(random_range(7, 12), ETC_RANDOM);
+            else
+                dazzle_target(victim, source, 149);
+            return you.can_see(*victim);
+        },
+    },
+};
+
+// Applies a debuff-style random chaos effect to an actor. This may have a source,
+// in the case of chaos weapons, or no source in the case of some clouds.
+//
+// Returns true if the effect was obvious to the player.
+bool chaos_affects_actor(actor* victim, actor* source)
+{
+    ASSERT(victim);
+
+    bool obvious_effect = false;
+
+    vector<pair<const chaos_effect&, int>> weights;
+    for (const chaos_effect &effect : chaos_effects)
+        if (!effect.valid || effect.valid(*victim))
+            weights.push_back({effect, effect.chance});
+
+    const chaos_effect &effect = *random_choose_weighted(weights);
+
+#ifdef NOTE_DEBUG_CHAOS_EFFECTS
+    take_note(Note(NOTE_MESSAGE, 0, 0, "CHAOS effect: " + effect.name), true);
+#endif
+
+    if (effect.misc_effect && effect.misc_effect(victim, source))
+        obvious_effect = true;
+
+    bolt beam;
+    beam.flavour = effect.flavour;
+    beam.no_saving_throw = true;
+
+    if (beam.flavour != BEAM_NONE)
+    {
+        if (victim->is_player() && have_passive(passive_t::no_haste)
+            && beam.flavour == BEAM_HASTE)
+        {
+            simple_god_message(" protects you from inadvertent hurry.");
+            return true;
+        }
+
+        beam.glyph        = 0;
+        beam.range        = 0;
+        beam.colour       = BLACK;
+        beam.effect_known = false;
+
+        beam.thrower =
+            source && source->is_player()                       ? KILL_YOU
+            : source && source->as_monster()->confused_by_you() ? KILL_YOU_CONF
+                                                                : KILL_MON;
+
+        if (beam.thrower == KILL_YOU || (source && source->as_monster()->friendly()))
+            beam.attitude = ATT_FRIENDLY;
+
+        beam.source_id = source ? source->mid : MID_NOBODY;
+
+        beam.source = victim->pos();
+        beam.target = victim->pos();
+        beam.ench_power = 100;
+
+        const bool you_could_see = you.can_see(*victim);
+        beam.fire();
+
+        if (you_could_see)
+        {
+            obvious_effect = beam.obvious_effect;
+            if (!victim->wont_attack() &&
+                (beam.flavour == BEAM_HASTE || beam.flavour == BEAM_MIGHT))
+            {
+                xom_is_stimulated(12);
+            }
+        }
+
+    }
+
+    return obvious_effect;
+}
