@@ -27,8 +27,10 @@ enum conduct_type
     DID_CAUSE_GLOWING,                    // Zin
     DID_UNCLEAN,                          // Zin (used unclean weapon/magic)
     DID_CHAOS,                            // Zin (used chaotic weapon/magic)
+#if TAG_MAJOR_VERSION == 34
     DID_DESECRATE_ORCISH_REMAINS,         // Beogh
     DID_KILL_SLIME,                       // Jiyva
+#endif
     DID_HASTY,                            // Cheibriados
     DID_ATTACK_IN_SANCTUARY,              // Zin
     DID_KILL_NONLIVING,
