@@ -76,6 +76,7 @@ static const char *daction_names[] =
     "ancestor vanishes",
     "upgrade ancestor",
     "remove Ignis altars",
+    "cleanup Beogh vengeance markers",
 };
 #endif
 
@@ -101,11 +102,10 @@ bool mons_matches_daction(const monster* mon, daction_type act)
     // Not a stored counter:
     case DACT_PIKEL_MINIONS:
         return mon->type == MONS_LEMURE
-               && testbits(mon->flags, MF_BAND_MEMBER)
                && mon->props.exists(PIKEL_BAND_KEY);
 
     case DACT_OLD_CHARMD_SOULS_POOF:
-        return mons_bound_soul(*mon);
+        return mon->type == MONS_BOUND_SOUL;
 
     case DACT_SLIME_NEW_ATTEMPT:
         return mons_is_slime(*mon);
@@ -131,6 +131,11 @@ bool mons_matches_daction(const monster* mon, daction_type act)
     case DACT_JIYVA_DEAD:
         return mon->type == MONS_DISSOLUTION;
 
+    case DACT_BEOGH_VENGEANCE_CLEANUP:
+        return mon->has_ench(ENCH_VENGEANCE_TARGET)
+               && mon->get_ench(ENCH_VENGEANCE_TARGET).degree
+                  <= you.props[BEOGH_VENGEANCE_NUM_KEY].get_int();
+
     default:
         return false;
     }
@@ -195,6 +200,11 @@ void apply_daction_to_mons(monster* mon, daction_type act, bool local,
             break;
 
         case DACT_ALLY_HEPLIAKLQANA:
+            // Skip this if we have since regained enough piety to get our
+            // ancestor back.
+            if (you_worship(GOD_HEPLIAKLQANA) && piety_rank() >= 1)
+                break;
+
             simple_monster_message(*mon, " returns to the mists of memory.");
             monster_die(*mon, KILL_DISMISSED, NON_MONSTER);
             break;
@@ -205,6 +215,10 @@ void apply_daction_to_mons(monster* mon, daction_type act, bool local,
             break;
 
         case DACT_OLD_CHARMD_SOULS_POOF:
+            // Skip if this is our CURRENT bound soul (ie: in our companion list)
+            if (companion_list.count(mon->mid))
+                break;
+
             simple_monster_message(*mon, " is freed.");
             // The monster disappears.
             monster_die(*mon, KILL_DISMISSED, NON_MONSTER);
@@ -253,6 +267,11 @@ void apply_daction_to_mons(monster* mon, daction_type act, bool local,
             mon->props[CUSTOM_SPELLS_KEY] = true;
             break;
 
+        case DACT_BEOGH_VENGEANCE_CLEANUP:
+            mon->del_ench(ENCH_VENGEANCE_TARGET);
+            mon->patrol_point.reset();
+            break;
+
         // The other dactions do not affect monsters directly.
         default:
             break;
@@ -283,6 +302,7 @@ static void _apply_daction(daction_type act)
     case DACT_KIRKE_HOGS:
     case DACT_BRIBE_TIMEOUT:
     case DACT_SET_BRIBES:
+    case DACT_BEOGH_VENGEANCE_CLEANUP:
         for (monster_iterator mi; mi; ++mi)
         {
             if (mons_matches_daction(*mi, act))
