@@ -56,7 +56,10 @@
 #include "religion.h"
 #include "shout.h"
 #include "sound.h"
+#include "spl-damage.h"
 #include "spl-selfench.h"
+#include "spl-summoning.h"
+#include "spl-transloc.h" // attract_monster
 #include "spl-util.h"
 #include "stairs.h"
 #include "state.h"
@@ -75,6 +78,8 @@ static const char *_activity_interrupt_name(activity_interrupt ai);
 
 static string _eq_category(const item_def &equip)
 {
+    if (is_weapon(equip))
+        return "weapon";
     return equip.base_type == OBJ_JEWELLERY ? "amulet" : "armour";
 }
 
@@ -153,6 +158,13 @@ bool MacroDelay::try_interrupt(bool /*force*/)
     // to the Lua function, it can't do damage.
 }
 
+const char* EquipOnDelay::get_verb()
+{
+    if (is_weapon(equip))
+        return "attuning to"; // coglin
+    return "putting on";
+}
+
 bool EquipOnDelay::try_interrupt(bool force)
 {
     bool interrupt = false;
@@ -172,12 +184,19 @@ bool EquipOnDelay::try_interrupt(bool force)
 
     if (interrupt)
     {
-        mprf("You stop putting on your %s.", _eq_category(equip).c_str());
+        mprf("You stop %s your %s.", get_verb(), _eq_category(equip).c_str());
         return true;
     }
     return false;
 }
 
+const char* EquipOffDelay::get_verb()
+{
+    if (is_weapon(equip))
+        return "parting from";
+    return "removing";
+}
+
 bool EquipOffDelay::try_interrupt(bool force)
 {
     bool interrupt = false;
@@ -186,8 +205,13 @@ bool EquipOffDelay::try_interrupt(bool force)
         interrupt = true;
     else if (duration > 1 && !was_prompted)
     {
+        const bool is_armour = equip.base_type == OBJ_ARMOUR
+                               // Shields and orbs aren't clothes.
+                               && get_armour_slot(equip) != EQ_OFFHAND;
+        const char* verb = is_armour ? "disrobing" : "removing your equipment";
+        const string prompt = make_stringf("Keep %s?", verb);
         if (!crawl_state.disables[DIS_CONFIRMATIONS]
-            && !yesno("Keep disrobing?", false, 0, false))
+            && !yesno(prompt.c_str(), false, 0, false))
         {
             interrupt = true;
         }
@@ -197,7 +221,7 @@ bool EquipOffDelay::try_interrupt(bool force)
 
     if (interrupt)
     {
-        mprf("You stop removing your %s.", _eq_category(equip).c_str());
+        mprf("You stop %s your %s.", get_verb(), _eq_category(equip).c_str());
         return true;
     }
     return false;
@@ -221,6 +245,8 @@ bool PasswallDelay::try_interrupt(bool /*force*/)
     if (interrupt_block::blocked())
         return false;
     mpr("Your meditation is interrupted.");
+    you.props.erase(PASSWALL_ARMOUR_KEY);
+    you.redraw_armour_class = true;
     return true;
 }
 
@@ -303,6 +329,31 @@ bool TransformDelay::try_interrupt(bool force)
     return true;
 }
 
+bool ImbueDelay::try_interrupt(bool force)
+{
+    bool interrupt = false;
+
+    if (force)
+        interrupt = true;
+    else if (duration > 1 && !was_prompted)
+    {
+        if (!crawl_state.disables[DIS_CONFIRMATIONS]
+            && !yesno("Keep imbuing your servitor?", false, 0, false))
+        {
+            interrupt = true;
+        }
+        else
+            was_prompted = true;
+    }
+
+    if (interrupt)
+    {
+        mpr("You stop imbuing your servitor.");
+        return true;
+    }
+    return false;
+}
+
 void stop_delay(bool stop_relocations, bool force)
 {
     if (you.delay_queue.empty())
@@ -415,13 +466,15 @@ void clear_macro_process_key_delay()
 
 void EquipOnDelay::start()
 {
-    mprf(MSGCH_MULTITURN_ACTION, "You start putting on your %s.",
+    mprf(MSGCH_MULTITURN_ACTION, "You start %s your %s.",
+         get_verb(),
          _eq_category(equip).c_str());
 }
 
 void EquipOffDelay::start()
 {
-    mprf(MSGCH_MULTITURN_ACTION, "You start removing your %s.",
+    mprf(MSGCH_MULTITURN_ACTION, "You start %s your %s.",
+         get_verb(),
          _eq_category(equip).c_str());
 }
 
@@ -456,6 +509,13 @@ void RevivifyDelay::start()
     mprf(MSGCH_MULTITURN_ACTION, "You begin the revivification ritual.");
 }
 
+void ImbueDelay::start()
+{
+    mprf(MSGCH_MULTITURN_ACTION, "You begin to imbue your servitor with "
+         "knowledge of %s.",
+         spell_title(spell));
+}
+
 void TransformDelay::start()
 {
     if (form == transformation::none)
@@ -501,10 +561,15 @@ void BaseRunDelay::handle()
 
     command_type cmd = CMD_NO_CMD;
 
-    if ((want_move() && you.confused())
-        || !(unsafe_once && first_move) && !i_feel_safe(true, want_move())
-        || you.running.is_rest()
-           && (!can_rest_here(true) || regeneration_is_inhibited()))
+    if (want_move() && you.confused())
+    {
+        mprf("You're confused, stopping %s.",
+             you.running.runmode_name().c_str());
+        stop_running();
+    }
+    else if (!(unsafe_once && first_move) && !i_feel_safe(true, want_move())
+            || you.running.is_rest()
+               && (!can_rest_here(true) || regeneration_is_inhibited()))
     {
         stop_running();
     }
@@ -714,18 +779,28 @@ void EquipOnDelay::finish()
 {
     const unsigned int old_talents = your_talents(false).size();
     const bool is_amulet = equip.base_type == OBJ_JEWELLERY;
-    const equipment_type eq_slot = is_amulet ? EQ_AMULET :
-                                               get_armour_slot(equip);
+    const equipment_type eq_slot = is_amulet ?      EQ_AMULET :
+                                   primary_weapon ? EQ_WEAPON :
+                                                    get_armour_slot(equip);
 
 #ifdef USE_SOUND
-    if (!is_amulet)
+    if (is_weapon(equip))
+        parse_sound(WIELD_WEAPON_SOUND);
+    else if (!is_amulet)
         parse_sound(EQUIP_ARMOUR_SOUND);
 #endif
-    mprf("You finish putting on %s.", equip.name(DESC_YOUR).c_str());
+    mprf("You finish %s %s.", get_verb(), equip.name(DESC_YOUR).c_str());
 
     equip_item(eq_slot, equip.link);
 
     check_item_hint(equip, old_talents);
+
+    if (is_weapon(equip))
+    {
+        maybe_name_weapon(equip);
+        you.wield_change  = true;
+        quiver::on_weapon_changed();
+    }
 }
 
 bool EquipOffDelay::invalidated()
@@ -736,16 +811,29 @@ bool EquipOffDelay::invalidated()
 void EquipOffDelay::finish()
 {
     const bool is_amu = equip.base_type == OBJ_JEWELLERY;
-    const equipment_type slot = is_amu ? EQ_AMULET : get_armour_slot(equip);
+    const equipment_type slot = is_amu ?         EQ_AMULET :
+                                primary_weapon ? EQ_WEAPON :
+                                                 get_armour_slot(equip);
     ASSERTM(you.equip[slot] == equip.link,
         "Mismatched link in EquipOffDelay::finish: slot is %d with link %d, link is %d",
         slot, you.equip[slot], equip.link);
 
 #ifdef USE_SOUND
-    parse_sound(is_amu ? REMOVE_JEWELLERY_SOUND : DEQUIP_ARMOUR_SOUND);
+    parse_sound(is_amu ?           REMOVE_JEWELLERY_SOUND :
+                is_weapon(equip) ? WIELD_NOTHING_SOUND :
+                                   DEQUIP_ARMOUR_SOUND);
 #endif
-    mprf("You finish taking off %s.", equip.name(DESC_YOUR).c_str());
+    if (is_weapon(equip))
+        say_farewell_to_weapon(equip);
+
+    mprf("You finish %s %s.", get_verb(), equip.name(DESC_YOUR).c_str());
     unequip_item(slot);
+
+    if (is_weapon(equip))
+    {
+        you.wield_change  = true;
+        quiver::on_weapon_changed();
+    }
 }
 
 void MemoriseDelay::finish()
@@ -886,6 +974,12 @@ void RevivifyDelay::finish()
     vampire_update_transformations();
 }
 
+void ImbueDelay::finish()
+{
+    mpr("You finish imbuing your servitor.");
+    you.props[SERVITOR_SPELL_KEY] = spell;
+}
+
 bool TransformDelay::invalidated()
 {
     // Got /poly'd while mid-transform?
@@ -1182,7 +1276,9 @@ static inline bool _monster_warning(activity_interrupt ai,
             god_warning += "shapeshifter.";
         }
 
+        // Refresh our monster info cache, so xv shows the ID'd items.
         monster_info mi(mon);
+        env.map_knowledge(mon->pos()).set_monster(mi);
 
         const string mweap = get_monster_equipment_desc(mi, DESC_IDENTIFIED,
                                                         DESC_NONE);
@@ -1274,7 +1370,7 @@ bool interrupt_activity(activity_interrupt ai,
         || ai == activity_interrupt::monster_attacks)
     {
         const monster* mon = at.mons_data;
-        if (mon && !mon->visible_to(&you) && !mon->submerged())
+        if (mon && !mon->visible_to(&you))
             autotoggle_autopickup(true);
     }
 
@@ -1363,7 +1459,7 @@ static const char *activity_interrupt_names[] =
 {
     "force", "keypress", "full_hp", "full_mp", "ancestor_hp", "message",
     "hp_loss", "stat", "monster", "monster_attack", "teleport", "hit_monster",
-    "sense_monster", MIMIC_KEY
+    "sense_monster", MIMIC_KEY, "ally_attacked", "abyss_exit_spawned"
 };
 
 static const char *_activity_interrupt_name(activity_interrupt ai)
