@@ -80,7 +80,7 @@ int god_favour_rank(god_type which_god)
 {
     if (which_god == GOD_GOZAG)
         return _gold_level();
-    else if (which_god == GOD_USKAYAW || which_god == GOD_YREDELEMNUL)
+    else if (which_god == GOD_USKAYAW)
         return _invocations_level();
     else
         return _piety_level(you.piety);
@@ -143,13 +143,13 @@ static const char *divine_title[][8] =
     {"Honourless",         "Acolyte",               "Righteous",                "Unflinching",
         "Holy Warrior",       "Exorcist",              "Demon Slayer",             "Bringer of Light"},
 
-    // Kikubaaqudgha -- scholarly death.
-    {"Tormented",          "Purveyor of Pain",      "Scholar of Death",         "Merchant of Misery",
-        "Artisan of Death",   "Dealer of Despair",     "Black Sun",                "Lord of Darkness"},
+    // Kikubaaqudgha -- death scholar theme.
+    {"Tormented",          "Purveyor of Pain",       "Pupil of Sorrows",        "Merchant of Misery",
+     "Scholar of Souls",   "Artisan of Death",       "Demagogue of Despair",    "Lord of Darkness"},
 
-    // Yredelemnul -- zombie death.
-    {"Traitor",            "Tainted",                "Torchbearer",             "Fey @Genus@",
-        "Black Crusader",     "Sculptor of Flesh",     "Harbinger of Death",       "Grim Reaper"},
+    // Yredelemnul -- ferverent death knight theme.
+    {"Traitor",            "Torchbearer",            "Despoiler",               "Black Crusader",
+     "Fallen @Genus@",     "Harbinger of Doom",      "Inexorable Tide",         "Bringer of Blasphemy"},
 
     // Xom.
     {"Toy",                "Toy",                   "Toy",                      "Toy",
@@ -167,7 +167,7 @@ static const char *divine_title[][8] =
     {"Orderly",            "Spawn of Chaos",        "Disciple of Destruction",  "Fanfare of Bloodshed",
         "Fiendish",           "Demolition @Genus@",    "Pandemonic",               "Champion of Chaos"},
 
-    // Sif Muna -- scholarly theme.
+    // Sif Muna -- generalist scholarly theme.
     {"Ignorant",           "Disciple",              "Student",                  "Adept",
         "Scribe",             "Scholar",               "Sage",                     "Genius of the Arcane"},
 
@@ -188,8 +188,8 @@ static const char *divine_title[][8] =
         "Agent of Entropy",   "Schismatic",            "Envoy of Void",            "Corrupter of Planes"},
 
     // Beogh -- messiah theme.
-    {"Apostate",           "Messenger",             "Proselytiser",             "Priest",
-        "Missionary",         "Evangelist",            "Apostle",                  "Messiah"},
+    {"Apostate",           "Convert",               "Proselytiser",             "Priest",
+        "Missionary",         "Evangelist",            "Unifier",                  "Messiah"},
 
     // Jiyva -- slime and jelly theme.
     {"Scum",               "Squelcher",             "Ooze",                     "Jelly",
@@ -208,8 +208,8 @@ static const char *divine_title[][8] =
         "Oracle",            "Illuminatus",            "Prince of Secrets",        "Omniscient"},
 
     // Dithmenos -- darkness theme
-    {"Ember",              "Gloomy",                "Darkened",                 "Extinguished",
-        "Caliginous",         "Umbral",                "Hand of Shadow",           "Eternal Night"},
+    {"Conspicuous",         "Nocturnal",            "Bump in the Night",        "Thespian",
+        "Tenebrous",          "Puppetmaster",          "@Walking@ Midnight",       "Who Hides the Stars"},
 
     // Gozag -- entrepreneur theme
     {"Profligate",         "Pauper",                "Entrepreneur",             "Capitalist",
@@ -252,7 +252,7 @@ string god_title(god_type which_god, species_type which_species, int piety)
     string title;
     if (player_under_penance(which_god))
         title = divine_title[which_god][0];
-    else if (which_god == GOD_USKAYAW || which_god == GOD_YREDELEMNUL)
+    else if (which_god == GOD_USKAYAW)
         title = divine_title[which_god][_invocations_level()];
     else if (which_god == GOD_GOZAG)
         title = divine_title[which_god][_gold_level()];
@@ -266,6 +266,7 @@ string god_title(god_type which_god, species_type which_species, int piety)
         { "Walking", species::walking_title(which_species) + "ing" },
         { "Walker", species::walking_title(which_species) + "er" },
         { "Child", species::child_name(which_species) },
+        { "Orc", species::orc_name(which_species) },
     };
 
     return replace_keys(title, replacements);
@@ -533,61 +534,15 @@ static formatted_string _beogh_extra_description()
 {
     formatted_string desc;
 
-    _add_par(desc, "Named Followers:");
+    for (int i = 1; i <= get_num_apostles(); ++i)
+        _add_par(desc, apostle_short_description(i));
 
-    vector<monster*> followers;
-
-    for (monster_iterator mi; mi; ++mi)
-        if (is_orcish_follower(**mi))
-            followers.push_back(*mi);
-    for (auto &entry : companion_list)
-        // if not elsewhere, follower already seen by monster_iterator
-        if (companion_is_elsewhere(entry.second.mons.mons.mid, true))
-            followers.push_back(&entry.second.mons.mons);
-
-    sort(followers.begin(), followers.end(),
-        [] (monster* a, monster* b) { return a->experience > b->experience;});
-
-    bool has_named_followers = false;
-    for (auto mons : followers)
+    if (you.duration[DUR_BEOGH_CAN_RECRUIT])
     {
-        if (!mons->is_named())
-            continue;
-        has_named_followers = true;
-
-        desc += mons->full_name(DESC_PLAIN);
-        if (companion_is_elsewhere(mons->mid))
-        {
-            desc += formatted_string::parse_string(
-                            " (<blue>on another level</blue>)");
-        }
-        else if (given_gift(mons))
-        {
-            mon_inv_type slot =
-                mons->props.exists(BEOGH_SH_GIFT_KEY) ? MSLOT_SHIELD :
-                mons->props.exists(BEOGH_ARM_GIFT_KEY) ? MSLOT_ARMOUR :
-                mons->props.exists(BEOGH_RANGE_WPN_GIFT_KEY) ? MSLOT_ALT_WEAPON :
-                MSLOT_WEAPON;
-
-            // An orc can still lose its gift, e.g. by being turned into a
-            // shapeshifter via a chaos cloud. TODO: should the gift prop be
-            // deleted at that point?
-            if (mons->inv[slot] != NON_ITEM)
-            {
-                desc.cprintf(" (");
-
-                item_def &gift = env.item[mons->inv[slot]];
-                desc += formatted_string::parse_string(
-                                    menu_colour_item_name(gift,DESC_PLAIN));
-                desc.cprintf(")");
-            }
-        }
-        desc.cprintf("\n");
+        _add_par(desc, "-----------------------------------\n");
+        _add_par(desc, apostle_short_description(0));
     }
 
-    if (!has_named_followers)
-        _add_par(desc, "None");
-
     return desc;
 }
 
@@ -809,6 +764,21 @@ static formatted_string _describe_god_powers(god_type which_god)
 
     switch (which_god)
     {
+    case GOD_BEOGH:
+    {
+        have_any = true;
+        if (have_passive(passive_t::convert_orcs))
+            desc.textcolour(god_colour(which_god));
+        else
+            desc.textcolour(DARKGREY);
+
+        if (piety >= piety_breakpoint(5))
+            desc.cprintf("Orcs frequently recognise you as Beogh's chosen one.\n");
+        else
+            desc.cprintf("Orcs sometimes recognise you as one of their own.\n");
+    }
+    break;
+
     case GOD_ZIN:
     {
         have_any = true;
@@ -836,42 +806,40 @@ static formatted_string _describe_god_powers(god_type which_god)
         have_any = true;
         desc.cprintf("%s prevents you from stabbing unaware foes.\n",
                 uppercase_first(god_name(which_god)).c_str());
-        if (piety < piety_breakpoint(1))
-            desc.textcolour(DARKGREY);
-        else
-            desc.textcolour(god_colour(which_god));
-        const char *how =
-            (piety >= piety_breakpoint(5)) ? "completely" :
-            (piety >= piety_breakpoint(3)) ? "mostly" :
-                                             "partially";
-
-        desc.cprintf("%s %s shields you from negative energy.\n",
-                uppercase_first(god_name(which_god)).c_str(), how);
 
         const int halo_size = you_worship(which_god) ? you.halo_radius() : -1;
         if (halo_size < 0)
             desc.textcolour(DARKGREY);
         else
             desc.textcolour(god_colour(which_god));
-        desc.cprintf("You radiate a%s righteous aura, and others within it are "
+        desc.cprintf("You radiate a%s righteous aura, and foes within it are "
                 "easier to hit.\n",
                 halo_size > 5 ? " large" :
                 halo_size > 3 ? "" :
                                 " small");
+
+        if (piety >= piety_breakpoint(1))
+            desc.textcolour(god_colour(which_god));
+        else
+            desc.textcolour(DARKGREY);
+        const char *how =
+            (piety >= piety_breakpoint(5)) ? "completely" :
+            (piety >= piety_breakpoint(3)) ? "mostly" :
+                                             "partially";
+        desc.cprintf("%s %s shields you from negative energy.\n",
+                uppercase_first(god_name(which_god)).c_str(), how);
         break;
     }
 
-    case GOD_FEDHAS:
-        have_any = true;
-        desc.cprintf("You can walk through plants and fire through allied plants.\n");
-        break;
-
     case GOD_JIYVA:
         have_any = true;
-        if (!have_passive(passive_t::jelly_regen))
-            desc.textcolour(DARKGREY);
-        else
+        desc.cprintf("Jellies are peaceful and will consume items off the floor.\n");
+        desc.cprintf("Jiyva prevents you from harming jellies.\n");
+
+        if (have_passive(passive_t::jelly_regen))
             desc.textcolour(god_colour(which_god));
+        else
+            desc.textcolour(DARKGREY);
         desc.cprintf("Your health and magic regeneration is %saccelerated.\n",
                      piety >= piety_breakpoint(5) ? "very greatly " :
                      piety >= piety_breakpoint(3) ? "greatly " :
@@ -910,31 +878,15 @@ static formatted_string _describe_god_powers(god_type which_god)
         }
         break;
 
-    case GOD_DITHMENOS:
-    {
-        have_any = true;
-        const int umbra_size = you_worship(which_god) ? you.umbra_radius() : -1;
-        if (umbra_size < 0)
-            desc.textcolour(DARKGREY);
-        else
-            desc.textcolour(god_colour(which_god));
-        desc.cprintf("You radiate a%s aura of darkness, enhancing your stealth "
-                "and reducing the accuracy of your foes.\n",
-                umbra_size > 5 ? " large" :
-                umbra_size > 3 ? "n" :
-                                 " small");
-        break;
-    }
-
-    case GOD_GOZAG:
-        have_any = true;
-        desc.cprintf("%s turns your defeated foes' bodies to gold.\n",
-                uppercase_first(god_name(which_god)).c_str());
-        desc.cprintf("Your enemies may become distracted by gold.\n");
+    case GOD_YREDELEMNUL:
+        // TODO: Vary the text depending on the size of the umbra.
+        desc.cprintf("You are surrounded by an umbra.\n"
+                     "Foes that die within your umbra may be raised as undead servants.\n");
         break;
 
     case GOD_HEPLIAKLQANA:
-        // XXX: move this logic back into the usual religion.cc god_powers block?
+        // Frailty occurs even under penance post-abandonment, so we can't put
+        // this in the usual god_powers block.
         have_any = true;
     {
         const auto textcol = have_passive(passive_t::frail) ? god_colour(which_god) : DARKGREY;
@@ -943,28 +895,9 @@ static formatted_string _describe_god_powers(god_type which_god)
         // Feature request: not this.
         desc.textcolour(textcol);
         desc.cprintf("Your life essence is reduced. (-10%% HP)\n");
-        desc.textcolour(textcol);
-        desc.cprintf("Your ancestor manifests to aid you.\n");
     }
         break;
 
-    case GOD_LUGONU:
-        have_any = true;
-        desc.cprintf("You are protected from the effects of unwielding distortion weapons.\n");
-        break;
-
-    case GOD_OKAWARU:
-        have_any = true;
-        desc.cprintf("%s requires that you fight alone, and prevents you from "
-                     "gaining allies.\n",
-                uppercase_first(god_name(which_god)).c_str());
-        break;
-
-    case GOD_IGNIS:
-        have_any = true;
-        desc.cprintf("You are resistant to fire.\n");
-        break;
-
     default:
         break;
     }
@@ -978,6 +911,13 @@ static formatted_string _describe_god_powers(god_type which_god)
         {
             continue;
         }
+        // Skip over Makhleb's brand options after the first one, since
+        // only the first one has an associated god ability.
+        if (power.abil == ABIL_MAKHLEB_BRAND_SELF_2
+            || power.abil == ABIL_MAKHLEB_BRAND_SELF_3)
+        {
+            continue;
+        }
         have_any = true;
 
         if (you_worship(which_god)
@@ -992,7 +932,38 @@ static formatted_string _describe_god_powers(god_type which_god)
         else
             desc.textcolour(DARKGREY);
 
+        // XXX: I don't like this, but there's no other obvious way to
+        //      slot the destruction upgrade mutations in at the right piety
+        //      point in the list for them.
+        if (which_god == GOD_MAKHLEB && power.rank == 4)
+        {
+            desc.textcolour(god_colour(which_god));
+            if (you.has_mutation(MUT_MAKHLEB_DESTRUCTION_GEH))
+                desc.cprintf("Your Destruction is augmented by the power of Gehenna.\n");
+            else if (you.has_mutation(MUT_MAKHLEB_DESTRUCTION_COC))
+                desc.cprintf("Your Destruction is augmented by the power of Cocytus.\n");
+            else if (you.has_mutation(MUT_MAKHLEB_DESTRUCTION_TAR))
+                desc.cprintf("Your Destruction is augmented by the power of Tartarus.\n");
+            else if (you.has_mutation(MUT_MAKHLEB_DESTRUCTION_DIS))
+                desc.cprintf("Your Destruction is augmented by the power of Dis.\n");
+            else
+            {
+                desc.textcolour(DARKGREY);
+                desc.cprintf("Your Destruction will be augmented by one of the Four Hells.\n");
+            }
+
+            continue;
+        }
+
         string buf = power.general;
+
+        // Skip listing powers with no description (they are intended to be hidden)
+        if (buf.length() == 0)
+        {
+            have_any = false;
+            continue;
+        }
+
         if (!isupper(buf[0])) // Complete sentence given?
             buf = "You can " + buf + ".";
         const int desc_len = buf.size();
@@ -1008,6 +979,17 @@ static formatted_string _describe_god_powers(god_type which_god)
     if (!have_any)
         desc.cprintf("None.\n");
 
+    // Show Jiyva's opening of the Slime Pits at the bottom of the list
+    // We want this to stay green permanently once the player hits 6*
+    if (which_god == GOD_JIYVA)
+    {
+        if (you.one_time_ability_used[which_god])
+            desc.textcolour(god_colour(which_god));
+        else
+            desc.textcolour(DARKGREY);
+        desc.cprintf("Jiyva will unlock the Slime Pits vaults.\n");
+    }
+
     return desc;
 }
 
@@ -1212,6 +1194,8 @@ void describe_god(god_type which_god)
 #endif
             return true;
         }
+        if (desc_sw->current_widget()->on_event(ev))
+            return true;
         return done = ui::key_exits_popup(key, false);
     });
 
