@@ -12,6 +12,7 @@
 #include "describe.h"
 #include "english.h"
 #include "externs.h"
+#include "god-abil.h"
 #include "invent.h"
 #include "libutil.h"
 #include "menu.h"
@@ -117,6 +118,12 @@ static string _ability_type_vulnerabilities(mon_spell_slot_flag type)
  */
 static string _booktype_header(mon_spell_slot_flag type, bool pronoun_plural)
 {
+    if (type == MON_SPELL_EVOKE)
+    {
+        return make_stringf("%s the following wand spells:",
+                            conjugate_verb("possess", pronoun_plural).c_str());
+    }
+
     const string vulnerabilities = _ability_type_vulnerabilities(type);
 
     if (type == MON_SPELL_WIZARD)
@@ -153,15 +160,11 @@ static void _monster_spellbooks(const monster_info &mi,
     if (book_slots.empty())
         return;
 
-    const string set_name = type == MON_SPELL_WIZARD ? "Book" : "Set";
-
     spellbook_contents output_book;
 
-    output_book.label +=
-        "\n" +
-        uppercase_first(mi.pronoun(PRONOUN_SUBJECTIVE)) +
-        " " +
-        _booktype_header(type, mi.pronoun_plurality());
+    output_book.label += make_stringf("\n%s %s",
+        uppercase_first(mi.pronoun(PRONOUN_SUBJECTIVE)).c_str(),
+        _booktype_header(type, mi.pronoun_plurality()).c_str());
 
     // Does the monster have a spell that allows them to cast Abjuration?
     bool mons_abjure = false;
@@ -189,6 +192,29 @@ static void _monster_spellbooks(const monster_info &mi,
     all_books.emplace_back(output_book);
 }
 
+static void _monster_wand_spellbook(const monster_info &mi,
+                                spellset &all_books)
+{
+    if (mi.itemuse() < MONUSE_STARTING_EQUIPMENT)
+        return;
+
+    const item_def* wand = mi.inv[MSLOT_WAND].get();
+    if (!wand)
+        return;
+
+    spellbook_contents book;
+
+    book.label += make_stringf("\n%s %s",
+        uppercase_first(mi.pronoun(PRONOUN_SUBJECTIVE)).c_str(),
+        _booktype_header(MON_SPELL_EVOKE, mi.pronoun_plurality()).c_str());
+
+    const wand_type wandtyp = static_cast<wand_type>(wand->sub_type);
+    ASSERT(wandtyp < NUM_WANDS);
+    book.spells.emplace_back(spell_in_wand(wandtyp));
+
+    all_books.emplace_back(book);
+}
+
 /**
  * Return a spellset containing the spells potentially given by the given
  * monster information.
@@ -216,6 +242,8 @@ spellset monster_spellset(const monster_info &mi)
     for (auto book_flag : book_flags)
         _monster_spellbooks(mi, book_flag, books);
 
+    _monster_wand_spellbook(mi, books);
+
     ASSERT(books.size());
     return books;
 }
@@ -310,19 +338,6 @@ static string _spell_schools(spell_type spell)
     return schools;
 }
 
-/**
- * Should spells from the given source be listed in two columns instead of
- * one?
- *
- * @param source_item   The source of the spells; a book, or nullptr in the
- *                      case of monster spellbooks.
- * @return              source_item == nullptr
- */
-static bool _list_spells_doublecolumn(const item_def* const source_item)
-{
-    return !source_item;
-}
-
 /**
  * Produce a mapping from characters (used as indices) to spell types in
  * the given spellset.
@@ -334,24 +349,13 @@ static bool _list_spells_doublecolumn(const item_def* const source_item)
  *                      either in original order or column-major order, the
  *                      latter in the case of a double-column layout.
  */
-vector<pair<spell_type,char>> map_chars_to_spells(const spellset &spells,
-                                       const item_def* const source_item)
+vector<pair<spell_type,char>> map_chars_to_spells(const spellset &spells)
 {
     char next_ch = 'a';
     const vector<spell_type> flat_spells = _spellset_contents(spells);
     vector<pair<spell_type,char>> ret;
-    if (!_list_spells_doublecolumn(source_item))
-    {
-        for (auto spell : flat_spells)
-            ret.emplace_back(pair<spell_type,char>(spell, next_ch++));
-    }
-    else
-    {
-        for (size_t i = 0; i < flat_spells.size(); i += 2)
-            ret.emplace_back(pair<spell_type,char>(flat_spells[i], next_ch++));
-        for (size_t i = 1; i < flat_spells.size(); i += 2)
-            ret.emplace_back(pair<spell_type,char>(flat_spells[i], next_ch++));
-    }
+    for (auto spell : flat_spells)
+        ret.emplace_back(pair<spell_type,char>(spell, next_ch++));
     return ret;
 }
 
@@ -359,7 +363,7 @@ static string _range_string(const spell_type &spell, const monster_info *mon_own
 {
     auto flags = get_spell_flags(spell);
     int pow = mons_power_for_hd(spell, hd);
-    int range = spell_range(spell, pow, false);
+    int range = spell_range(spell, pow, mon_owner && mon_owner->is(MB_PLAYER_SERVITOR));
     const bool has_range = mon_owner
                         && range > 0
                         && !testbits(flags, spflag::selfench);
@@ -382,6 +386,8 @@ static dice_def _spell_damage(spell_type spell, int hd)
     {
         case SPELL_FREEZE:
             return freeze_damage(pow, false);
+        case SPELL_SCORCH:
+            return scorch_damage(pow, false);
         case SPELL_WATERSTRIKE:
             return waterstrike_damage(hd);
         case SPELL_IOOD:
@@ -404,6 +410,13 @@ static dice_def _spell_damage(spell_type spell, int hd)
             return resonance_strike_base_damage(hd);
         case SPELL_POLAR_VORTEX:
             return polar_vortex_dice(pow, false);
+        case SPELL_ELECTROLUNGE:
+            return electrolunge_damage(pow);
+        case SPELL_FULMINANT_PRISM:
+        case SPELL_SHADOW_PRISM:
+            return prism_damage(prism_hd(pow, false), true);
+        case SPELL_HELLFIRE_MORTAR:
+            return hellfire_mortar_damage(pow);
 
         // This is the per-turn *sticky flame* damage against the player.
         // The spell has no impact damage and otherwise uses different numbers
@@ -426,6 +439,8 @@ static int _spell_hd(spell_type spell, const monster_info &mon_owner)
 {
     if (spell == SPELL_SEARING_BREATH && mon_owner.type == MONS_XTAHUA)
         return mon_owner.hd * 3 / 2;
+    if (spell == SPELL_COLD_BREATH && mons_is_draconian(mon_owner.type))
+        return mon_owner.hd * 5 / 6;
     if (mons_spell_is_spell(spell))
         return mon_owner.spell_hd();
     return mon_owner.hd;
@@ -442,8 +457,11 @@ static colour_t _spell_colour(spell_type spell)
             return LIGHTBLUE;
         case SPELL_IOOD:
             return LIGHTMAGENTA;
+        case SPELL_SCORCH:
         case SPELL_ERUPTION:
             return RED;
+        case SPELL_ELECTROLUNGE:
+            return LIGHTCYAN;
         default:
             break;
     }
@@ -489,7 +507,13 @@ static string _effect_string(spell_type spell, const monster_info *mon_owner)
         }
         if (you.immune_to_hex(spell))
             return "(immune)";
-        return make_stringf("(%d%%)", hex_chance(spell, mon_owner));
+
+        const string hex_str = make_stringf("%d%%", hex_chance(spell, mon_owner));
+
+        const dice_def dam = _spell_damage(spell, hd);
+        if (!dam.size || !dam.num)
+            return make_stringf("(%s)", hex_str.c_str());
+        return make_stringf("(%s,%dd%d)", hex_str.c_str(), dam.num, dam.size);
     }
 
     if (spell == SPELL_SMITING)
@@ -504,6 +528,30 @@ static string _effect_string(spell_type spell, const monster_info *mon_owner)
         return make_stringf("0-%d MP", pow / 8); // >_> >_>
     }
 
+    if (spell == SPELL_WIND_BLAST)
+    {
+        const int pow = mons_power_for_hd(SPELL_WIND_BLAST, hd);
+        return make_stringf("2d%d*", default_collision_damage(pow, false).size);
+    }
+
+    if (spell == SPELL_FORCE_LANCE)
+    {
+        const int pow = mons_power_for_hd(SPELL_FORCE_LANCE, hd);
+            return make_stringf("%dd%d(+%dd%d)",
+                _spell_damage(spell, hd).num,
+                _spell_damage(spell, hd).size,
+                default_collision_damage(pow, false).num,
+                default_collision_damage(pow, false).size);
+    }
+
+    if (spell == SPELL_HOARFROST_BULLET)
+    {
+        const int pow = mons_power_for_hd(spell, hd);
+        return make_stringf("3d(%d/%d)",
+            zap_damage(ZAP_HOARFROST_BULLET, pow, true, false).size,
+            zap_damage(ZAP_HOARFROST_BULLET_FINALE, pow, true, false).size);
+    }
+
     const dice_def dam = _spell_damage(spell, hd);
     if (dam.num == 0 || dam.size == 0)
         return "";
@@ -520,10 +568,17 @@ static string _effect_string(spell_type spell, const monster_info *mon_owner)
     }
 
     string mult = "";
-    if (spell == SPELL_MARSHLIGHT || spell == SPELL_PLASMA_BEAM)
+    if (spell == SPELL_MARSHLIGHT
+        || spell == SPELL_FOXFIRE
+        || spell == SPELL_PLASMA_BEAM
+        || spell == SPELL_PERMAFROST_ERUPTION)
+    {
         mult = "2x";
+    }
     else if (spell == SPELL_CONJURE_BALL_LIGHTNING)
         mult = "3x";
+    else if (spell == SPELL_ELECTROLUNGE)
+        mult = "+";
     const char* asterisk = (spell == SPELL_LRD || spell == SPELL_PYRE_ARROW) ? "*" : "";
     return make_stringf("(%s%dd%d%s)", mult.c_str(), dam.num, dam.size, asterisk);
 }
@@ -561,7 +616,7 @@ static void _describe_book(const spellbook_contents &book,
     description.cprintf("\n");
 
     // list spells in two columns, instead of one? (monster books)
-    const bool doublecolumn = _list_spells_doublecolumn(source_item);
+    const bool doublecolumn = source_item == nullptr;
 
     bool first_line_element = true;
     const int hd = mon_owner ? mon_owner->spell_hd() : 0;
@@ -584,10 +639,20 @@ static void _describe_book(const spellbook_contents &book,
         const string range_str = _range_string(spell, mon_owner, hd);
         string effect_str = _effect_string(spell, mon_owner);
 
+        const string dith_marker = mon_owner
+                                   && crawl_state.need_save
+                                   && you_worship(GOD_DITHMENOS)
+                                        ? !valid_marionette_spell(spell)
+                                          ? "<magenta>!</magenta>"
+                                          : spell_has_marionette_override(spell)
+                                          ? "<lightmagenta>*</lightmagenta>" : ""
+                                        : "";
+
         const int effect_len = effect_str.length();
         const int range_len = range_str.empty() ? 0 : 3;
         const int effect_range_space = effect_len && range_len ? 1 : 0;
-        const int chop_len = 30 - effect_len - range_len - effect_range_space;
+        const int chop_len = 30 - effect_len - range_len - effect_range_space
+                                - (dith_marker.length() > 0 ? 1 : 0);
 
         if (effect_len && !testbits(get_spell_flags(spell), spflag::WL_check))
             effect_str = colourize_str(effect_str, _spell_colour(spell));
@@ -600,7 +665,8 @@ static void _describe_book(const spellbook_contents &book,
             spell_name = "Crystal Spear";
         }
         description += formatted_string::parse_string(
-                make_stringf("%c - %s%s%s%s", spell_letter,
+                make_stringf("%c - %s%s%s%s%s", spell_letter,
+                             dith_marker.c_str(),
                              chop_string(spell_name, chop_len).c_str(),
                              effect_str.c_str(),
                              effect_range_space ? " " : "",
@@ -656,7 +722,7 @@ void describe_spellset(const spellset &spells,
                        formatted_string &description,
                        const monster_info *mon_owner)
 {
-    auto spell_map = map_chars_to_spells(spells, source_item);
+    auto spell_map = map_chars_to_spells(spells);
     for (auto book : spells)
         _describe_book(book, spell_map, source_item, description, mon_owner);
 }
@@ -674,7 +740,17 @@ static void _write_book(const spellbook_contents &book,
     for (auto spell : book.spells)
     {
         tiles.json_open_object();
-        tiles.json_write_string("title", spell_title(spell));
+
+        const string dith_marker = mon_owner
+                                   && crawl_state.need_save
+                                   && you_worship(GOD_DITHMENOS)
+                                        ? !valid_marionette_spell(spell)
+                                          ? "<magenta>!</magenta>"
+                                          : spell_has_marionette_override(spell)
+                                          ? "<lightmagenta>*</lightmagenta>" : ""
+                                        : "";
+
+        tiles.json_write_string("title", dith_marker + spell_title(spell));
         tiles.json_write_int("colour", _spell_colour(spell, source_item));
         tiles.json_write_name("tile");
         tiles.write_tileidx(tileidx_spell(spell));
@@ -712,7 +788,7 @@ void write_spellset(const spellset &spells,
                        const item_def* const source_item,
                        const monster_info *mon_owner)
 {
-    auto spell_map = map_chars_to_spells(spells, source_item);
+    auto spell_map = map_chars_to_spells(spells);
     tiles.json_open_array("spellset");
     for (auto book : spells)
         _write_book(book, spell_map, source_item, mon_owner);
