@@ -97,12 +97,19 @@ void add_auto_excludes()
         monster *mon = monster_at(*ri);
         if (!mon || mon->is_summoned())
             continue;
+
         // Something of a speed hack, but some vaults have a TON of plants.
-        if (mon->type == MONS_PLANT)
+        if (mons_is_firewood(*mon))
             continue;
         if (_need_auto_exclude(mon) && !is_exclude_root(*ri))
         {
             int radius = _get_full_exclusion_radius();
+            // Sting and Harpoon Shot's minimum ranges, respectively.
+            if (mon->type == MONS_SCRUB_NETTLE)
+                radius = min(radius, 4);
+            else if (mon->type == MONS_STARFLOWER)
+                radius = min(radius, 6);
+
             set_exclude(*ri, radius, true);
             mons.emplace_back(mon);
         }
