@@ -361,21 +361,24 @@ public:
     void fire() override;
 
     static void schedule(coord_def pos, int revives, beh_type attitude,
-                         unsigned short foe, bool duel)
+                         unsigned short foe, bool duel, mon_enchant gozag_bribe)
     {
-        final_effect::schedule(new bennu_revive_fineff(pos, revives, attitude, foe, duel));
+        final_effect::schedule(new bennu_revive_fineff(pos, revives, attitude,
+                                                       foe, duel, gozag_bribe));
     }
 protected:
     bennu_revive_fineff(coord_def pos, int _revives, beh_type _att,
-                        unsigned short _foe, bool _duel)
+                        unsigned short _foe, bool _duel,
+                        mon_enchant _gozag_bribe)
         : final_effect(0, 0, pos), revives(_revives), attitude(_att), foe(_foe),
-          duel(_duel)
+          duel(_duel), gozag_bribe(_gozag_bribe)
     {
     }
     int revives;
     beh_type attitude;
     unsigned short foe;
     bool duel;
+    mon_enchant gozag_bribe;
 };
 
 class avoided_death_fineff : public final_effect
@@ -494,15 +497,19 @@ public:
     bool mergeable(const final_effect &) const override { return false; };
     void fire() override;
 
-    static void schedule(const actor &attack, const actor &defend)
+    static void schedule(const actor &attack, const actor &defend,
+                         const item_def *weapon)
     {
-        final_effect::schedule(new spectral_weapon_fineff(attack, defend));
+        final_effect::schedule(new spectral_weapon_fineff(attack, defend, weapon));
     }
 protected:
-    spectral_weapon_fineff(const actor &attack, const actor &defend)
-        : final_effect(&attack, &defend, coord_def())
+    spectral_weapon_fineff(const actor &attack, const actor &defend,
+                           const item_def *wpn)
+        : final_effect(&attack, &defend, coord_def()), weapon(wpn)
     {
     }
+
+    const item_def *weapon;
 };
 
 class lugonu_meddle_fineff : public final_effect
@@ -535,4 +542,59 @@ protected:
     }
 };
 
+class beogh_resurrection_fineff : public final_effect
+{
+public:
+    bool mergeable(const final_effect &a) const override;
+    void fire() override;
+
+    static void schedule(bool end_ostracism_only = false)
+    {
+        final_effect::schedule(new beogh_resurrection_fineff(end_ostracism_only));
+    }
+protected:
+    beogh_resurrection_fineff(bool end_ostracism_only)
+        : final_effect(nullptr, nullptr, coord_def()), ostracism_only(end_ostracism_only)
+    {
+    }
+    const bool ostracism_only;
+};
+
+class dismiss_divine_allies_fineff : public final_effect
+{
+public:
+    bool mergeable(const final_effect &) const override { return false; }
+    void fire() override;
+
+    static void schedule(const god_type god)
+    {
+        final_effect::schedule(new dismiss_divine_allies_fineff(god));
+    }
+protected:
+    dismiss_divine_allies_fineff(const god_type _god)
+        : final_effect(0, 0, coord_def()), god(_god)
+    {
+    }
+    const god_type god;
+};
+
+class death_spawn_fineff : public final_effect
+{
+public:
+    bool mergeable(const final_effect &) const override { return false; }
+    void fire() override;
+
+    static void schedule(monster_type mon_type, coord_def pos, int dur)
+    {
+        final_effect::schedule(new death_spawn_fineff(mon_type, pos, dur));
+    }
+protected:
+    death_spawn_fineff(monster_type type, coord_def pos, int dur)
+        : final_effect(0, 0, pos), mon_type(type), duration(dur)
+    {
+    }
+    const monster_type mon_type;
+    const int duration;
+};
+
 void fire_final_effects();
