@@ -23,6 +23,7 @@
 #include "mpr.h"
 #include "ng-input.h"
 #include "options.h"
+#include "random-pick.h"
 #include "skills.h"
 #include "spl-util.h"
 #include "state.h"
@@ -145,6 +146,7 @@ void ghost_demon::reset()
     colour           = COLOUR_UNDEF;
     flies            = false;
     cloud_ring_ench  = ENCH_NONE;
+    umbra_rad        = -1;
 }
 
 #define ADD_SPELL(which_spell) \
@@ -225,6 +227,7 @@ void ghost_demon::set_pan_lord_special_attack()
         // Low chance
         10, _brand_attack(SPWPN_VENOM),
         10, _brand_attack(SPWPN_DRAINING),
+        2, _brand_attack(SPWPN_FOUL_FLAME),
         4, _flavour_attack(AF_DRAIN_STR),
         4, _flavour_attack(AF_DRAIN_INT),
         2, _flavour_attack(AF_DRAIN_DEX),
@@ -264,6 +267,10 @@ void ghost_demon::set_pan_lord_special_attack()
         default:
             break;
     }
+
+    // Foul flame always gets radius-1 umbra.
+    if (brand == SPWPN_FOUL_FLAME)
+        umbra_rad = 1;
 }
 
 void ghost_demon::set_pan_lord_cloud_ring()
@@ -526,6 +533,298 @@ void ghost_demon::init_player_ghost()
     add_spells();
 }
 
+int apostle_type_by_name(const string name)
+{
+    for (int i = 0; i < NUM_APOSTLE_TYPES; ++i)
+        if (apostle_type_names[i] == lowercase_string(name))
+            return i;
+    return APOSTLE_WARRIOR;
+}
+
+static const vector<random_pick_entry<spell_type>> wizard_primary_spells =
+{
+  { -10,   5, 175, PEAK, SPELL_THROW_FLAME },
+  { -10,   5, 175, PEAK, SPELL_THROW_FROST },
+  { -10,   25, 100, SEMI, SPELL_SOUL_SPLINTER },
+  { -10,   35, 135, SEMI, SPELL_SHOCK },
+  { -10,   35, 175, SEMI, SPELL_STING },
+  { 0,  40, 200, FALL, SPELL_FOXFIRE },
+  { 0,  45, 175, SEMI, SPELL_SANDBLAST },
+  { 15,  85, 160, SEMI, SPELL_SCORCH },
+  { 15,  65,  95, SEMI, SPELL_GRAVE_CLAW },
+  { 25,  80, 150, SEMI, SPELL_STONE_ARROW },
+  { 25,  85, 105, SEMI, SPELL_SEARING_RAY },
+  { 45, 120, 150, SEMI, SPELL_LRD },
+  { 45, 140, 170, PEAK, SPELL_BORGNJORS_VILE_CLUTCH },
+  { 45, 125, 150, SEMI, SPELL_FIREBALL },
+  { 40, 125, 100, SEMI, SPELL_ICEBLAST },
+  { 45, 100,  80, FLAT, SPELL_AIRSTRIKE },
+  { 35, 75,  120, SEMI, SPELL_VENOM_BOLT },
+  { 40, 90,  190, SEMI, SPELL_BOLT_OF_DRAINING },
+  { 25, 110, 120, SEMI, SPELL_FORCE_LANCE },
+  { 30, 100,  90, PEAK, SPELL_LIGHTNING_BOLT },
+  { 20, 85,  140, PEAK, SPELL_BOLT_OF_MAGMA },
+  { 20, 65,  140, PEAK, SPELL_THROW_ICICLE },
+  { 40, 90,  110, SEMI, SPELL_POISON_ARROW },
+  { 35, 90, 155, RISE, SPELL_BOLT_OF_COLD },
+  { 35, 90, 280, RISE, SPELL_BOLT_OF_FIRE },
+  { 50, 100, 180, RISE, SPELL_BOMBARD },
+  { 60, 100,  50, RISE, SPELL_PERMAFROST_ERUPTION },
+  { 65, 100,  80, RISE, SPELL_PLASMA_BEAM },
+  { 70, 100,  70, RISE, SPELL_HELLFIRE_MORTAR },
+  { 70, 100,  90, RISE, SPELL_LEHUDIBS_CRYSTAL_SPEAR },
+};
+
+static const vector<random_pick_entry<spell_type>> wizard_secondary_spells =
+{
+  { 0,  100, 200, FLAT, SPELL_HASTE },
+  { 0,  120, 250, FALL, SPELL_CONFUSE },
+  { 0,  70,  250, FALL, SPELL_SLOW },
+  { 0,  150, 325, FALL, SPELL_BLINK },
+  { 30, 100, 150, RISE, SPELL_BLINK_RANGE },
+  { 15, 130, 100,  PEAK, SPELL_VITRIFY },
+  { 25, 130,  80,  PEAK, SPELL_PARALYSE },
+  { 25, 130, 100,  PEAK, SPELL_PETRIFY },
+  { 35, 100, 100,  FLAT, SPELL_BATTLESPHERE },
+};
+
+static const vector<random_pick_entry<spell_type>> priest_summon_spells =
+{
+  { -20,  30,  150, SEMI, SPELL_CALL_IMP },
+  { -20,  35,  110, SEMI, SPELL_SUMMON_SCORPIONS },
+  { 10, 100, 200, SEMI, SPELL_STICKS_TO_SNAKES },
+  { 10, 120, 110, SEMI, SPELL_DIVINE_ARMAMENT},
+  { 25, 100, 90, SEMI, SPELL_VANQUISHED_VANGUARD },
+  { 20, 100, 300, PEAK, SPELL_SUMMON_DEMON },
+  { 55, 150, 110, SEMI, SPELL_SUMMON_VERMIN },
+  { 85, 100, 35, RISE, SPELL_SUMMON_GREATER_DEMON },
+};
+
+static const vector<random_pick_entry<spell_type>> priest_spells =
+{
+  { 0,  40,  150, FALL, SPELL_MINOR_HEALING },
+  { 35, 100, 50,  FLAT, SPELL_INJURY_BOND },
+  { 10, 130, 100, PEAK, SPELL_MIGHT_OTHER },
+  { 10, 130, 100, PEAK, SPELL_HASTE_OTHER },
+  { 0, 100, 100, FLAT, SPELL_REGENERATE_OTHER },
+  { 0, 100,  50, SEMI, SPELL_REGENERATE_OTHER },
+  { 65, 100, 70, RISE, SPELL_MASS_REGENERATION },
+  { 40, 80, 200, SEMI, SPELL_AGONY},
+  { 40, 100, 60, SEMI, SPELL_STUNNING_BURST},
+  { 45, 100, 65,  FLAT, SPELL_PRAYER_OF_BRILLIANCE },
+  { 65, 100, 65,  RISE, SPELL_CALL_DOWN_LIGHTNING },
+  { 70, 100, 40, RISE, SPELL_CONJURE_LIVING_SPELLS },
+};
+
+static const vector<random_pick_entry<spell_type>> warrior_spells =
+{
+  { 0,  50,  175, FALL, SPELL_NO_SPELL },
+  { 45, 175,  50, SEMI, SPELL_HASTE },
+  { 40, 175,  70, SEMI, SPELL_MIGHT },
+  { 15, 100, 100, RISE, SPELL_HARPOON_SHOT },
+  { 20, 100,  80, RISE, SPELL_ELECTROLUNGE },
+  { 40, 100,  80, RISE, SPELL_MANIFOLD_ASSAULT },
+  { 25, 100,  30, FLAT, SPELL_INJURY_BOND },
+};
+
+// Map to modify base frequency, for spells that should be used much more/less than others
+static const map<spell_type, int> freq_map =
+{
+    {SPELL_BLINK, 15},
+    {SPELL_BLINK_RANGE, 30},
+    {SPELL_PARALYSE, 20},
+    {SPELL_SLOW, 20},
+    {SPELL_CONFUSE, 20},
+    {SPELL_PETRIFY, 20},
+    {SPELL_MIGHT, 35},
+    {SPELL_HASTE, 35},
+    {SPELL_BATTLESPHERE, 80},
+    {SPELL_PRAYER_OF_BRILLIANCE, 60},
+    {SPELL_CALL_DOWN_LIGHTNING, 15},
+    {SPELL_SEARING_RAY, 30},
+    {SPELL_CONJURE_LIVING_SPELLS, 40},
+    {SPELL_MANIFOLD_ASSAULT, 40},
+};
+
+static bool _has_matching_elemental_school(spell_type spell1, spell_type spell2)
+{
+    const spschools_type disciplines1 = get_spell_disciplines(spell1);
+    const spschools_type disciplines2 = get_spell_disciplines(spell2);
+
+    if (disciplines1 & spschool::air && disciplines2 & spschool::air)
+        return true;
+    if (disciplines1 & spschool::fire && disciplines2 & spschool::fire)
+        return true;
+    if (disciplines1 & spschool::ice && disciplines2 & spschool::ice)
+        return true;
+    if (disciplines1 & spschool::earth && disciplines2 & spschool::earth)
+        return true;
+    if (disciplines1 & spschool::alchemy && disciplines2 & spschool::alchemy)
+        return true;
+
+    return false;
+}
+
+static void _add_apostle_spell(monster_spells& spells, spell_type type,
+                               int default_pow, mon_spell_slot_flag flag)
+{
+    if (type == SPELL_NO_SPELL)
+        return;
+
+    const int* freq = map_find(freq_map, type);
+    if (freq)
+        default_pow = *freq;
+
+    spells.emplace_back(type, default_pow, flag);
+}
+
+void ghost_demon::pick_apostle_spells(apostle_type type, int pow)
+{
+    random_picker<spell_type, NUM_SPELLS> picker;
+
+    switch (type)
+    {
+        case APOSTLE_WARRIOR:
+            // Warriors can know battlecry, and then have a chance of one other spell
+            if (pow > 20 && x_chance_in_y(pow, 150))
+                spells.emplace_back(SPELL_BATTLECRY, 50, MON_SPELL_VOCAL);
+            if (coinflip())
+            {
+                _add_apostle_spell(spells, picker.pick(warrior_spells, pow, SPELL_NO_SPELL),
+                                   20, MON_SPELL_WIZARD);
+            }
+
+            // Spell-less warriors get a little bonus health
+            if (spells.empty())
+                max_hp = max_hp * 7 / 6;
+
+        break;
+
+        case APOSTLE_WIZARD:
+        {
+            // Wizards always know at least 2 spells - a primary spell and either
+            // another primary or a secondary. If they know 3, 2 of them will
+            // be primary and one secondary.
+
+            // When picking a second primary spell, several attempts are made to
+            // find another spell that shares a school with their first spell
+            // that is not conjurations. Just to make the spellsets seem slightly
+            // more themed than they would by chance.
+
+            const spell_type primary =
+                picker.pick(wizard_primary_spells, pow, SPELL_NO_SPELL);
+
+            spell_type primary2 = SPELL_NO_SPELL;
+            int pow2 = max(-10, pow - random_range(0, 20));
+            for (int tries = 0; tries < 3; ++tries)
+            {
+                // Keep rerolling until we find something other than our starting
+                // spell. (This is almost guaranteed to take very few tries)
+                while (primary2 == primary || primary2 == SPELL_NO_SPELL)
+                    primary2 = picker.pick(wizard_primary_spells, pow2, SPELL_NO_SPELL);
+
+                if (_has_matching_elemental_school(primary, primary2))
+                    break;
+            }
+            const spell_type secondary =
+                picker.pick(wizard_secondary_spells, pow, SPELL_NO_SPELL);
+
+            _add_apostle_spell(spells, primary, 45, MON_SPELL_WIZARD);
+
+            if (coinflip() || x_chance_in_y(pow, 50))
+                _add_apostle_spell(spells, primary2, 35, MON_SPELL_WIZARD);
+
+            if (spells.size() == 1 || coinflip() || x_chance_in_y(pow, 60))
+                _add_apostle_spell(spells, secondary, 30, MON_SPELL_WIZARD);
+
+            if (spells.size() == 3 && one_chance_in(4))
+            {
+                spell_type secondary2 = SPELL_NO_SPELL;
+                while (secondary2 == secondary || secondary2 == SPELL_NO_SPELL)
+                    secondary2 = picker.pick(wizard_secondary_spells, pow, SPELL_NO_SPELL);
+                _add_apostle_spell(spells, secondary2, 30, MON_SPELL_WIZARD);
+            }
+        }
+        break;
+
+        case APOSTLE_PRIEST:
+        {
+            // Priests always know smite (cast chance rises with level)
+            spells.emplace_back(SPELL_SMITING, 20 + div_rand_round(pow, 4), MON_SPELL_PRIEST);
+
+            // Then they know at least one other spell, and maybe 2, but never
+            // more than one summon.
+            bool summon_learned = false;
+            int num_spells = 1 + x_chance_in_y(pow + 15, 100);
+            spell_type spell = SPELL_NO_SPELL;
+            for (int i = 0; i < num_spells; ++i)
+            {
+                if (!summon_learned && one_chance_in(3))
+                {
+                    spell = picker.pick(priest_summon_spells, pow, SPELL_NO_SPELL);
+                    summon_learned = true;
+                }
+                else
+                    spell = picker.pick(priest_spells, pow, SPELL_NO_SPELL);
+
+                spells.emplace_back(spell, 15 + div_rand_round(pow, 9), MON_SPELL_PRIEST);
+            }
+        }
+        break;
+
+        default:
+            break;
+    }
+}
+
+void ghost_demon::init_orc_apostle(apostle_type type, int pow)
+{
+    const monsterentry* stats = get_monster_data(MONS_ORC_APOSTLE);
+
+    speed = stats->speed;
+    ev = stats->ev;
+    ac = stats->AC + div_rand_round(pow, 10);
+    move_energy = stats->energy_usage.move;
+    see_invis = true;
+
+    // Base hp scales from ~50 at 0 power to ~150 at 100 power
+    max_hp = 50 + pow * 7 / 10 + (max(0, pow - 40) * 5 / 10);
+    xl = 5 + div_rand_round(pow, 7) + random_range(-2, 2);
+
+    // Adjust stats based on class
+    switch (type)
+    {
+        default:
+        case APOSTLE_WARRIOR:
+            max_hp = max_hp * 6 / 5;
+            damage = 5 + div_rand_round(pow, 4);
+            if (pow > 50)
+                damage += div_rand_round(pow, 7);
+            break;
+
+        case APOSTLE_WIZARD:
+            max_hp = max_hp * 5 / 6;
+            damage = 2 + div_rand_round(pow, 8);
+            ev += div_rand_round(pow, 12);
+
+            // A little spellpower boost compared to others (but not for the
+            // very earliest challenges)
+            if (xl > 5)
+                xl += 2;
+            break;
+
+        case APOSTLE_PRIEST:
+            damage = 3 + div_rand_round(pow, 6);
+            break;
+    }
+
+    pick_apostle_spells(type, pow);
+
+    // Randomize stats
+    max_hp = max_hp + random_range(-(max_hp / 5), max_hp / 5);
+    damage = damage + random_range(-(damage / 4), damage / 4);
+}
+
 static colour_t _ugly_thing_assign_colour(colour_t force_colour,
                                           colour_t force_not_colour)
 {
@@ -727,9 +1026,6 @@ void ghost_demon::init_dancing_weapon(const item_def& weapon, int power)
     damage  = 2 * damg;
     max_hp  = delay * 2;
 
-    // Don't allow the speed to become too low.
-    speed = max(3, (speed / 2) * (1 + power / 100));
-
     ev    = max(3, ev * power / 100);
     ac = ac * power / 100;
     max_hp = max(5, max_hp * power / 100);
@@ -770,7 +1066,7 @@ void ghost_demon::add_spells()
 {
     spells.clear();
 
-    for (int i = 0; i < you.spell_no; i++)
+    for (int i = 0; i < MAX_KNOWN_SPELLS; i++)
     {
         const int chance = max(0, 50 - failure_rate_to_int(raw_spell_fail(you.spells[i])));
         const spell_type spell = translate_spell(you.spells[i]);
@@ -801,16 +1097,18 @@ spell_type ghost_demon::translate_spell(spell_type spell) const
     case SPELL_CONTROLLED_BLINK:
         return SPELL_BLINK;
 #endif
-    case SPELL_DRAGON_CALL:
-        return SPELL_SUMMON_DRAGON;
     case SPELL_SWIFTNESS:
         return SPELL_SPRINT;
-    case SPELL_NECROTISE:
-        return SPELL_PAIN;
     case SPELL_CONFUSING_TOUCH:
         return SPELL_CONFUSE;
     case SPELL_CURSE_OF_AGONY:
         return SPELL_AGONY;
+    case SPELL_STARBURST:
+        return SPELL_BOLT_OF_FIRE;
+    case SPELL_DRAGON_CALL:
+        return SPELL_SUMMON_DRAGON;
+    case SPELL_ELECTRIC_CHARGE:
+        return SPELL_ELECTROLUNGE;
     default:
         break;
     }
