@@ -16,6 +16,7 @@
 #include "artefact.h"
 #include "attitude-change.h"
 #include "cleansing-flame-source-type.h"
+#include "colour.h"
 #include "coordit.h"
 #include "corpse.h"
 #include "database.h"
@@ -128,29 +129,6 @@ static mgen_data _wrath_mon_data(monster_type mtyp, god_type god)
     return mg;
 }
 
-static bool _yred_random_zombified_hostile()
-{
-    const bool skel = one_chance_in(4);
-
-    monster_type z_base;
-
-    do
-    {
-        // XXX: better zombie selection?
-        level_id place(BRANCH_DUNGEON,
-                       min(27, you.experience_level + 5));
-        z_base = pick_local_zombifiable_monster(place, RANDOM_MONSTER,
-                                                you.pos());
-    }
-    while (skel && !mons_skeleton(z_base));
-
-    mgen_data temp = _wrath_mon_data(skel ? MONS_SKELETON : MONS_ZOMBIE,
-                                     GOD_YREDELEMNUL)
-                     .set_base(z_base);
-
-    return create_monster(temp, false);
-}
-
 static const vector<pop_entry> _okawaru_servants =
 { // warriors
   {  1,  3,   3, FALL, MONS_ORC },
@@ -194,17 +172,6 @@ static bool _okawaru_random_servant()
     return create_monster(temp, false);
 }
 
-static bool _dithmenos_random_shadow(const int count, const int tier)
-{
-    monster_type mon_type = MONS_SHADOW;
-    if (tier >= 2 && count == 0 && coinflip())
-        mon_type = MONS_TZITZIMITL;
-    else if (tier >= 1 && count < 3 && coinflip())
-        mon_type = MONS_SHADOW_DEMON;
-
-    return create_monster(_wrath_mon_data(mon_type, GOD_DITHMENOS), false);
-}
-
 /**
  * Summon divine warriors of the Shining One to punish the player.
  */
@@ -219,10 +186,16 @@ static void _tso_summon_warriors()
             success = true;
     }
 
-    simple_god_message(success ? " sends the divine host to punish "
-                       "you for your evil ways!"
-                       : "'s divine host fails to appear.", GOD_SHINING_ONE);
-
+    if (success)
+    {
+        simple_god_message(" sends the divine host to punish you for your evil "
+                           "ways!", false, GOD_SHINING_ONE);
+    }
+    else
+    {
+        simple_god_message(" divine host fails to appear.",
+                           true, GOD_SHINING_ONE);
+    }
 }
 
 /**
@@ -230,9 +203,8 @@ static void _tso_summon_warriors()
  */
 static void _tso_shouts()
 {
-    simple_god_message(" booms out: "
-                       "\"Take the path of righteousness! REPENT!\"",
-                       GOD_SHINING_ONE);
+    simple_god_message(" booms out: Take the path of righteousness! REPENT!",
+                       false, GOD_SHINING_ONE);
     noisy(25, you.pos()); // same as scroll of noise
 }
 
@@ -284,7 +256,7 @@ static bool _zin_remove_good_mutations()
     const god_type god = GOD_ZIN;
     bool success = false;
 
-    simple_god_message(" draws some chaos from your body!", god);
+    simple_god_message(" draws some chaos from your body!", false, god);
 
     bool failMsg = true;
 
@@ -302,7 +274,7 @@ static bool _zin_remove_good_mutations()
     }
 
     if (success && !you.how_mutated())
-        simple_god_message(" rids your body of chaos!", god);
+        simple_god_message(" rids your body of chaos!", false, god);
     return success;
 }
 
@@ -323,8 +295,8 @@ static bool _zin_retribution()
     case 2:
     case 3:
     case 4: // recital
-        simple_god_message(" recites the Axioms of Law to you!", god);
-        switch (random2(3))
+        simple_god_message(" recites the Axioms of Law to you!", false, god);
+        switch (random2(4))
         {
         case 0:
             confuse_player(5 + random2(3));
@@ -335,10 +307,13 @@ static bool _zin_retribution()
         case 2:
             paralyse_player(_god_wrath_name(god));
             return false;
+        case 3:
+            blind_player(20 + random2(15), ETC_SILVER);
+            return false;
         }
         break;
     case 5: // noisiness
-        simple_god_message(" booms out: \"Turn to the light! REPENT!\"", god);
+        simple_god_message(" booms out: Turn to the light! REPENT!", false, god);
         noisy(25, you.pos()); // same as scroll of noise
         break;
     }
@@ -383,7 +358,7 @@ static bool _cheibriados_retribution()
     // Very high tension wrath.
     // Add noise then start sleeping and slow the player with 2/3 chance.
     case 4:
-        simple_god_message(" strikes the hour.", god);
+        simple_god_message(" strikes the hour.", false, god);
         noisy(40, you.pos());
         dec_penance(god, 1); // and fall-through.
     // High tension wrath
@@ -461,7 +436,7 @@ static void _spell_retribution(monster* avatar, spell_type spell, god_type god,
                                const char* message = nullptr)
 {
     simple_god_message(message ? message : " rains destruction down upon you!",
-                       god);
+                       false, god);
     bolt beam;
     beam.source = you.pos();
     beam.target = you.pos();
@@ -517,22 +492,25 @@ static spell_type _makhleb_destruction_type()
  * @param god           The god doing the wrath-hurling.
  * @return              An avatar monster, or nullptr if none could be set up.
  */
-static monster* get_avatar(god_type god)
+static monster* _get_wrath_avatar(god_type god)
 {
-    // TODO: it would be better to abstract the fake monster code from both
-    // this and shadow monster and possibly use different monster types --
-    // doing it this way makes it easier for bugs where the two are conflated
-    // to creep in
-    monster* avatar = shadow_monster(false);
+    if (monster_at(you.pos()))
+        return nullptr;
+
+    monster* avatar = get_free_monster();
     if (!avatar)
         return nullptr;
 
-    // shadow_monster() has the player's mid, which is no good here.
+    avatar->type       = MONS_GOD_WRATH_AVATAR;
+    avatar->behaviour  = BEH_SEEK;
+    avatar->attitude   = ATT_HOSTILE;
+    avatar->flags      = MF_NO_REWARD | MF_JUST_SUMMONED | MF_SEEN
+                         | MF_WAS_IN_VIEW | MF_HARD_RESET | MF_NAME_REPLACE;
+    avatar->god        = god;
+    avatar->set_position(you.pos());
     avatar->set_new_monster_id();
-
+    env.mgrid(you.pos()) = avatar->mindex();
     avatar->mname = _god_wrath_name(god);
-    avatar->flags |= MF_NAME_REPLACE;
-    avatar->attitude = ATT_HOSTILE;
     avatar->set_hit_dice(you.experience_level);
 
     return avatar;
@@ -542,7 +520,7 @@ static monster* get_avatar(god_type god)
 static void _reset_avatar(monster &avatar)
 {
     env.mid_cache.erase(avatar.mid);
-    shadow_monster_reset(&avatar);
+    avatar.reset();
 }
 
 /**
@@ -554,12 +532,13 @@ static bool _makhleb_call_down_destruction()
 {
     const god_type god = GOD_MAKHLEB;
 
-    monster* avatar = get_avatar(god);
+    monster* avatar = _get_wrath_avatar(god);
     // can't be const because mons_cast() doesn't accept const monster*
 
     if (avatar == nullptr)
     {
-        simple_god_message(" has no time to deal with you just now.", god);
+        simple_god_message(" has no time to deal with you just now.", false,
+                           god);
         return false; // not a very dazzling divine experience...
     }
 
@@ -636,9 +615,14 @@ static bool _makhleb_summon_servants()
             summoned++;
     }
 
-    simple_god_message(summoned > 1 ? " sends minions to punish you." :
-                       summoned > 0 ? " sends a minion to punish you."
-                       : "'s minions fail to arrive.", GOD_MAKHLEB);
+    if (summoned > 0)
+    {
+        simple_god_message(summoned > 1 ? " sends minions to punish you." :
+                                          " sends a minion to punish you.",
+                           false, GOD_MAKHLEB);
+    }
+    else
+        simple_god_message(" minions fail to arrive.", true, GOD_MAKHLEB);
 
     return true;
 
@@ -717,53 +701,29 @@ static bool _kikubaaqudgha_retribution()
 
 static bool _yredelemnul_retribution()
 {
-    // undead theme
     const god_type god = GOD_YREDELEMNUL;
 
+    int how_many = random_range(2, 4);
+    int count = 0;
+    for (int i = 0; i < how_many; ++i)
+    {
+        if (yred_random_servant(you.experience_level, true), true)
+            ++count;
+    }
+
+    if (count > 0)
+    {
+        simple_god_message(count > 1 ? " sends servants to punish you." :
+                                       " sends a servant to punish you.",
+                           false, god);
+    }
+    else
+        simple_god_message(" servants fail to arrive.", true, god);
+
     if (coinflip())
     {
-        if (you_worship(god) && coinflip() && yred_reclaim_souls())
-            ;
-        else
-        {
-            int how_many = 1 + random2avg(1 + (you.experience_level / 5), 2);
-            int count = 0;
-
-            for (; how_many > 0; --how_many)
-            {
-                if (one_chance_in(you.experience_level))
-                {
-                    if (_yred_random_zombified_hostile())
-                        ++count;
-                }
-                else
-                {
-                    if (yred_random_servant(0, true))
-                        ++count;
-                    else
-                        ++how_many;
-                }
-            }
-
-            simple_god_message(count > 1 ? " sends servants to punish you." :
-                               count > 0 ? " sends a servant to punish you."
-                                         : "'s servants fail to arrive.", god);
-        }
-    }
-    else
-    {
-        monster* avatar = get_avatar(god);
-        // can't be const because mons_cast() doesn't accept const monster*
-
-        if (avatar == nullptr)
-        {
-            simple_god_message(" has no time to deal with you just now.", god);
-            return false;
-        }
-
-        _spell_retribution(avatar, SPELL_BOLT_OF_DRAINING, god,
-                           "'s anger turns toward you for a moment.");
-        _reset_avatar(*avatar);
+        simple_god_message(" binds you in chains!", false, god);
+        you.increase_duration(DUR_NO_MOMENTUM, random_range(3, 8));
     }
 
     return true;
@@ -806,11 +766,11 @@ static bool _trog_retribution()
         simple_god_message(count > 1 ? " sends monsters to punish you." :
                            count > 0 ? " sends a monster to punish you."
                                      : " has no time to punish you... now.",
-                           god);
+                           false, god);
     }
     else if (!one_chance_in(3))
     {
-        simple_god_message("'s voice booms out, \"Feel my wrath!\"", god);
+        simple_god_message(" voice booms out: Feel my wrath!", true, god);
 
         // A collection of physical effects that might be better
         // suited to Trog than wild fire magic... messages could
@@ -846,12 +806,13 @@ static bool _trog_retribution()
         // A fireball is magic when used by a mortal but just a manifestation
         // of pure rage when used by a god. --ebering
 
-        monster* avatar = get_avatar(god);
+        monster* avatar = _get_wrath_avatar(god);
         // can't be const because mons_cast() doesn't accept const monster*
 
         if (avatar == nullptr)
         {
-            simple_god_message(" has no time to deal with you just now.", god);
+            simple_god_message(" has no time to deal with you just now.", false,
+                               god);
             return false; // not a very dazzling divine experience...
         }
 
@@ -922,15 +883,10 @@ static bool _beogh_retribution()
             msg << " throws "
                 << (num_created == 1 ? "an implement" : "implements")
                 << " of electrocution at you.";
-            simple_god_message(msg.str().c_str(), god);
+            simple_god_message(msg.str().c_str(), false, god);
             break;
         } // else fall through
     }
-    case 3: // 25%, relatively harmless
-    case 4: // in effect, only for penance
-        if (you_worship(god) && beogh_followers_abandon_you())
-            break;
-        // else fall through
     default: // send orcs after you (3/8 to 5/8)
     {
         const int points = you.experience_level + 3
@@ -960,7 +916,7 @@ static bool _beogh_retribution()
 
         simple_god_message(
             mons ? " sends forth an army of orcs."
-                 : " is still gathering forces against you.", god);
+                 : " is still gathering forces against you.", false, god);
     }
     }
 
@@ -978,8 +934,10 @@ static bool _okawaru_retribution()
     for (; how_many > 0; --how_many)
         count += _okawaru_random_servant();
 
-    simple_god_message(count > 0 ? " sends forces against you!"
-                                 : "'s forces are busy with other wars.", god);
+    if (count > 0)
+        simple_god_message(" sends forces against you!", false, god);
+    else
+        simple_god_message(" forces are busy with other wars.", true, god);
 
     return true;
 }
@@ -989,7 +947,7 @@ static bool _sif_muna_retribution()
     // magic/intelligence theme
     const god_type god = GOD_SIF_MUNA;
 
-    simple_god_message("'s wrath finds you.", god);
+    simple_god_message(" wrath finds you.", true, god);
 
     switch (random2(10))
     {
@@ -1038,12 +996,12 @@ static void _lugonu_transloc_retribution()
     if (coinflip())
     {
         // Give extra opportunities for embarrassing teleports.
-        simple_god_message("'s wrath scatters you!", god);
+        simple_god_message(" wrath scatters you!", true, god);
         you_teleport_now(false, true, "Space warps around you!");
     }
     else if (coinflip())
     {
-        simple_god_message(" draws you home!", god);
+        simple_god_message(" draws you home!", false, god);
         you.banish(nullptr, "Lugonu's touch", you.get_experience_level(), true);
     }
 }
@@ -1102,8 +1060,10 @@ static void _lugonu_minion_retribution()
             success = true;
     }
 
-    simple_god_message(success ? " sends minions to punish you."
-                               : "'s minions fail to arrive.", god);
+    if (success)
+        simple_god_message(" sends minions to punish you.", false, god);
+    else
+        simple_god_message(" minions fail to arrive.", true, god);
 }
 
 /**
@@ -1187,17 +1147,19 @@ static bool _vehumet_retribution()
 {
     const god_type god = GOD_VEHUMET;
 
-    monster* avatar = get_avatar(god);
+    monster* avatar = _get_wrath_avatar(god);
     if (!avatar)
     {
-        simple_god_message(" has no time to deal with you just now.", god);
+        simple_god_message(" has no time to deal with you just now.", false,
+                           god);
         return false;
     }
 
     const spell_type spell = _vehumet_wrath_type();
     if (spell == SPELL_NO_SPELL)
     {
-        simple_god_message(" has no time to deal with you just now.", god);
+        simple_god_message(" has no time to deal with you just now.", false,
+                           god);
         _reset_avatar(*avatar);
         return false;
     }
@@ -1212,7 +1174,8 @@ static bool _nemelex_retribution()
     // card theme
     const god_type god = GOD_NEMELEX_XOBEH;
 
-    simple_god_message(" makes you draw from the deck of Punishment.", god);
+    simple_god_message(" makes you draw from the deck of Punishment.", false,
+                       god);
     draw_from_deck_of_punishment();
     return true;
 }
@@ -1246,7 +1209,7 @@ static void _jiyva_remove_slime_mutation()
         return;
 
     const god_type god = GOD_JIYVA;
-    simple_god_message("'s gift of slime is revoked.", god);
+    simple_god_message(" gift of slime is revoked.", true, god);
     delete_mutation(RANDOM_SLIME_MUTATION, _god_wrath_name(god),
                     true, false, true);
 }
@@ -1346,12 +1309,13 @@ static void _fedhas_nature_retribution()
 {
     const god_type god = GOD_FEDHAS;
 
-    monster* avatar = get_avatar(god);
+    monster* avatar = _get_wrath_avatar(god);
     // can't be const because mons_cast() doesn't accept const monster*
 
     if (avatar == nullptr)
     {
-        simple_god_message(" has no time to deal with you just now.", god);
+        simple_god_message(" has no time to deal with you just now.", false,
+                           god);
         return;
     }
 
@@ -1790,7 +1754,7 @@ static bool _fedhas_retribution()
         // fall through to the elemental miscast effects.
         if (_fedhas_corpse_spores(BEH_HOSTILE))
         {
-            simple_god_message(" produces spores.", god);
+            simple_god_message(" produces spores.", false, god);
             return true;
         }
 
@@ -1804,6 +1768,36 @@ static bool _fedhas_retribution()
     }
 }
 
+static spell_type _get_hostile_shadow_spell()
+{
+    vector<spell_type> spells;
+
+    for (spell_type player_spell : you.spells)
+    {
+        const spschools_type schools = get_spell_disciplines(player_spell);
+
+        // Including only the spells that would work fine for an independent
+        // player shadow for now.
+        if (schools & spschool::fire)
+            spells.push_back(SPELL_SHADOW_BALL);
+        if (schools & spschool::ice)
+            spells.push_back(SPELL_CREEPING_SHADOW);
+        if (schools & spschool::earth)
+            spells.push_back(SPELL_SHADOW_SHARD);
+        if (schools & spschool::conjuration)
+            spells.push_back(SPELL_SHADOW_BEAM);
+        if (schools & spschool::hexes)
+            spells.push_back(SPELL_SHADOW_TORPOR);
+        if (schools & spschool::summoning)
+            spells.push_back(SPELL_SHADOW_PUPPET);
+    }
+
+    if (spells.empty())
+        return SPELL_NO_SPELL;
+    else
+        return spells[random2(spells.size())];
+}
+
 static bool _dithmenos_retribution()
 {
     // shadow theme
@@ -1814,42 +1808,38 @@ static bool _dithmenos_retribution()
     case 0:
     {
         int count = 0;
-        int how_many = 3 + random2avg(div_rand_round(you.experience_level, 3),
-                                      2);
-        const int tier = div_rand_round(you.experience_level, 9);
+        int how_many = random_range(3, 5);
+        mgen_data mg = _wrath_mon_data(MONS_SHADOW_PUPPET, GOD_DITHMENOS);
+        mg.hd = 3 + div_rand_round(you.experience_level * 2, 3);
         while (how_many-- > 0)
         {
-            if (_dithmenos_random_shadow(count, tier))
+            if (create_monster(mg, false))
                 count++;
         }
-        simple_god_message(count ? " calls forth shadows to punish you."
+        simple_god_message(count ? " weaves the shadows around you into monsters."
                                  : " fails to incite the shadows against you.",
-                           god);
+                           false, god);
+
+        if (coinflip())
+        {
+            mpr("You feel lethargic.");
+            you.increase_duration(DUR_SLOW, random_range(10, 20), 100);
+        }
+
         break;
     }
     case 1:
     {
-        int count = 0;
-        int how_many = 2 + random2avg(div_rand_round(you.experience_level, 4),
-                                      4);
-        for (int i = 0; i < how_many; ++i)
+        spell_type spell = _get_hostile_shadow_spell();
+        coord_def pos = find_newmons_square(MONS_PLAYER_SHADOW, you.pos());
+        if (!pos.origin())
         {
-            if (create_monster(
-                    mgen_data(
-                        RANDOM_MOBILE_MONSTER, BEH_HOSTILE, you.pos(), MHITYOU,
-                              MG_NONE, god)
-                            .set_place(level_id(BRANCH_DUNGEON,
-                                                min(27,
-                                                    you.experience_level + 5)))
-                            .set_summoned(nullptr, 4, MON_SUMM_WRATH)
-                            .set_non_actor_summoner(_god_wrath_name(god))))
-            {
-                count++;
-            }
+            monster* shadow = create_player_shadow(pos, false, spell);
+            simple_god_message(shadow ? " turns your shadow against you."
+                                      : " fails to turn your shadows against you.",
+                               false, god);
         }
-        simple_god_message(count ? " weaves monsters from the shadows."
-                                 : " fails to weave monsters from the shadows.",
-                           god);
+
         break;
     }
     case 2:
@@ -1860,7 +1850,7 @@ static bool _dithmenos_retribution()
         break;
     }
     case 3:
-        simple_god_message(" tears the shadows away from you.", god);
+        simple_god_message(" tears the darkness away from you.", false, god);
         you.sentinel_mark();
         break;
     }
@@ -1910,9 +1900,15 @@ static void _qazlal_summon_elementals()
     }
 
     if (success)
-        simple_god_message(" incites the elements against you!", god);
+    {
+        simple_god_message(" incites the elements against you!", false,
+                           god);
+    }
     else
-        simple_god_message(" fails to incite the elements against you.", god);
+    {
+        simple_god_message(" fails to incite the elements against you.", false,
+                           god);
+    }
 }
 
 /**
@@ -1926,12 +1922,12 @@ static void _qazlal_elemental_vulnerability()
                false, true, false, MUTCLASS_TEMPORARY))
     {
         simple_god_message(" strips away your elemental protection.",
-                           god);
+                           false, god);
     }
     else
     {
         simple_god_message(" fails to strip away your elemental protection.",
-                           god);
+                           false, god);
     }
 }
 
@@ -1946,7 +1942,7 @@ static bool _qazlal_retribution()
 {
     if (coinflip())
     {
-        simple_god_message(" causes a mighty clap of thunder!",
+        simple_god_message(" causes a mighty clap of thunder!", false,
                            GOD_QAZLAL);
         noisy(25, you.pos());
     }
@@ -2017,11 +2013,11 @@ static bool _wu_jian_retribution()
         {
         case 0:
             wu_jian_sifu_message(" says: Die by a thousand cuts!");
-            you.set_duration(DUR_BARBS, random_range(5, 10));
+            barb_player(random_range(5, 10), 5);
             break;
         case 1:
             wu_jian_sifu_message(" whispers: Nowhere to run...");
-            you.set_duration(DUR_SLOW, random_range(5, 10));
+            slow_player(random_range(5, 10));
             break;
         case 2:
             wu_jian_sifu_message(" whispers: These will loosen your tongue!");
@@ -2035,7 +2031,7 @@ static bool _wu_jian_retribution()
         }
     }
     else
-        simple_god_message("'s divine weapons fail to arrive.", god);
+        simple_god_message(" divine weapons fail to arrive.", true, god);
 
     return true;
 }
@@ -2055,7 +2051,7 @@ static void _summon_ignis_elementals()
         god_speaks(god, msg.c_str());
     }
     else
-        simple_god_message("' divine wrath fails to arrive.", god);
+        simple_god_message(" divine wrath fails to arrive.", true, god);
 }
 
 static bool _ignis_shaft()
@@ -2065,7 +2061,8 @@ static bool _ignis_shaft()
     if (!you.shaftable())
         return false;
 
-    simple_god_message(" burns the ground from beneath your feet!", GOD_IGNIS);
+    simple_god_message(" burns the ground from beneath your feet!", false,
+                       GOD_IGNIS);
 
     // player::do_shaft() already checks resist_dislodge, but the message is a
     // bit worse.
@@ -2137,8 +2134,9 @@ static bool _ignis_champion()
         return false;
     // Message ordering is a bit touchy here.
     // First, we say what we're doing. TODO: more fun messages
-    simple_god_message(make_stringf(" anoints %s as an instrument of vengeance!",
-                                    mon->name(DESC_THE).c_str()).c_str(), GOD_IGNIS);
+    simple_god_message(make_stringf(" anoints %s as an instrument of "
+                                    "vengeance!", mon->name(DESC_THE).c_str()).c_str(),
+                                    false, GOD_IGNIS);
     // Then, we add the ench. This makes it visible if it wasn't, since it's both
     // confusing and unfun for players otherwise.
     mon->add_ench(mon_enchant(ENCH_FIRE_CHAMPION, 1));
@@ -2172,7 +2170,7 @@ static bool _uskayaw_retribution()
 
     // check if we have monsters around
     monster* mon = nullptr;
-    mon = choose_random_nearby_monster(0, _choose_hostile_monster);
+    mon = choose_random_nearby_monster(_choose_hostile_monster);
 
     switch (random2(5))
     {
@@ -2181,7 +2179,8 @@ static bool _uskayaw_retribution()
         if (mon && mon->can_go_berserk())
         {
             simple_god_message(make_stringf(" drives %s into a dance frenzy!",
-                                     mon->name(DESC_THE).c_str()).c_str(), god);
+                                     mon->name(DESC_THE).c_str()).c_str(),
+                                     false, god);
             mon->go_berserk(true);
             return true;
         }
@@ -2191,8 +2190,8 @@ static bool _uskayaw_retribution()
     case 3:
         if (mon)
         {
-            simple_god_message(" booms out, \"Time for someone else to take a solo\"",
-                                    god);
+            simple_god_message(" booms out: Time for someone else to take a "
+                               "solo!", false, god);
             paralyse_player(_god_wrath_name(god));
             dec_penance(god, 1);
             return false;
@@ -2200,7 +2199,8 @@ static bool _uskayaw_retribution()
         // else we intentionally fall through
 
     case 4:
-        simple_god_message(" booms out: \"Revellers, it's time to dance!\"", god);
+        simple_god_message(" booms out: Revellers, it's time to dance!",
+                           false, god);
         noisy(35, you.pos());
         break;
     }
@@ -2214,10 +2214,10 @@ bool divine_retribution(god_type god, bool no_bonus, bool force)
     if (is_unavailable_god(god))
         return false;
 
-    // Good gods don't use divine retribution on their followers, and
-    // gods don't use divine retribution on followers of gods they don't
+    // Good gods (and Beogh) don't use divine retribution on their followers,
+    // and gods don't use divine retribution on followers of gods they don't
     // hate.
-    if (!force && ((god == you.religion && is_good_god(god))
+    if (!force && ((god == you.religion && (is_good_god(god) || god == GOD_BEOGH))
         || (god != you.religion && !god_hates_your_god(god))))
     {
         return false;
@@ -2305,7 +2305,7 @@ static void _tso_blasts_cleansing_flame(const char *message)
     if (message)
         god_speaks(GOD_SHINING_ONE, message);
 
-    simple_god_message(" blasts you with cleansing flame!",
+    simple_god_message(" blasts you with cleansing flame!", false,
                        GOD_SHINING_ONE);
 
     // damage is 2d(pow), *3/2 for undead and demonspawn
@@ -2345,7 +2345,7 @@ static void _god_smites_you(god_type god, const char *message,
     for (int i = 0; i < 5; ++i)
         divine_hurt += random2(you.experience_level);
 
-    simple_god_message(" smites you!", god);
+    simple_god_message(" smites you!", false, god);
     ouch(divine_hurt, death_type, MID_NOBODY, aux.c_str());
 }
 
