@@ -144,12 +144,14 @@ public:
     // NOTE: Does not set menu title, ever! You *must* set the title explicitly
     menu_letter load_items(const vector<const item_def*> &items,
                            function<MenuEntry* (MenuEntry*)> procfn = nullptr,
-                           menu_letter ckey = 'a', bool sort = true);
+                           menu_letter ckey = 'a', bool sort = true,
+                           bool subkeys = false);
 
     // Make sure this menu does not outlive items, or mayhem will ensue!
     menu_letter load_items(const vector<item_def>& items,
                            function<MenuEntry* (MenuEntry*)> procfn = nullptr,
-                           menu_letter ckey = 'a', bool sort = true);
+                           menu_letter ckey = 'a', bool sort = true,
+                           bool subkeys = false);
 
     // Loads items from the player's inventory into the menu, and sets the
     // title to the stock title. If "procfn" is provided, it'll be called for
