@@ -85,9 +85,9 @@ enum armour_type
 #endif
     ARM_STORM_DRAGON_ARMOUR,
 #if TAG_MAJOR_VERSION == 34
-    ARM_GOLD_DRAGON_HIDE,
+    ARM_GOLDEN_DRAGON_HIDE,
 #endif
-    ARM_GOLD_DRAGON_ARMOUR,
+    ARM_GOLDEN_DRAGON_ARMOUR,
 #if TAG_MAJOR_VERSION == 34
     ARM_SWAMP_DRAGON_HIDE,
 #endif
@@ -181,6 +181,7 @@ enum brand_type // item_def.special
 #endif
     SPWPN_WEAKNESS,
     SPWPN_VULNERABILITY,
+    SPWPN_FOUL_FLAME,
     SPWPN_DEBUG_RANDART,
     NUM_SPECIAL_WEAPONS,
 };
@@ -225,7 +226,7 @@ enum jewellery_type
     RING_WIZARDRY,
     RING_MAGICAL_POWER,
     RING_FLIGHT,
-    RING_LIFE_PROTECTION,
+    RING_POSITIVE_ENERGY,
     RING_WILLPOWER,
     RING_FIRE,
     RING_ICE,
@@ -320,6 +321,7 @@ enum misc_item_type
 #endif
     MISC_TIN_OF_TREMORSTONES,
     MISC_CONDENSER_VANE,
+    MISC_GRAVITAMBOURINE,
 
     NUM_MISCELLANY,
     MISC_DECK_UNKNOWN = NUM_MISCELLANY,
@@ -351,6 +353,7 @@ const vector<misc_item_type> misc_types =
 #endif
     MISC_TIN_OF_TREMORSTONES,
     MISC_CONDENSER_VANE,
+    MISC_GRAVITAMBOURINE,
 };
 
 enum missile_type
@@ -369,6 +372,8 @@ enum missile_type
     MI_THROWING_NET,
     MI_BOOMERANG,
 
+    MI_SLUG,
+
     NUM_MISSILES,
     MI_NONE             // was MI_EGGPLANT... used for launch type detection
 };
@@ -578,7 +583,7 @@ enum stave_type
     STAFF_AIR,
     STAFF_EARTH,
 #if TAG_MAJOR_VERSION == 34
-    STAFF_CHANNELING,
+    STAFF_CHANNELLING,
 #endif
     NUM_STAVES,
 };
@@ -793,6 +798,7 @@ enum wand_type
     WAND_LIGHT,
     WAND_QUICKSILVER,
     WAND_ROOTS,
+    WAND_WARPING,
     NUM_WANDS
 };
 
@@ -834,6 +840,7 @@ enum item_set_type
     ITEM_SET_ALLY_SCROLLS,
     ITEM_SET_AREA_MISCELLANY,
     ITEM_SET_ALLY_MISCELLANY,
+    ITEM_SET_CONTROL_MISCELLANY,
     NUM_ITEM_SET_TYPES
 };
 
@@ -850,3 +857,12 @@ enum talisman_type
     TALISMAN_FLUX,
     NUM_TALISMANS,
 };
+
+enum special_gizmo_type
+{
+    SPGIZMO_NORMAL,
+    SPGIZMO_SPELLMOTOR,
+    SPGIZMO_GADGETEER,
+    SPGIZMO_PARRYREV,
+    SPGIZMO_AUTODAZZLE,
+};
