@@ -17,13 +17,15 @@ int create_item_named(string name, coord_def pos, string *error);
 
 int items(bool allow_uniques, object_class_type force_class, int force_type,
           int item_level, int force_ego = 0, int agent = NO_AGENT,
-          string custom_name = "");
+          string custom_name = "",
+          CrawlHashTable const *fixed_props = nullptr);
 
 void item_colour(item_def &item);
 
 jewellery_type get_random_ring_type();
 jewellery_type get_random_amulet_type();
 misc_item_type get_misc_item_type(int force_type, bool exclude = true);
+void handle_generated_misc(misc_item_type typ);
 void item_set_appearance(item_def &item);
 
 bool is_weapon_brand_ok(int type, int brand, bool strict);
