@@ -4,6 +4,7 @@
 
 #include "attack.h"
 #include "fight.h"
+#include "item-prop-enum.h" // vorpal_damage_type
 #include "random-var.h"
 #include "tag-version.h"
 
@@ -21,15 +22,19 @@ enum unarmed_attack_type
     UNAT_PSEUDOPODS,
     UNAT_TENTACLES,
     UNAT_MAW,
+    UNAT_EXECUTIONER_BLADE,
     UNAT_FIRST_ATTACK = UNAT_CONSTRICT,
-    UNAT_LAST_ATTACK = UNAT_MAW,
+    UNAT_LAST_ATTACK = UNAT_EXECUTIONER_BLADE,
     NUM_UNARMED_ATTACKS,
 };
 
 class melee_attack : public attack
 {
 public:
-    // mon_attack_def stuff
+    // For monsters, attack_number indicates which of their attacks is used.
+    // For players, attack_number is used when using off-hand weapons, and
+    // indicates whether this is their primary attack or their follow-up.
+    // Note that we randomize which hand is used for the primary attack.
     int       attack_number;
     int       effective_attack_number;
 
@@ -44,13 +49,14 @@ public:
     wu_jian_attack_type wu_jian_attack;
     int wu_jian_number_of_targets;
     coord_def attack_position;
+    item_def *mutable_wpn;
 
 public:
     melee_attack(actor *attacker, actor *defender,
-                 int attack_num = -1, int effective_attack_num = -1,
-                 bool is_cleaving = false);
+                 int attack_num = 0, int effective_attack_num = 0);
+    void set_weapon(item_def *weapon, bool offhand = false);
 
-    // Applies attack damage and other effects.
+    bool launch_attack_set(bool allow_rev = true);
     bool attack();
     int calc_to_hit(bool random) override;
     int post_roll_to_hit_modifiers(int mhit, bool random) override;
@@ -59,9 +65,12 @@ public:
 
     static void chaos_affect_actor(actor *victim);
 
+    bool player_do_aux_attack(unarmed_attack_type atk);
+
 private:
     /* Attack phases */
     bool handle_phase_attempted() override;
+    bool handle_phase_blocked() override;
     bool handle_phase_dodged() override;
     bool handle_phase_hit() override;
     bool handle_phase_damaged() override;
@@ -83,9 +92,13 @@ private:
 
     void rot_defender(int amount);
 
-    bool consider_decapitation(int damage_done, int damage_type = -1);
-    bool attack_chops_heads(int damage_done, int damage_type);
-    void decapitate(int dam_type);
+    bool consider_decapitation(int damage_done);
+    bool attack_chops_heads(int damage_done);
+    void decapitate();
+
+    bool run_attack_set();
+    bool swing_with(item_def &weapon, bool offhand);
+    void force_cleave(item_def &weapon, coord_def target);
 
     /* Axe cleaving */
     void cleave_setup();
@@ -100,9 +113,7 @@ private:
     /* Mutation Effects */
     void do_spines();
     void do_passive_freeze();
-#if TAG_MAJOR_VERSION == 34
-    void do_passive_heat();
-#endif
+    void do_foul_flame();
     void emit_foul_stench();
 
     /* Divine Effect */
@@ -134,12 +145,13 @@ private:
     void mons_do_eyeball_confusion();
     void mons_do_tendril_disarm();
     void apply_black_mark_effects();
+    void apply_sign_of_ruin_effects();
     void do_ooze_engulf();
+    void try_parry_disarm();
 private:
     // Player-attack specific stuff
     // Auxiliary unarmed attacks.
-    bool player_aux_unarmed();
-    bool player_gets_aux_punch();
+    bool player_do_aux_attacks();
     void player_aux_setup(unarmed_attack_type atk);
     bool player_aux_test_hit();
     bool player_aux_apply(unarmed_attack_type atk);
@@ -151,17 +163,21 @@ private:
     void player_exercise_combat_skills() override;
     bool player_monattk_hit_effects();
     void attacker_sustain_passive_damage();
-    int  staff_damage(skill_type skill);
+    int  staff_damage(stave_type staff) const;
+    string staff_message(stave_type staff, int damage) const;
     bool apply_staff_damage();
     void player_stab_check() override;
     bool player_good_stab() override;
     void player_announce_aux_hit();
     string charge_desc();
+    string weapon_desc();
     void player_warn_miss();
     void player_weapon_upsets_god();
     bool bad_attempt();
-    bool player_unrand_bad_attempt(bool check_only = false);
+    bool player_unrand_bad_attempt(const item_def *offhand,
+                                   bool check_only = false);
     void _defender_die();
+    void handle_spectral_brand();
 
     // Added in, were previously static methods of fight.cc
     bool _extra_aux_attack(unarmed_attack_type atk);
@@ -169,8 +185,22 @@ private:
                                      bool needs_bite_msg = false);
     bool _vamp_wants_blood_from_monster(const monster* mon);
 
-    bool can_reach();
+    bool can_reach(int dist);
+
+    item_def *offhand_weapon() const;
+
+    // XXX: set up a copy constructor instead?
+    void copy_to(melee_attack &other);
+
+    vorpal_damage_type damage_type;
+
+    // Is a special stab against a sleeping monster by a Dithmenos player
+    // shadow (affects messaging).
+    bool is_shadow_stab;
 };
 
 string aux_attack_desc(unarmed_attack_type unat, int force_damage = -1);
 string mut_aux_attack_desc(mutation_type mut);
+vector<string> get_player_aux_names();
+
+bool coglin_spellmotor_attack();
