@@ -49,7 +49,6 @@ static const vector<monster_info_flag_name> monster_info_flag_names = {
     { MB_STILL_WINDS, "stilling wind", "stilling the winds", "stilling wind"},
     { MB_SILENCING, "silencing", "radiating silence", "silencing"},
     { MB_READY_TO_HOWL, "can howl", "ready to howl", "can howl"},
-    { MB_BRILLIANCE_AURA, "brilliance aura", "brilliance aura", "brilliance auras"},
     { MB_VORTEX, "vortex", "surrounded by a freezing vortex", "vortices"},
     { MB_VORTEX_COOLDOWN, "gusty", "surrounded by restless winds", "gusty"},
     { MB_FRENZIED, "frenzied", "frenzied and wild", "frenzied"},
@@ -58,7 +57,7 @@ static const vector<monster_info_flag_name> monster_info_flag_names = {
     { MB_REPEL_MSL, "repels missiles", "repelling missiles", "repel missiles"},
     { MB_TOXIC_RADIANCE, "toxic aura", "radiating toxic energy", "toxic auras"},
     { MB_CONCENTRATE_VENOM, "curare", "concentrated venom", "curare" },
-    { MB_BLACK_MARK, "black mark", "absorbing vital energies", "black marks"},
+    { MB_SIGN_OF_RUIN, "sign of ruin", "marked with the sign of ruin", "signs of ruin"},
     { MB_RESISTANCE, "resistant", "unusually resistant", "resistant"},
     { MB_INVISIBLE, "invisible", "slightly transparent", "invisible"},
     { MB_REGENERATION, "regenerating", "regenerating", "regenerating"},
@@ -77,7 +76,7 @@ static const vector<monster_info_flag_name> monster_info_flag_names = {
     { MB_FIRE_CHAMPION, "flame-wreathed", "flame-wreathed", "flame-wreathed"},
     { MB_SILENCE_IMMUNE, "unsilenced", "unaffected by silence", "unsilenced" },
     // Vulnerabilities
-    { MB_POSSESSABLE, "possessable", "possessable", "possessable"},
+    { MB_POSSESSABLE, "soul-gripped", "soul-gripped", "soul-gripped"},
     { MB_CAUGHT, "caught", "entangled in a net", "caught"},
     { MB_WEBBED, "webbed", "entangled in a web", "webbed"},
     { MB_PARALYSED, "paralysed", "paralysed", "paralysed"},
@@ -142,4 +141,14 @@ static const vector<monster_info_flag_name> monster_info_flag_names = {
     { MB_BULLSEYE_TARGET, "bullseye target", "targeted by your dimensional bullseye", "bullseye target" },
     { MB_VITRIFIED, "vitrified", "fragile as glass", "vitrified" },
     { MB_CURSE_OF_AGONY, "agonized", "cursed with the promise of agony", "cursed with agony" },
+    { MB_RETREATING, "retreating", "retreating", "retreating"},
+    { MB_TOUCH_OF_BEOGH, "divinely empowered", "empowered by the touch of Beogh", "divinely empowered"},
+    { MB_AWAITING_RECRUITMENT, "anointable", "ready to become your apostle", "anointable"},
+    { MB_VENGEANCE_TARGET, "target of vengeance", "target of orcish vengeance", "targets of vengeance"},
+    { MB_MAGNETISED, "magnetised", "covered in magnetic dust", "magnetised"},
+    { MB_RIMEBLIGHT, "rimeblight", "afflicted by rimeblight", "rimeblight"},
+    { MB_SHADOWLESS, "shadowless", "missing a shadow", "shadowless"},
+    { MB_FROZEN_IN_TERROR, "paralysed with fear", "paralysed with fear", "paralysed with fear"},
+    { MB_SOUL_SPLINTERED, "soul-splintered", "soul-splintered", "soul-splintered"},
+    { MB_ENGULFING_PLAYER, "engulfing you", "engulfing you", "engulfing you"},
 };
