@@ -7,7 +7,7 @@
 #define POP_DEPTHS \
 { /* Depths (OOD cap: 13)*/ \
   { -3,  3,  250, SEMI, MONS_RAKSHASA },\
-  { -3,  3,  100, SEMI, MONS_WIZARD },\
+  { -3,  3,  100, SEMI, MONS_OCCULTIST },\
   { -3,  3,  100, SEMI, MONS_NECROMANCER },\
   { -3,  9,  250, PEAK, MONS_FIRE_GIANT },\
   { -3,  9,  260, PEAK, MONS_FROST_GIANT },\
@@ -33,7 +33,7 @@
   {  0, 13,   60, SEMI, MONS_SPHINX },\
   {  1,  7,  135, SEMI, MONS_SPRIGGAN_AIR_MAGE },\
   {  1,  7,  185, SEMI, MONS_SPRIGGAN_BERSERKER },\
-  {  1,  9,   25, FLAT, MONS_FLOATING_EYE },\
+  {  1,  9,   45, FLAT, MONS_GLASS_EYE },\
   {  2, 13,   45, FLAT, MONS_SPRIGGAN_DEFENDER },\
   {  3, 13,   80, SEMI, MONS_TENTACLED_MONSTROSITY },\
   {  3, 13,   40, FLAT, MONS_STORM_DRAGON },\
@@ -79,14 +79,14 @@ static const vector<pop_entry> population[] =
   {  1,  8,  200, PEAK, MONS_GNOLL },
 
 // Shouldn't show up on D:1.
-  {  2,  5,  500, FALL, MONS_WORM },
+  {  2,  5,  500, FALL, MONS_RIBBON_WORM },
   {  2,  6, 1000, FLAT, MONS_ADDER },
   {  3,  8, 1000, FLAT, MONS_ORC },
+  {  3,  7,  400, PEAK, MONS_BOMBARDIER_BEETLE},
 
 // These historically don't get kills after D:7ish.
   {  4,  7,  800, PEAK, MONS_SCORPION },
   {  4,  7, 1000, PEAK, MONS_HOUND },
-  {  4,  7,  350, PEAK, MONS_UFETUBUS },
   {  4,  7,  500, PEAK, MONS_OGRE },
   {  4,  7, 1000, PEAK, MONS_IGUANA },
   {  4,  7,  350, PEAK, MONS_PHANTOM },
@@ -98,12 +98,14 @@ static const vector<pop_entry> population[] =
   {  4, 12,  150, PEAK, MONS_ORC_PRIEST },
   {  4, 14,  500, PEAK, MONS_ORC_WIZARD },
   {  4, 14,  350, PEAK, MONS_HOWLER_MONKEY },
+  {  4,  7,  350, PEAK, MONS_UFETUBUS },
+  {  4,  7,  300, PEAK, MONS_SHADOW_IMP },
   {  5,  8,  500, PEAK, MONS_WHITE_IMP },
-  {  5,  8,  300, PEAK, MONS_SHADOW_IMP },
   {  5,  8,  500, PEAK, MONS_ICE_BEAST },
   {  5,  9,  400, PEAK, MONS_WATER_MOCCASIN },
   {  5, 13,  600, PEAK, MONS_CENTAUR },
   {  5, 13,  200, PEAK, MONS_GNOLL_SERGEANT },
+  {  5,  9,  100, PEAK, MONS_MARROWCUDA },
 
   {  6,  8,  500, PEAK, MONS_SKY_BEAST },
   {  6,  9, 1000, PEAK, MONS_BULLFROG },
@@ -117,7 +119,7 @@ static const vector<pop_entry> population[] =
   {  8, 12,  800, PEAK, MONS_YAK },
   {  8, 14,  350, PEAK, MONS_ACID_DRAGON },
   {  8, 14,  600, PEAK, MONS_TWO_HEADED_OGRE },
-  {  9, 13,  350, PEAK, MONS_SHADOW },
+  {  9, 13,  350, PEAK, MONS_SHADOWGHAST },
   {  9, 14,  200, PEAK, MONS_KOBOLD_DEMONOLOGIST },
   {  9, 14,  500, PEAK, MONS_WRAITH },
   {  9, 14,  800, PEAK, MONS_TROLL },
@@ -137,25 +139,26 @@ static const vector<pop_entry> population[] =
   { 13, 20,  150, SEMI, MONS_MELIAI },
   { 13, 21,  192, PEAK, MONS_DEATH_KNIGHT },
   { 13, 21,   89, PEAK, MONS_RAKSHASA },
-  { 13, 23,  540, SEMI, MONS_HYDRA },
+  { 13, 23,  530, SEMI, MONS_HYDRA },
   { 13, 17,  925, FLAT, MONS_CENTAUR_WARRIOR },
   { 18, 23,  925, FALL, MONS_CENTAUR_WARRIOR },
+  { 13, 18,   70, FLAT, MONS_ARCANIST },
   { 13, 19,  365, SEMI, MONS_TENGU_WARRIOR },
   { 13, 19,  205, SEMI, MONS_TENGU_CONJURER },
   { 13, 19,  285, SEMI, MONS_EFREET },
   { 13, 23,  675, SEMI, MONS_SHAPESHIFTER },
   { 13, 27,   89, FALL, MONS_CATOBLEPAS },
-  { 14, 19,  140, FLAT, MONS_WIZARD },
-  { 14, 22,  260, SEMI, MONS_FREEZING_WRAITH },
+  { 13, 22,  245, SEMI, MONS_LAUGHING_SKULL },
+  { 14, 22,  240, SEMI, MONS_FREEZING_WRAITH },
   { 14, 22,  270, SEMI, MONS_SIMULACRUM },
   { 14, 22,  115, SEMI, MONS_BOGGART },
   { 14, 27,  192, PEAK, MONS_FIRE_DRAGON },
+  { 15, 19,   70, FLAT, MONS_OCCULTIST },
   { 15, 20,  335, SEMI, MONS_YAKTAUR },
   { 15, 24,  345, FALL, MONS_ORC_KNIGHT },
   { 15, 25,  315, FALL, MONS_OGRE_MAGE },
   { 15, 25,  275, PEAK, MONS_WOLF_SPIDER },
   { 16, 24,   89, PEAK, MONS_REDBACK },
-  { 16, 24,   25, PEAK, MONS_SHADOW_WRAITH },
   { 16, 27,   92, PEAK, MONS_ICE_DRAGON },
   { 17, 23,   89, PEAK, MONS_VERY_UGLY_THING },
   { 17, 24,  102, SEMI, MONS_GREAT_ORB_OF_EYES },
@@ -183,13 +186,13 @@ static const vector<pop_entry> population[] =
   { 23, 29,   25, PEAK, MONS_HELL_KNIGHT },
   { 23, 31,   89, PEAK, MONS_PHANTASMAL_WARRIOR },
   { 23, 27,  136, FLAT, MONS_GLOWING_SHAPESHIFTER },
+  { 23, 27,   22, RISE, MONS_THERMIC_DYNAMO },
   { 24, 27,   22, RISE, MONS_DEEP_ELF_ANNIHILATOR },
   { 24, 27,   22, RISE, MONS_DEEP_ELF_HIGH_PRIEST },
   { 24, 27,   22, RISE, MONS_VAMPIRE_KNIGHT },
   { 24, 27,  136, RISE, MONS_TENGU_REAVER },
   { 24, 27,    8, RISE, MONS_DANCING_WEAPON },
   { 24, 27,   25, FLAT, MONS_WAR_GARGOYLE },
-  { 25, 27,   12, RISE, MONS_CRYSTAL_GUARDIAN },
   { 27, 27,   18, FLAT, MONS_SHADOW_DRAGON },
   { 27, 27,    8, FLAT, MONS_IRON_DRAGON },
 },
@@ -198,22 +201,21 @@ static const vector<pop_entry> population[] =
 },
 
 { // Orcish Mines
-  {  1,  4,  192, FLAT, MONS_ORC_WARRIOR },
-  {  1,  4,   25, FLAT, MONS_ORC_PRIEST },
-  {  1,  4,   25, FLAT, MONS_ORC_WIZARD },
-  {  1,  4,   25, FLAT, MONS_ORC_KNIGHT },
+  {  1,  4,  190, FLAT, MONS_ORC_WARRIOR },
+  {  1,  4,   24, FLAT, MONS_ORC_PRIEST },
+  {  1,  4,   24, FLAT, MONS_ORC_WIZARD },
+  {  1,  4,   24, FLAT, MONS_ORC_KNIGHT },
   {  1,  4,    8, FLAT, MONS_ORC_HIGH_PRIEST },
-  {  1,  4,   25, FLAT, MONS_ORC_SORCERER },
+  {  1,  4,   24, FLAT, MONS_ORC_SORCERER },
   {  1,  4,    8, FLAT, MONS_ORC_WARLORD },
   {  1,  4,   40, FLAT, MONS_WARG },
-  {  1,  4,    3, FLAT, MONS_GNOLL },
-  {  1,  4,    2, FLAT, MONS_GNOLL_BOUDA },
-  {  1,  4,    1, FLAT, MONS_GNOLL_SERGEANT },
-  {  1,  4,   89, FLAT, MONS_OGRE },
+  {  1,  4,   18, FLAT, MONS_OBSIDIAN_BAT },
+  {  1,  4,   32, FLAT, MONS_KOBOLD_BLASTMINER },
+  {  1,  4,   80, FLAT, MONS_OGRE },
   {  1,  4,    8, FLAT, MONS_TWO_HEADED_OGRE },
-  {  1,  4,    1, FLAT, MONS_OGRE_MAGE },
-  {  1,  4,   40, FLAT, MONS_TROLL },
-  {  1,  4,    8, FLAT, MONS_CYCLOPS },
+  {  1,  4,    2, FLAT, MONS_OGRE_MAGE },
+  {  1,  4,   32, FLAT, MONS_TROLL },
+  {  1,  4,    4, FLAT, MONS_CYCLOPS },
   {  1,  4,    4, FLAT, MONS_ETTIN },
   {  3,  4,    4, FLAT, MONS_STONE_GIANT },
 },
@@ -221,15 +223,16 @@ static const vector<pop_entry> population[] =
 { // Elven Halls (OOD cap: 7)
   {  1,  5,   50, FLAT, MONS_ORC_HIGH_PRIEST },
   {  1,  6,   50, FLAT, MONS_ORC_SORCERER },
-  {  1,  7, 1385, FLAT, MONS_DEEP_ELF_AIR_MAGE },
-  {  1,  7, 1385, FLAT, MONS_DEEP_ELF_FIRE_MAGE },
-  {  1,  7, 1385, FLAT, MONS_DANCING_WEAPON },
+  {  1,  7, 1360, FLAT, MONS_DEEP_ELF_AIR_MAGE },
+  {  1,  7, 1360, FLAT, MONS_DEEP_ELF_FIRE_MAGE },
+  {  1,  7, 1360, FLAT, MONS_DANCING_WEAPON },
   {  1,  7,  925, FLAT, MONS_DEEP_ELF_KNIGHT },
   {  1,  7,  925, FLAT, MONS_DEEP_ELF_ARCHER },
-  {  1,  7,  345, FLAT, MONS_FIRE_ELEMENTAL },
-  {  1,  7,  345, FLAT, MONS_WATER_ELEMENTAL },
-  {  1,  7,  345, FLAT, MONS_AIR_ELEMENTAL },
-  {  1,  7,  345, FLAT, MONS_EARTH_ELEMENTAL },
+  {  1,  7,  320, FLAT, MONS_FIRE_ELEMENTAL },
+  {  1,  7,  320, FLAT, MONS_WATER_ELEMENTAL },
+  {  1,  7,  320, FLAT, MONS_AIR_ELEMENTAL },
+  {  1,  7,  320, FLAT, MONS_EARTH_ELEMENTAL },
+  { -1,  8,  340, SEMI, MONS_THERMIC_DYNAMO },
   {  1,  8,   65, SEMI, MONS_DEEP_ELF_SORCERER },
   {  1,  7,   40, SEMI, MONS_DEEP_ELF_ANNIHILATOR },
   {  1,  8,   65, SEMI, MONS_DEEP_ELF_DEMONOLOGIST },
@@ -259,7 +262,6 @@ static const vector<pop_entry> population[] =
 #endif
 
 { // Lair (OOD cap: 11)
-  {  0,  4, 4500, FLAT, MONS_NO_MONSTER }, // Roughly old D:1-4 chaff weight
   { -2,  4,  100, SEMI, MONS_BLACK_BEAR },
   { -2,  6,  310, SEMI, MONS_BASILISK },
   { -1,  3,   24, PEAK, MONS_SCORPION },
@@ -371,6 +373,7 @@ static const vector<pop_entry> population[] =
   {  1,  7,  135, PEAK, MONS_MERFOLK_JAVELINEER },
   {  1,  7,  110, PEAK, MONS_ALLIGATOR_SNAPPING_TURTLE },
   {  2,  4,  190, SEMI, MONS_SATYR },
+  {  3,  7,   30, PEAK, MONS_FORMLESS_JELLYFISH },
 },
 
 { // Snake Pit
@@ -454,10 +457,11 @@ static const vector<pop_entry> population[] =
   {  1,  5,  515, FLAT, MONS_QUICKSILVER_OOZE },
   {  1,  5,  515, FLAT, MONS_SHINING_EYE },
   {  1,  5,  200, FLAT, MONS_GOLDEN_EYE },
+  {  1,  5,  200, FLAT, MONS_FORMLESS_JELLYFISH },
   {  1,  8,  300, SEMI, MONS_EYE_OF_DEVASTATION },
-  {  1,  8,  390, SEMI, MONS_GREAT_ORB_OF_EYES },
+  {  1,  8,  265, SEMI, MONS_GREAT_ORB_OF_EYES },
   {  2,  5,  100, RISE, MONS_GLOWING_ORANGE_BRAIN },
-  {  2,  8,   50, SEMI, MONS_FLOATING_EYE },
+  {  2,  8,  315, SEMI, MONS_GLASS_EYE },
 },
 
 { // The Vaults (OOD cap: 12)
@@ -480,6 +484,7 @@ static const vector<pop_entry> population[] =
   // Harder enemies:
   {  1,  4, 1250, FLAT, MONS_VAULT_SENTINEL },
   {  1,  4,  150, FLAT, MONS_ENTROPY_WEAVER },
+  {  1,  4,  100, FLAT, MONS_POLTERGUARDIAN },
   {  1,  4, 1200, FLAT, MONS_IRONBOUND_CONVOKER },
   {  1,  4, 1200, FLAT, MONS_IRONBOUND_PRESERVER },
   {  1,  4,  725, FLAT, MONS_IRONBOUND_FROSTHEART },
@@ -491,10 +496,10 @@ static const vector<pop_entry> population[] =
   {  1,  4,  325, RISE, MONS_GREAT_ORB_OF_EYES },
   {  1,  4,   50, FLAT, MONS_GLOWING_ORANGE_BRAIN },
   {  1,  4,   50, RISE, MONS_GLOWING_ORANGE_BRAIN },
-  {  1,  4,  150, FLAT, MONS_VERY_UGLY_THING },
-  {  1,  4,  100, RISE, MONS_VERY_UGLY_THING },
-  {  1,  4,  350, FLAT, MONS_WIZARD },
-  {  1,  4,  350, RISE, MONS_WIZARD },
+  {  1,  4,  150, FLAT, MONS_FORMLESS_JELLYFISH },
+  {  1,  4,  100, RISE, MONS_FORMLESS_JELLYFISH },
+  {  1,  4,  350, FLAT, MONS_ARCANIST },
+  {  1,  4,  350, RISE, MONS_ARCANIST },
   {  1,  4,   75, FLAT, MONS_NECROMANCER },
   {  1,  4,   75, RISE, MONS_NECROMANCER },
   {  1,  4,  500, FLAT, MONS_YAKTAUR_CAPTAIN },
@@ -542,6 +547,7 @@ static const vector<pop_entry> population[] =
   {  5, 12,  330, FALL, MONS_VERY_UGLY_THING },
   {  5, 12,  260, FALL, MONS_SPHINX },
   {  5, 12,  260, FALL, MONS_WAR_GARGOYLE },
+  {  5, 12,  300, FALL, MONS_POLTERGUARDIAN },
   // V:5 chaff we should really get rid of
   {  5,  5,  650, FLAT, MONS_ORC_KNIGHT },
   {  5,  5,   60, FLAT, MONS_REDBACK },
@@ -552,12 +558,12 @@ static const vector<pop_entry> population[] =
   {  5,  5,   60, FLAT, MONS_GREAT_ORB_OF_EYES },
 
   // Out-of-depth enemies.
-  // These total 845 weight, to avoid being too common
+  // These total 860 weight, to avoid being too common
   // relative to the 'normal' enemies.
   {  5, 12,  190, RISE, MONS_TITAN },
-  {  5, 12,  170, RISE, MONS_SHADOW_DRAGON },
-  {  5, 12,  120, RISE, MONS_TENTACLED_MONSTROSITY },
-  {  5, 12,  110, RISE, MONS_STORM_DRAGON },
+  {  5, 12,  155, RISE, MONS_SHADOW_DRAGON },
+  {  5, 12,  150, RISE, MONS_UNDYING_ARMOURY },
+  {  5, 12,  100, RISE, MONS_STORM_DRAGON },
   {  5, 12,   70, RISE, MONS_GOLDEN_DRAGON },
   {  5, 12,   50, RISE, MONS_QUICKSILVER_DRAGON },
   {  5, 12,   45, RISE, MONS_IRON_DRAGON },
@@ -572,7 +578,7 @@ static const vector<pop_entry> population[] =
 #endif
 
 { // Crypt (OOD cap: 5)
-  { -4,  3,   45, PEAK, MONS_FLYING_SKULL },
+  { -4,  3,  125, SEMI, MONS_LAUGHING_SKULL },
   { -4,  3,   75, SEMI, MONS_NECROMANCER },
   { -3,  3,   75, SEMI, MONS_DEATH_KNIGHT },
   { -2,  3,  125, SEMI, MONS_WRAITH },
@@ -598,11 +604,10 @@ static const vector<pop_entry> population[] =
 },
 
 { // Tomb (OOD cap: 5)
-  {  1,  5,  200, FLAT, MONS_SKELETON },
-  {  1,  5,  185, FLAT, MONS_ZOMBIE },
-  {  1,  5,   50, SEMI, MONS_SIMULACRUM },
-  {  1,  5, 1000, FLAT, MONS_MUMMY },
-  {  1,  5,  230, FLAT, MONS_FLYING_SKULL },
+  {  1,  5,  300, FLAT, MONS_SKELETON },
+  {  1,  5,  235, FLAT, MONS_ZOMBIE },
+  {  1,  5,   80, SEMI, MONS_SIMULACRUM },
+  {  1,  5, 1050, FLAT, MONS_MUMMY },
   {  1,  5,  600, FLAT, MONS_GUARDIAN_MUMMY },
   { -1,  9,  335, SEMI, MONS_MUMMY_PRIEST },
   {  4,  5,    3, RISE, MONS_ROYAL_MUMMY },
@@ -621,7 +626,7 @@ POP_DEPTHS,
   {  1,  1,   89, FLAT, MONS_HELL_RAT },
   {  1,  1,   89, FLAT, MONS_DEMONIC_CRAWLER },
   {  1,  1,   89, FLAT, MONS_HELL_HOUND },
-  {  1,  1,  136, FLAT, MONS_HELL_HOG },
+  {  1,  1,  145, FLAT, MONS_HELL_HOG },
   {  1,  1,  136, FLAT, MONS_HELL_KNIGHT },
   {  1,  1,  136, FLAT, MONS_NECROMANCER },
   {  1,  1,   25, FLAT, MONS_HELLEPHANT },
@@ -630,14 +635,13 @@ POP_DEPTHS,
   {  1,  1,   89, FLAT, MONS_ICE_DEVIL },
   {  1,  1,  515, FLAT, MONS_SUN_DEMON },
   {  1,  1,  515, FLAT, MONS_SOUL_EATER },
-  {  1,  1,  515, FLAT, MONS_REAPER },
-  {  1,  1,   89, FLAT, MONS_LOROCYPROCA },
+  {  1,  1,  545, FLAT, MONS_REAPER },
   {  1,  1,   25, FLAT, MONS_HELLION },
   {  1,  1,   25, FLAT, MONS_TORMENTOR },
   {  1,  1,   89, FLAT, MONS_RUST_DEVIL },
-  {  1,  1,   52, FLAT, MONS_GREEN_DEATH },
-  {  1,  1,   52, FLAT, MONS_BLIZZARD_DEMON },
-  {  1,  1,   52, FLAT, MONS_BALRUG },
+  {  1,  1,   89, FLAT, MONS_GREEN_DEATH },
+  {  1,  1,   62, FLAT, MONS_BLIZZARD_DEMON },
+  {  1,  1,   62, FLAT, MONS_BALRUG },
   {  1,  1,   52, FLAT, MONS_CACODEMON },
 },
 
@@ -749,22 +753,23 @@ POP_DEPTHS,
 { // Zot
   {  1,  5,  970, FLAT, MONS_MOTH_OF_WRATH },
   {  1,  5,  192, RISE, MONS_GHOST_MOTH },
-  {  1,  5,  113, FLAT, MONS_BLACK_DRACONIAN },
-  {  1,  5,  113, FLAT, MONS_YELLOW_DRACONIAN },
-  {  1,  5,  113, FLAT, MONS_GREEN_DRACONIAN },
-  {  1,  5,  113, FLAT, MONS_PURPLE_DRACONIAN },
-  {  1,  5,  113, FLAT, MONS_RED_DRACONIAN },
-  {  1,  5,  113, FLAT, MONS_WHITE_DRACONIAN },
-  {  1,  5,   70, FLAT, MONS_DRACONIAN_STORMCALLER },
-  {  1,  5,   70, FLAT, MONS_DRACONIAN_MONK },
-  {  1,  5,   70, FLAT, MONS_DRACONIAN_SHIFTER },
-  {  1,  5,   70, FLAT, MONS_DRACONIAN_ANNIHILATOR },
-  {  1,  5,   70, FLAT, MONS_DRACONIAN_KNIGHT },
-  {  1,  5,   70, FLAT, MONS_DRACONIAN_SCORCHER },
+  {  1,  5,  100, FLAT, MONS_BLACK_DRACONIAN },
+  {  1,  5,  100, FLAT, MONS_YELLOW_DRACONIAN },
+  {  1,  5,  100, FLAT, MONS_GREEN_DRACONIAN },
+  {  1,  5,  100, FLAT, MONS_PURPLE_DRACONIAN },
+  {  1,  5,  100, FLAT, MONS_RED_DRACONIAN },
+  {  1,  5,  100, FLAT, MONS_WHITE_DRACONIAN },
+  {  1,  5,   67, FLAT, MONS_DRACONIAN_STORMCALLER },
+  {  1,  5,   67, FLAT, MONS_DRACONIAN_MONK },
+  {  1,  5,   67, FLAT, MONS_DRACONIAN_SHIFTER },
+  {  1,  5,   67, FLAT, MONS_DRACONIAN_ANNIHILATOR },
+  {  1,  5,   67, FLAT, MONS_DRACONIAN_KNIGHT },
+  {  1,  5,   67, FLAT, MONS_DRACONIAN_SCORCHER },
   {  1,  5,  200, FLAT, MONS_QUICKSILVER_DRAGON },
   {  1,  5,  200, FLAT, MONS_SHADOW_DRAGON },
   { -4,  5,  515, RISE, MONS_STORM_DRAGON },
   {  1, 11,  365, SEMI, MONS_GOLDEN_DRAGON },
+  {  1,  5,  112, FLAT, MONS_PROTEAN_PROGENITOR },
   {  2,  8,   52, SEMI, MONS_KILLER_KLOWN },
   {  1,  5,  335, FLAT, MONS_DEATH_COB },
   {  1,  5,  150, RISE, MONS_CURSE_TOE },
@@ -802,13 +807,15 @@ POP_DEPTHS,
 { // Abyss
   // Sorted by 'home depth', somewhat arbitrarily defined.
   // Abyss:1
-  { -1,  6, 1500, FALL, MONS_ABOMINATION_SMALL },
+  { -1,  6, 1150, FALL, MONS_ABOMINATION_SMALL },
+  { -1,  6,  150, FALL, MONS_BRAIN_WORM },
+  { -1,  6,  200, FALL, MONS_WEEPING_SKULL },
 
   {  1,  4,   25, FALL, MONS_CRIMSON_IMP },
-  {  1,  4,  192, FALL, MONS_WHITE_IMP },
-  {  1,  4,  192, FALL, MONS_QUASIT },
-  {  1,  4,  192, FALL, MONS_UFETUBUS },
-  {  1,  4,  192, FALL, MONS_IRON_IMP },
+  {  1,  4,  180, FALL, MONS_WHITE_IMP },
+  {  1,  4,  180, FALL, MONS_QUASIT },
+  {  1,  4,  180, FALL, MONS_UFETUBUS },
+  {  1,  4,  180, FALL, MONS_IRON_IMP },
   {  1,  4,   25, FALL, MONS_SHADOW_IMP },
   {  1,  5,    8, FALL, MONS_RED_DEVIL },
   {  1,  5,   25, FALL, MONS_ICE_DEVIL },
@@ -816,14 +823,14 @@ POP_DEPTHS,
   {  1,  8,   73, FALL, MONS_HELLWING },
   {  1,  8,   91, FALL, MONS_SIXFIRHY },
   {  1,  5,  125, FALL, MONS_ORANGE_DEMON },
-  {  1,  8,  315, FALL, MONS_YNOXINUL },
+  {  1,  8,  300, FALL, MONS_YNOXINUL },
+  {  1,  6,   65, FALL, MONS_OBSIDIAN_BAT },
 
-  {  1,  5,  110, FALL, MONS_SKELETON },
-  {  1,  5,  104, FALL, MONS_ZOMBIE },
+  {  1,  5,  130, FALL, MONS_SKELETON },
+  {  1,  5,  124, FALL, MONS_ZOMBIE },
   {  1,  4,   35, FALL, MONS_WIGHT },
-  {  1,  8,   60, FALL, MONS_FLYING_SKULL },
-  {  1,  8,   60, FALL, MONS_WRAITH },
-  {  1,  8,   30, FALL, MONS_SHADOW },
+  {  1,  8,   70, FALL, MONS_WRAITH },
+  {  1,  8,   10, FALL, MONS_SHADOWGHAST },
   {  1,  8,   25, FALL, MONS_VAMPIRE },
 
   {  1,  5,    4, FALL, MONS_BASILISK },
@@ -880,10 +887,12 @@ POP_DEPTHS,
   {  4,  6,   18, FALL, MONS_FLAYED_GHOST },
   {  1,  6,    8, FLAT, MONS_HELL_KNIGHT },
   {  1,  6,    8, FLAT, MONS_NECROMANCER },
-  {  1,  6,    8, FLAT, MONS_WIZARD },
+  {  1,  6,    8, FLAT, MONS_OCCULTIST },
 
-  {  1,  5, 1000, FLAT, MONS_ABOMINATION_LARGE },
-  {  6,  7, 1000, FALL, MONS_ABOMINATION_LARGE },
+  {  1,  5,  150, FLAT, MONS_LAUGHING_SKULL },
+  {  6,  7,  150, FALL, MONS_LAUGHING_SKULL },
+  {  1,  5,  850, FLAT, MONS_ABOMINATION_LARGE },
+  {  6,  7,  850, FALL, MONS_ABOMINATION_LARGE },
   {  1,  5,   52, FLAT, MONS_VERY_UGLY_THING },
   {  6,  7,   52, FALL, MONS_VERY_UGLY_THING },
   {  1,  5,  300, FLAT, MONS_RAIJU },
@@ -922,21 +931,20 @@ POP_DEPTHS,
   {  6, 10,   10, RISE, MONS_NARGUN },
   {  6, 10,   10, RISE, MONS_PUTRID_MOUTH },
   // Generic
-  {  1,  7,  335, FLAT, MONS_WRETCHED_STAR },
-  {  1,  7,  335, FLAT, MONS_TENTACLED_STARSPAWN },
-  {  1,  7,  335, FLAT, MONS_ANCIENT_ZYME },
+  {  1,  7,  340, FLAT, MONS_WRETCHED_STAR },
+  {  1,  7,  340, FLAT, MONS_TENTACLED_STARSPAWN },
+  {  1,  7,  340, FLAT, MONS_ANCIENT_ZYME },
   {  1,  7,  335, FLAT, MONS_STARCURSED_MASS },
   {  1,  7,  335, FLAT, MONS_THRASHING_HORROR },
-  {  1,  7,   89, FLAT, MONS_LURKING_HORROR },
-  {  1,  7,   89, FLAT, MONS_APOCALYPSE_CRAB },
+  {  1,  7,   90, FLAT, MONS_LURKING_HORROR },
+  {  1,  7,   90, FLAT, MONS_APOCALYPSE_CRAB },
 
   {  1,  7,  225, FLAT, MONS_NEQOXEC },
   {  1,  7,  300, FLAT, MONS_SMOKE_DEMON },
-  {  1,  7,   25, FLAT, MONS_LOROCYPROCA },
   {  1,  7,   25, FLAT, MONS_HELLION },
   {  1,  7,   21, FLAT, MONS_TORMENTOR },
-  {  1,  7,   12, FLAT, MONS_REAPER },
-  {  1,  7,   22, FLAT, MONS_GREEN_DEATH },
+  {  1,  7,   15, FLAT, MONS_REAPER },
+  {  1,  7,   25, FLAT, MONS_GREEN_DEATH },
   {  1,  7,   22, FLAT, MONS_BLIZZARD_DEMON },
   {  1,  7,   22, FLAT, MONS_BALRUG },
   {  1,  7,   25, FLAT, MONS_CACODEMON },
@@ -944,7 +952,7 @@ POP_DEPTHS,
   {  1,  7,   10, FLAT, MONS_HELL_SENTINEL },
 
   {  1,  7,   23, FLAT, MONS_SHADOW_WRAITH },
-  {  1,  7,    8, FLAT, MONS_SILENT_SPECTRE },
+  {  1,  7,   10, FLAT, MONS_SILENT_SPECTRE },
   {  1,  7,    8, FLAT, MONS_PHANTASMAL_WARRIOR },
   {  1,  7,   80, FLAT, MONS_BONE_DRAGON },
   {  1,  7,    9, FLAT, MONS_REVENANT },
@@ -1007,10 +1015,9 @@ POP_DEPTHS,
   {  1,  1, 1000, FLAT, MONS_CACODEMON },
   {  1,  1,  335, FLAT, MONS_HELLION },
   {  1,  1,  335, FLAT, MONS_TORMENTOR },
-  {  1,  1,  335, FLAT, MONS_REAPER },
-  {  1,  1,  335, FLAT, MONS_SHADOW_DEMON },
-  {  1,  1,  535, FLAT, MONS_HELL_BEAST },
-  {  1,  1,  535, FLAT, MONS_LOROCYPROCA },
+  {  1,  1,  535, FLAT, MONS_REAPER },
+  {  1,  1,  535, FLAT, MONS_SHADOW_DEMON },
+  {  1,  1,  675, FLAT, MONS_SIN_BEAST },
   {  1,  1, 1000, FLAT, MONS_EXECUTIONER },
   {  1,  1,  335, FLAT, MONS_BRIMSTONE_FIEND },
   {  1,  1,  335, FLAT, MONS_ICE_FIEND },
@@ -1020,7 +1027,7 @@ POP_DEPTHS,
   {  1,  1,  675, FLAT, MONS_DEMONSPAWN_BLOOD_SAINT },
   {  1,  1,  675, FLAT, MONS_DEMONSPAWN_WARMONGER },
   {  1,  1,  675, FLAT, MONS_DEMONSPAWN_CORRUPTER },
-  {  1,  1,  675, FLAT, MONS_DEMONSPAWN_BLACK_SUN },
+  {  1,  1,  675, FLAT, MONS_DEMONSPAWN_SOUL_SCHOLAR },
   {  1,  1,   50, FLAT, MONS_ANGEL },
   {  1,  1,   40, FLAT, MONS_CHERUB },
   {  1,  1,   25, FLAT, MONS_DAEVA },
@@ -1046,13 +1053,16 @@ POP_DEPTHS,
   {  1,  1, 1000, FLAT, MONS_BAT },
   {  1,  1,  515, FLAT, MONS_BALL_PYTHON },
   {  1,  1,  515, FLAT, MONS_ADDER },
-  {  1,  1,  515, FLAT, MONS_WORM },
+  {  1,  1,  515, FLAT, MONS_RIBBON_WORM },
   {  1,  1,  515, FLAT, MONS_ENDOPLASM },
   {  1,  1,  515, FLAT, MONS_GIANT_COCKROACH },
+  {  1,  1,   55, FLAT, MONS_BRAIN_WORM },
+  {  1,  1,   55, FLAT, MONS_CROCODILE },
 },
 
 { // Ossuary
-  {  1,  1,   89, FLAT, MONS_MUMMY },
+  {  1,  1,   40, FLAT, MONS_WEEPING_SKULL },
+  {  1,  1,   90, FLAT, MONS_MUMMY },
   {  1,  1,  515, FLAT, MONS_SKELETON },
   {  1,  1,  515, FLAT, MONS_ZOMBIE },
 },
@@ -1115,6 +1125,24 @@ POP_DEPTHS,
 { // Arena
 },
 
+{ // Crucible
+  {  1,  1,  100, FLAT, MONS_ICE_DEVIL },
+  {  1,  1,  100, FLAT, MONS_ORANGE_DEMON },
+  {  1,  1,  100, FLAT, MONS_RUST_DEVIL },
+  {  1,  1,  100, FLAT, MONS_RED_DEVIL },
+  {  1,  1,   75, FLAT, MONS_HELLWING },
+  {  1,  1,  100, FLAT, MONS_SOUL_EATER },
+  {  1,  1,  120, FLAT, MONS_YNOXINUL },
+  {  1,  1,  120, FLAT, MONS_SMOKE_DEMON },
+  {  1,  1,  120, FLAT, MONS_SUN_DEMON },
+  {  1,  1,  120, FLAT, MONS_SIXFIRHY },
+  {  1,  1,  100, FLAT, MONS_BLIZZARD_DEMON },
+  {  1,  1,  100, FLAT, MONS_GREEN_DEATH },
+  {  1,  1,   65, FLAT, MONS_CACODEMON },
+  {  1,  1,   50, FLAT, MONS_BALRUG },
+  {  1,  1,   50, FLAT, MONS_EXECUTIONER },
+},
+
 };
 
 COMPILE_CHECK(ARRAYSZ(population) == NUM_BRANCHES);
@@ -1128,9 +1156,10 @@ static const vector<pop_entry> pop_generic_late_zombie =
   {  5,  15,  500, FLAT, MONS_ANACONDA },
   {  5,  15,  500, FLAT, MONS_NAGA_WARRIOR },
   {  5,  15,  500, FLAT, MONS_NAGARAJA },
-  // Tramplers
-  {  5,   8,  222, FALL, MONS_FIRE_DRAGON },
-  {  5,   8,  222, FALL, MONS_ICE_DRAGON },
+  // Draggers and tramplers
+  {  5,   8,  222, FALL, MONS_ALLIGATOR },
+  {  5,   8,  111, FALL, MONS_FIRE_DRAGON },
+  {  5,   8,  111, FALL, MONS_ICE_DRAGON },
   {  5,   8,  222, FALL, MONS_SWAMP_DRAGON },
   {  5,   8,  222, FLAT, MONS_SHADOW_DRAGON },
   {  5,   8,  222, FLAT, MONS_STORM_DRAGON },
@@ -1196,10 +1225,11 @@ static const vector<pop_entry> pop_generic_late_zombie =
 static const vector<pop_entry> population_water[] =
 {
     { // Dungeon water monsters
+      {  5,  13,   90, SEMI, MONS_MARROWCUDA },
       {  5,  16,   60, FLAT, MONS_ELECTRIC_EEL },
       {  7,  16,  185, PEAK, MONS_ELECTRIC_EEL },
       {  11, 27,  600, RISE, MONS_WATER_ELEMENTAL },
-      {  5,  22,  130, FLAT, MONS_NO_MONSTER },
+      {  5,  22,  110, FLAT, MONS_NO_MONSTER },
       {  9,  32,  250, SEMI, MONS_NO_MONSTER },
     },
     GENERIC_WATER_POP, // Temple
@@ -1265,6 +1295,7 @@ static const vector<pop_entry> population_water[] =
     GENERIC_WATER_POP, // Gauntlet
 #endif
     GENERIC_WATER_POP, // Arena
+    GENERIC_WATER_POP, // Crucible
 };
 COMPILE_CHECK(ARRAYSZ(population_water) == NUM_BRANCHES);
 
@@ -1272,14 +1303,13 @@ COMPILE_CHECK(ARRAYSZ(population_water) == NUM_BRANCHES);
   {  1,  27,  100, FLAT, MONS_FIRE_BAT },\
   {  1,  27,  100, FLAT, MONS_FIRE_ELEMENTAL },\
   {  1,  27,   50, FLAT, MONS_MOLTEN_GARGOYLE },\
-  {  1,  27,   50, FLAT, MONS_FIRE_VORTEX },\
   {  1,  27,  145, FLAT, MONS_LAVA_SNAKE },\
   {  1,  27,   15, FLAT, MONS_SALAMANDER },\
-  {  1,  27,  290, FLAT, MONS_NO_MONSTER },\
+  {  1,  27,  340, FLAT, MONS_NO_MONSTER },\
 }
 
 #define HELL_LAVA_POP {\
-  {  1,  8,   300, FALL, MONS_FIRE_VORTEX },\
+  {  1,  8,   300, FALL, MONS_NO_MONSTER }, \
   {  1,  7,    50, RISE, MONS_STOKER },\
   {  1,  7,    50, FLAT, MONS_CREEPING_INFERNO },\
   {  1,  7,   100, FLAT, MONS_NO_MONSTER },\
@@ -1290,7 +1320,6 @@ COMPILE_CHECK(ARRAYSZ(population_water) == NUM_BRANCHES);
 #define DEPTHS_LAVA_POP {\
   {  1,  6,   22, FALL, MONS_FIRE_ELEMENTAL },\
   {  1,  6,   22, FALL, MONS_FIRE_BAT },\
-  {  1,  6,   11, FALL, MONS_FIRE_VORTEX },\
   {  1,  6,   11, FALL, MONS_MOLTEN_GARGOYLE },\
   {  1,  6,   60, FLAT, MONS_SALAMANDER },\
   {  1,  8,   85, SEMI, MONS_SALAMANDER_MYSTIC },\
@@ -1303,9 +1332,9 @@ static const vector<pop_entry> population_lava[] =
 {
     { // Dungeon lava monsters
       {  7,  27,  145, FLAT, MONS_LAVA_SNAKE },
-      {  11, 27,  290, RISE, MONS_FIRE_ELEMENTAL },
+      {  11, 27,  360, RISE, MONS_FIRE_ELEMENTAL },
       {  11, 27,  145, RISE, MONS_MOLTEN_GARGOYLE },
-      {  11, 27,  145, RISE, MONS_FIRE_VORTEX },
+      {  11, 27,   75, RISE, MONS_NO_MONSTER },
       {  7,  27,  290, FLAT, MONS_NO_MONSTER },
     },
     GENERIC_LAVA_POP, // Temple
@@ -1368,6 +1397,7 @@ static const vector<pop_entry> population_lava[] =
     GENERIC_LAVA_POP, // Gauntlet
 #endif
     GENERIC_LAVA_POP, // Arena
+    GENERIC_LAVA_POP, // Crucible
 };
 
 COMPILE_CHECK(ARRAYSZ(population_lava) == NUM_BRANCHES);
