@@ -134,13 +134,13 @@ static const mon_spellbook mspell_list[] =
        { SPELL_SLOW, 12, MON_SPELL_WIZARD },
        { SPELL_CONFUSE, 12, MON_SPELL_WIZARD },
        { SPELL_REPEL_MISSILES, 12, MON_SPELL_WIZARD },
-       { SPELL_BLINK_AWAY, 36, MON_SPELL_WIZARD | MON_SPELL_SHORT_RANGE },
+       { SPELL_BLINK_RANGE, 36, MON_SPELL_WIZARD | MON_SPELL_SHORT_RANGE },
       }
     },
 
     {  MST_DEEP_ELF_ELEMENTALIST,
       {
-       { SPELL_FREEZING_CLOUD, 22, MON_SPELL_WIZARD },
+       { SPELL_HOARFROST_CANNONADE, 33, MON_SPELL_WIZARD },
        { SPELL_FIREBALL, 11, MON_SPELL_WIZARD },
        { SPELL_LRD, 33, MON_SPELL_WIZARD },
        { SPELL_REPEL_MISSILES, 11, MON_SPELL_WIZARD },
@@ -159,7 +159,7 @@ static const mon_spellbook mspell_list[] =
 
     {  MST_DEEP_ELF_HIGH_PRIEST,
       {
-       { SPELL_AURA_OF_BRILLIANCE, 25, MON_SPELL_PRIEST },
+       { SPELL_PRAYER_OF_BRILLIANCE, 25, MON_SPELL_PRIEST },
        { SPELL_CALL_DOWN_DAMNATION, 12, MON_SPELL_PRIEST },
        { SPELL_MALIGN_OFFERING, 18, MON_SPELL_PRIEST },
        { SPELL_SMITING, 18, MON_SPELL_PRIEST },
@@ -423,7 +423,7 @@ static const mon_spellbook mspell_list[] =
     {  MST_ARCANIST,
       {
        { SPELL_BOLT_OF_COLD, 12, MON_SPELL_WIZARD },
-       { SPELL_FORCE_LANCE, 10, MON_SPELL_WIZARD },
+       { SPELL_SEARING_RAY, 16, MON_SPELL_WIZARD },
        { SPELL_VITRIFY, 12, MON_SPELL_WIZARD },
        { SPELL_HASTE, 14, MON_SPELL_WIZARD },
       }
@@ -549,7 +549,7 @@ static const mon_spellbook mspell_list[] =
       {
         { SPELL_SMITING, 20, MON_SPELL_PRIEST },
         { SPELL_SUMMON_DRAKES, 20, MON_SPELL_PRIEST },
-        { SPELL_UPHEAVAL, 20, MON_SPELL_PRIEST },
+        { SPELL_UPHEAVAL, 30, MON_SPELL_PRIEST },
       }
     },
 
@@ -566,8 +566,8 @@ static const mon_spellbook mspell_list[] =
     // ('r') rodents
     {  MST_CRYSTAL_ECHIDNA,
       {
-       { SPELL_CRYSTAL_BOLT, 33, MON_SPELL_NATURAL },
-       { SPELL_THROW_BARBS, 33, MON_SPELL_NATURAL },
+       { SPELL_CRYSTALLIZING_SHOT, 30, MON_SPELL_NATURAL },
+       { SPELL_THROW_BARBS, 30, MON_SPELL_NATURAL },
       }
     },
 
@@ -575,7 +575,7 @@ static const mon_spellbook mspell_list[] =
     {  MST_JUMPING_SPIDER,
       {
        { SPELL_BLINK_CLOSE, 29, MON_SPELL_NATURAL },
-       { SPELL_BLINK_AWAY, 29, MON_SPELL_NATURAL },
+       { SPELL_BLINK_RANGE, 29, MON_SPELL_NATURAL },
       }
     },
 
@@ -617,6 +617,13 @@ static const mon_spellbook mspell_list[] =
     },
 
     // ('v') Vortices.
+    {  MST_THERMIC_DYNAMO,
+      {
+       { SPELL_REBOUNDING_BLAZE, 32, MON_SPELL_MAGICAL },
+       { SPELL_REBOUNDING_CHILL, 32, MON_SPELL_MAGICAL },
+      }
+    },
+
     {  MST_WILL_O_THE_WISP,
       {
        { SPELL_MARSHLIGHT, 50, MON_SPELL_MAGICAL },
@@ -624,6 +631,19 @@ static const mon_spellbook mspell_list[] =
       }
     },
 
+    {  MST_POLTERGUARDIAN,
+      {
+       { SPELL_FORCE_LANCE, 40, MON_SPELL_MAGICAL },
+      }
+    },
+
+    {  MST_UNDYING_ARMOURY,
+      {
+       { SPELL_BESTOW_ARMS, 40, MON_SPELL_NATURAL },
+       { SPELL_FLASHING_BALESTRA, 35, MON_SPELL_NATURAL },
+      }
+    },
+
     // ('w') Slugs and worms.
     { MST_DART_SLUG,
       {
@@ -784,6 +804,14 @@ static const mon_spellbook mspell_list[] =
       }
     },
 
+    {  MST_XAKKRIXIS,
+      {
+       { SPELL_VENOM_BOLT, 24, MON_SPELL_WIZARD},
+       { SPELL_IGNITE_POISON, 35, MON_SPELL_WIZARD},
+       { SPELL_FULMINANT_PRISM, 21, MON_SPELL_WIZARD},
+      }
+    },
+
     // ('C') Giants.
     {  MST_FIRE_GIANT,
       {
@@ -1005,6 +1033,39 @@ static const mon_spellbook mspell_list[] =
       }
     },
 
+    // ('I') Immotile non-plants - statues, machines.
+    {  MST_ICE_STATUE,
+      {
+       { SPELL_BOLT_OF_COLD, 20, MON_SPELL_MAGICAL },
+       { SPELL_THROW_ICICLE, 20, MON_SPELL_MAGICAL },
+       { SPELL_FREEZING_CLOUD, 20, MON_SPELL_MAGICAL },
+       { SPELL_SUMMON_ICE_BEAST, 40, MON_SPELL_MAGICAL },
+      }
+    },
+
+    {  MST_OBSIDIAN_STATUE,
+      {
+       { SPELL_SUMMON_MINOR_DEMON, 33, MON_SPELL_MAGICAL },
+       { SPELL_SUMMON_DEMON, 33, MON_SPELL_MAGICAL },
+       { SPELL_MESMERISE, 33, MON_SPELL_MAGICAL },
+      }
+    },
+
+    {  MST_ORANGE_CRYSTAL_STATUE,
+      {
+       { SPELL_SHADOW_CREATURES, 50, MON_SPELL_MAGICAL },
+       { SPELL_DRAINING_GAZE, 25, MON_SPELL_MAGICAL },
+       { SPELL_MASS_CONFUSION, 25, MON_SPELL_MAGICAL },
+      }
+    },
+
+    {  MST_STRANGE_MACHINE,
+      {
+       { SPELL_SUMMON_ILLUSION, 50, MON_SPELL_MAGICAL },
+       { SPELL_SHADOW_CREATURES, 20, MON_SPELL_MAGICAL },
+      }
+    },
+
     // ('K') Kobolds.
     {  MST_KOBOLD_DEMONOLOGIST,
       {
@@ -1164,14 +1225,14 @@ static const mon_spellbook mspell_list[] =
     {  MST_IRONBOUND_THUNDERHULK,
       {
        { SPELL_CALL_DOWN_LIGHTNING, 50, MON_SPELL_MAGICAL },
-       { SPELL_BLINK_RANGE, 25, MON_SPELL_MAGICAL },
+       { SPELL_BLINK_RANGE, 20, MON_SPELL_MAGICAL },
       }
     },
 
     // ('P') Immobile plants.
-    {  MST_BURNING_BUSH,
+    {  MST_SCRUB_NETTLE,
       {
-       { SPELL_THROW_FLAME, 67, MON_SPELL_NATURAL },
+       { SPELL_STING, 60, MON_SPELL_NATURAL },
       }
     },
 
@@ -1317,7 +1378,7 @@ static const mon_spellbook mspell_list[] =
     },
 
     // ('W') Incorporeal undead.
-    {  MST_SHADOW,
+    {  MST_SHADOWGHAST,
       {
        { SPELL_INVISIBILITY, 33, MON_SPELL_NATURAL },
       }
@@ -1551,44 +1612,11 @@ static const mon_spellbook mspell_list[] =
       }
     },
 
-    {  MST_DEMONSPAWN_BLACK_SUN,
+    {  MST_DEMONSPAWN_SOUL_SCHOLAR,
       {
        { SPELL_BOLT_OF_DRAINING, 18, MON_SPELL_PRIEST },
        { SPELL_DISPEL_UNDEAD_RANGE, 18, MON_SPELL_PRIEST },
-       { SPELL_BLACK_MARK, 28, MON_SPELL_PRIEST },
-      }
-    },
-
-    // ('8') Statues.
-    {  MST_ICE_STATUE,
-      {
-       { SPELL_BOLT_OF_COLD, 20, MON_SPELL_MAGICAL },
-       { SPELL_THROW_ICICLE, 20, MON_SPELL_MAGICAL },
-       { SPELL_FREEZING_CLOUD, 20, MON_SPELL_MAGICAL },
-       { SPELL_SUMMON_ICE_BEAST, 40, MON_SPELL_MAGICAL },
-      }
-    },
-
-    {  MST_OBSIDIAN_STATUE,
-      {
-       { SPELL_SUMMON_MINOR_DEMON, 33, MON_SPELL_MAGICAL },
-       { SPELL_SUMMON_DEMON, 33, MON_SPELL_MAGICAL },
-       { SPELL_MESMERISE, 33, MON_SPELL_MAGICAL },
-      }
-    },
-
-    {  MST_ORANGE_CRYSTAL_STATUE,
-      {
-       { SPELL_SHADOW_CREATURES, 50, MON_SPELL_MAGICAL },
-       { SPELL_DRAINING_GAZE, 25, MON_SPELL_MAGICAL },
-       { SPELL_MASS_CONFUSION, 25, MON_SPELL_MAGICAL },
-      }
-    },
-
-    {  MST_STRANGE_MACHINE,
-      {
-       { SPELL_SUMMON_ILLUSION, 50, MON_SPELL_MAGICAL },
-       { SPELL_SHADOW_CREATURES, 20, MON_SPELL_MAGICAL },
+       { SPELL_SIGN_OF_RUIN, 28, MON_SPELL_PRIEST },
       }
     },
 
@@ -1629,7 +1657,19 @@ static const mon_spellbook mspell_list[] =
 
     {  MST_CRYSTAL_GUARDIAN,
       {
-       { SPELL_CRYSTAL_BOLT, 66, MON_SPELL_MAGICAL },
+       { SPELL_CRYSTALLIZING_SHOT, 50, MON_SPELL_MAGICAL },
+      }
+    },
+
+    {  MST_HOARFROST_CANNON,
+      {
+       { SPELL_HOARFROST_BULLET, 200, MON_SPELL_MAGICAL },
+      }
+    },
+
+    {  MST_HELLFIRE_MORTAR,
+      {
+       { SPELL_BOLT_OF_MAGMA, 200, MON_SPELL_MAGICAL },
       }
     },
 
@@ -1734,6 +1774,13 @@ static const mon_spellbook mspell_list[] =
       }
     },
 
+    {  MST_WIGLAF,
+      {
+       { SPELL_MIGHT, 34, MON_SPELL_WIZARD },
+       { SPELL_HELLFIRE_MORTAR, 34, MON_SPELL_WIZARD },
+      }
+    },
+
     // ('h') Carnivorous quadrupeds.
     {  MST_NATASHA,
       {
@@ -1744,7 +1791,7 @@ static const mon_spellbook mspell_list[] =
     },
 
     // ('i') Spriggans.
-    { MST_THE_ENCHANTRESS,
+    { MST_ENCHANTRESS,
       {
        { SPELL_SLOW, 11, MON_SPELL_WIZARD },
        { SPELL_DIMENSION_ANCHOR, 11, MON_SPELL_WIZARD },
@@ -1767,20 +1814,21 @@ static const mon_spellbook mspell_list[] =
     },
 
     // ('o') Orcs.
-    {  MST_BLORK,
+    {  MST_BLORKULA,
       {
-       { SPELL_CANTRIP, 9, MON_SPELL_WIZARD },
-       { SPELL_MAGIC_DART, 9, MON_SPELL_WIZARD },
+       { SPELL_SANDBLAST, 9, MON_SPELL_WIZARD },
+       { SPELL_STING, 9, MON_SPELL_WIZARD },
+       { SPELL_SHOCK, 9, MON_SPELL_WIZARD },
        { SPELL_THROW_FROST, 9, MON_SPELL_WIZARD },
        { SPELL_THROW_FLAME, 9, MON_SPELL_WIZARD },
-       { SPELL_HASTE, 9, MON_SPELL_WIZARD },
+       { SPELL_HASTE, 12, MON_SPELL_WIZARD },
       }
     },
 
     {  MST_NERGALLE,
       {
        { SPELL_BOLT_OF_DRAINING, 12, MON_SPELL_WIZARD },
-       { SPELL_SUMMON_SPECTRAL_ORCS, 24, MON_SPELL_WIZARD },
+       { SPELL_VANQUISHED_VANGUARD, 24, MON_SPELL_WIZARD },
        { SPELL_HASTE_OTHER, 12, MON_SPELL_WIZARD },
       }
     },
@@ -1808,7 +1856,7 @@ static const mon_spellbook mspell_list[] =
       {
        { SPELL_SYMBOL_OF_TORMENT, 11, MON_SPELL_MAGICAL | MON_SPELL_NOISY },
        { SPELL_SUMMON_UNDEAD, 33, MON_SPELL_MAGICAL | MON_SPELL_NOISY },
-       { SPELL_BLACK_MARK, 22, MON_SPELL_MAGICAL | MON_SPELL_NOISY },
+       { SPELL_SIGN_OF_RUIN, 22, MON_SPELL_MAGICAL | MON_SPELL_NOISY },
       }
     },
 
@@ -1896,10 +1944,10 @@ static const mon_spellbook mspell_list[] =
     // ('H') Hybrids.
     {  MST_ARACHNE,
       {
-       { SPELL_VENOM_BOLT, 11, MON_SPELL_WIZARD },
-       { SPELL_POISON_ARROW, 11, MON_SPELL_WIZARD },
-       { SPELL_ENSNARE, 34, MON_SPELL_WIZARD },
-       { SPELL_BLINK, 11, MON_SPELL_WIZARD },
+       { SPELL_VENOM_BOLT, 18, MON_SPELL_WIZARD },
+       { SPELL_POISON_ARROW, 18, MON_SPELL_WIZARD },
+       { SPELL_GREATER_ENSNARE, 34, MON_SPELL_WIZARD },
+       { SPELL_BLINK_RANGE, 11, MON_SPELL_WIZARD },
       }
     },
 
@@ -1911,6 +1959,16 @@ static const mon_spellbook mspell_list[] =
       }
     },
 
+    // ('I') Statues and other immotile non-plants.
+    { MST_ROXANNE,
+      {
+       { SPELL_BOLT_OF_MAGMA, 13, MON_SPELL_WIZARD },
+       { SPELL_IRON_SHOT, 13, MON_SPELL_WIZARD },
+       { SPELL_LEHUDIBS_CRYSTAL_SPEAR, 13, MON_SPELL_WIZARD },
+       { SPELL_BLINK_OTHER_CLOSE, 13, MON_SPELL_WIZARD },
+      }
+    },
+
     // ('J') Jellies.
     { MST_ENDOPLASM,
       {
@@ -1967,7 +2025,7 @@ static const mon_spellbook mspell_list[] =
        { SPELL_PORTAL_PROJECTILE, 22, MON_SPELL_WIZARD },
        { SPELL_BLINK_OTHER, 11, MON_SPELL_WIZARD | MON_SPELL_EMERGENCY },
        { SPELL_BLINK_ALLIES_AWAY, 22, MON_SPELL_WIZARD },
-       { SPELL_BLINK_AWAY, 11, MON_SPELL_WIZARD },
+       { SPELL_BLINK_RANGE, 11, MON_SPELL_WIZARD },
       }
     },
 
@@ -2036,6 +2094,7 @@ static const mon_spellbook mspell_list[] =
     {  MST_MOON_TROLL,
       {
        { SPELL_CORROSIVE_BOLT, 50, MON_SPELL_WIZARD },
+       { SPELL_DAZZLING_FLASH, 50, MON_SPELL_NATURAL },
       }
     },
 
@@ -2074,16 +2133,6 @@ static const mon_spellbook mspell_list[] =
       }
     },
 
-    // ('8') Statues.
-    { MST_ROXANNE,
-      {
-       { SPELL_BOLT_OF_MAGMA, 13, MON_SPELL_WIZARD },
-       { SPELL_IRON_SHOT, 13, MON_SPELL_WIZARD },
-       { SPELL_LEHUDIBS_CRYSTAL_SPEAR, 13, MON_SPELL_WIZARD },
-       { SPELL_BLINK_OTHER_CLOSE, 13, MON_SPELL_WIZARD },
-      }
-    },
-
     // ('9') Mobile constructs.
     {  MST_VV,
       {
@@ -2242,8 +2291,8 @@ static const mon_spellbook mspell_list[] =
 
     {  MST_NORRIS,
       {
-       { SPELL_BRAIN_BITE, 24, MON_SPELL_PRIEST },
-       { SPELL_PRIMAL_WAVE, 20, MON_SPELL_PRIEST },
+       { SPELL_DRAINING_GAZE, 36, MON_SPELL_PRIEST },
+       { SPELL_PRIMAL_WAVE, 30, MON_SPELL_PRIEST },
       }
     },
 
@@ -2266,7 +2315,7 @@ static const mon_spellbook mspell_list[] =
     // ('&', mostly) Demon lords.
     {  MST_GERYON,
       {
-       { SPELL_SUMMON_HELL_BEAST, 65, MON_SPELL_VOCAL },
+       { SPELL_SUMMON_SIN_BEAST, 55, MON_SPELL_VOCAL },
       }
     },
 
