@@ -95,6 +95,13 @@ private:
     }
 };
 
+enum body_part_class_flags
+{
+    BPART_INTERNAL = 1,
+    BPART_EXTERNAL = 1 << 1,
+    BPART_ANY      = (BPART_INTERNAL|BPART_EXTERNAL),
+};
+
 /// in what ways do a monster's tiles vary?
 enum mon_type_tile_variation
 {
@@ -153,6 +160,7 @@ struct monsterentry
     mon_itemuse_type gmon_use;
     size_type size;
     mon_body_shape shape;
+    god_type god;
     mon_type_tile_info tile;
     tileidx_t corpse_tile; // XXX: ideally this would be autogenerated...
 };
@@ -198,7 +206,7 @@ monster *monster_at(const coord_def &pos);
 // this is the old moname()
 string mons_type_name(monster_type type, description_level_type desc);
 
-bool give_monster_proper_name(monster& mon, bool orcs_only = true);
+bool give_monster_proper_name(monster& mon);
 
 bool mons_flattens_trees(const monster& mon);
 size_type mons_class_body_size(monster_type mc);
@@ -254,6 +262,8 @@ const char * single_holiness_description(mon_holy_type holiness);
 string holiness_description(mon_holy_type holiness);
 mon_holy_type mons_class_holiness(monster_type mc);
 
+const char * intelligence_description(mon_intel_type intel);
+
 void discover_mimic(const coord_def& pos);
 void discover_shifter(monster& shifter);
 
@@ -273,6 +283,7 @@ bool mons_blows_up(const monster& m);
 bool mons_destroyed_on_impact(const monster& m);
 
 bool should_shout_at_mons(const monster &m);
+bool should_attract_mons(const monster &m);
 
 mon_intel_type mons_class_intel(monster_type mc);
 mon_intel_type mons_intel(const monster& mon);
@@ -318,13 +329,12 @@ bool mons_is_zombified(const monster& mons);
 bool mons_class_can_be_zombified(monster_type mc);
 bool mons_can_be_zombified(const monster& mon);
 bool mons_class_can_be_spectralised(monster_type mc, bool divine = false);
-bool mons_can_be_spectralised(const monster& mon, bool divine = false);
+bool mons_can_be_spectralised(const monster& mon, bool divine = false,
+                              bool only_known = false);
 bool mons_class_can_use_stairs(monster_type mc);
 bool mons_class_can_use_transporter(monster_type mc);
 bool mons_can_use_stairs(const monster& mon,
                          dungeon_feature_type stair = DNGN_UNSEEN);
-bool mons_bound_body_and_soul(const monster& mon);
-bool mons_bound_soul(const monster& mon);
 void name_zombie(monster& mon, monster_type mc, const string &mon_name);
 void name_zombie(monster& mon, const monster& orig);
 
@@ -421,8 +431,8 @@ bool mons_is_projectile(const monster& mon);
 bool mons_is_object(monster_type mc);
 bool mons_has_blood(monster_type mc);
 bool mons_is_sensed(monster_type mc);
-bool mons_allows_beogh(const monster& mon);
-bool mons_allows_beogh_now(const monster& mon);
+bool mons_offers_beogh_conversion(const monster& mon);
+bool mons_offers_beogh_conversion_now(const monster& mon);
 
 bool invalid_monster(const monster* mon);
 bool invalid_monster_type(monster_type mt);
@@ -467,6 +477,8 @@ monster_type random_monster_at_grid(const coord_def& p, bool species = false);
 void         init_mon_name_cache();
 monster_type get_monster_by_name(string name, bool substring = false);
 
+string random_body_part_name(bool plural, int part_class);
+
 string do_mon_str_replacements(const string &msg, const monster& mons,
                                int s_type = -1);
 
@@ -497,6 +509,7 @@ int get_dist_to_nearest_monster();
 bool monster_nearby();
 actor *actor_by_mid(mid_t m, bool require_valid = false);
 monster *monster_by_mid(mid_t m, bool require_valid = false);
+monster *cached_monster_copy_by_mid(mid_t m);
 bool mons_is_recallable(const actor* caller, const monster& targ);
 void init_anon();
 actor *find_agent(mid_t m, kill_category kc);
@@ -519,16 +532,12 @@ void debug_monspells();
 
 bool choose_any_monster(const monster& mon);
 monster *choose_random_nearby_monster(
-    int weight,
     bool (*suitable)(const monster& mon) =
-        choose_any_monster,
-    bool prefer_named_or_priest = false);
+        choose_any_monster);
 
 monster *choose_random_monster_on_level(
-    int weight,
     bool (*suitable)(const monster& mon) =
-        choose_any_monster,
-    bool prefer_named_or_priest = false);
+        choose_any_monster);
 
 int spell_freq_for_hd(int hd);
 void normalize_spell_freq(monster_spells &spells, int total_freq);
@@ -566,3 +575,5 @@ bool apply_visible_monsters(monster_func mf,
                             los_type los = LOS_NO_TRANS);
 
 int derived_undead_avg_hp(monster_type mtype, int hd, int scale = 10);
+
+int touch_of_beogh_hp_mult(const monster& mon);
