@@ -17,6 +17,7 @@
 #include "initfile.h"
 #include "item-name.h" // make_name
 #include "item-prop.h"
+#include "job-groups.h"
 #include "jobs.h"
 #include "libutil.h"
 #include "macro.h"
@@ -266,15 +267,29 @@ static void _resolve_job(newgame_def& ng, const newgame_def& ng_choice)
 
 static void _resolve_species_job(newgame_def& ng, const newgame_def& ng_choice)
 {
-    // Since recommendations are no longer bidirectional, pick one of
-    // species or job to start. If one but not the other was specified
-    // as "viable", always choose that one last; otherwise use a random
-    // order.
-    const bool spfirst  = ng_choice.species != SP_VIABLE
-                          && ng_choice.job == JOB_VIABLE;
-    const bool jobfirst = ng_choice.species == SP_VIABLE
-                          && ng_choice.job != JOB_VIABLE;
-    if (spfirst || !jobfirst && coinflip())
+    bool spfirst;
+    // If both are "viable" or both are "random" resolve them in a random order,
+    // to avoid bias.
+    if (ng_choice.species == SP_VIABLE && ng_choice.job == JOB_VIABLE
+        || ng_choice.species == SP_RANDOM && ng_choice.job == JOB_RANDOM)
+    {
+        spfirst = coinflip();
+    }
+    // If exactly one is "viable", resolve that second.
+    else if (ng_choice.species == SP_VIABLE)
+        spfirst = false;
+    else if (ng_choice.job == JOB_VIABLE)
+        spfirst = true;
+    // If one is fixed and the other is "random", resolve that second (to avoid
+    // picking a random choice that is banned for the fixed one).
+    else if (ng_choice.species == SP_RANDOM)
+        spfirst = false;
+    else if (ng_choice.job == JOB_RANDOM)
+        spfirst = true;
+    // If we're here then they're both fixed and the order doesn't matter.
+    else
+        spfirst = true;
+    if (spfirst)
     {
         _resolve_species(ng, ng_choice);
         _resolve_job(ng, ng_choice);
@@ -354,7 +369,7 @@ static bool _reroll_random(newgame_def& ng)
 #ifdef USE_TILE_LOCAL
     auto tile = make_shared<ui::PlayerDoll>(doll);
     tile->set_margin_for_sdl(0, 10, 0, 0);
-    title_hbox->add_child(move(tile));
+    title_hbox->add_child(std::move(tile));
 #endif
 #endif
     title_hbox->add_child(make_shared<Text>(prompt));
@@ -363,9 +378,9 @@ static bool _reroll_random(newgame_def& ng)
     title_hbox->set_margin_for_crt(0, 0, 1, 0);
 
     auto vbox = make_shared<Box>(Box::VERT);
-    vbox->add_child(move(title_hbox));
+    vbox->add_child(std::move(title_hbox));
     vbox->add_child(make_shared<Text>("Do you want to play this combination? [Y/n/q]"));
-    auto popup = make_shared<ui::Popup>(move(vbox));
+    auto popup = make_shared<ui::Popup>(std::move(vbox));
 
     bool done = false;
     int c;
@@ -381,7 +396,7 @@ static bool _reroll_random(newgame_def& ng)
     tiles.push_ui_layout("newgame-random-combo", 0);
     popup->on_layout_pop([](){ tiles.pop_ui_layout(); });
 #endif
-    ui::run_layout(move(popup), done);
+    ui::run_layout(std::move(popup), done);
 
     // XX mouse interface in local tiles for `y` -- right now right click does
     // `q` only
@@ -412,7 +427,7 @@ static void _choose_char(newgame_def& ng, newgame_def& choice,
 
 #if defined(DGAMELAUNCH) && defined(TOURNEY)
     // Apologies to non-public servers.
-    if (ng.type == GAME_TYPE_NORMAL)
+    if (ng.type == GAME_TYPE_NORMAL || ng.type == GAME_TYPE_DESCENT)
     {
         if (!yesno("Trunk doesn't count for the tournament, you want "
                    TOURNEY ". Play trunk anyway? (Y/N)", false, 'n'))
@@ -523,12 +538,12 @@ static void _add_menu_sub_item(shared_ptr<OuterMenu>& menu, int x, int y, const
     tmp->set_margin_for_crt(0, 2, 0, 0);
 
     auto btn = make_shared<MenuButton>();
-    btn->set_child(move(tmp));
+    btn->set_child(std::move(tmp));
     btn->id = id;
     btn->description = description;
     btn->hotkey = letter;
     btn->highlight_colour = STARTUP_HIGHLIGHT_CONTROL;
-    menu->add_button(move(btn), x, y);
+    menu->add_button(std::move(btn), x, y);
 }
 
 #ifndef DGAMELAUNCH
@@ -577,7 +592,7 @@ static void _choose_name(newgame_def& ng, newgame_def& choice)
 #ifdef USE_TILE_LOCAL
     auto tile = make_shared<ui::PlayerDoll>(doll);
     tile->set_margin_for_sdl(0, 10, 0, 0);
-    title_hbox->add_child(move(tile));
+    title_hbox->add_child(std::move(tile));
 #endif
 #endif
     title_hbox->add_child(make_shared<Text>(title));
@@ -586,7 +601,7 @@ static void _choose_name(newgame_def& ng, newgame_def& choice)
     title_hbox->set_margin_for_crt(0, 0, 1, 0);
 
     auto vbox = make_shared<Box>(Box::VERT);
-    vbox->add_child(move(title_hbox));
+    vbox->add_child(std::move(title_hbox));
     auto prompt_ui = make_shared<Text>();
     vbox->add_child(prompt_ui);
 
@@ -622,7 +637,7 @@ static void _choose_name(newgame_def& ng, newgame_def& choice)
         });
         tmp->set_margin_for_sdl(4,8);
         tmp->set_margin_for_crt(0, 2, 0, 0);
-        btn->set_child(move(tmp));
+        btn->set_child(std::move(tmp));
         btn->id = CK_ENTER;
         btn->description = "";
         btn->hotkey = CK_ENTER;
@@ -640,7 +655,7 @@ static void _choose_name(newgame_def& ng, newgame_def& choice)
         sub_items->add_button(btn, 2, 0);
     }
 
-    auto popup = make_shared<ui::Popup>(move(vbox));
+    auto popup = make_shared<ui::Popup>(std::move(vbox));
 
     sub_items->on_activate_event([&](const ActivateEvent& event) {
         const auto button = static_pointer_cast<MenuButton>(event.target());
@@ -648,10 +663,14 @@ static void _choose_name(newgame_def& ng, newgame_def& choice)
         switch (id)
         {
             case '*':
+                if (overwrite_prompt)
+                    break; // too weird
                 reader.putkey(CK_END);
                 reader.putkey(CONTROL('U'));
                 for (char ch : newgame_random_name())
                     reader.putkey(ch);
+                good_name = is_good_name(buf, true);
+                ok_switcher->current() = good_name ? 0 : 1;
                 break;
             case CK_ESCAPE: // redundant with key_exits_popup check below
                 done = cancel = true;
@@ -680,7 +699,7 @@ static void _choose_name(newgame_def& ng, newgame_def& choice)
         return true;
     });
 
-    ui::push_layout(move(popup));
+    ui::push_layout(std::move(popup));
     while (!done && !crawl_state.seen_hups)
     {
         formatted_string prompt;
@@ -849,7 +868,7 @@ static void _choose_seed(newgame_def& ng, newgame_def& choice,
     clear_btn_label->set_margin_for_sdl(4, 4);
     clear_btn_label->set_margin_for_crt(0, 1, 0, 1);
     auto clear_btn = make_shared<MenuButton>();
-    clear_btn->set_child(move(clear_btn_label));
+    clear_btn->set_child(std::move(clear_btn_label));
     clear_btn->set_sync_id("btn-clear");
     clear_btn->hotkey = '-';
     clear_btn->set_margin_for_sdl(0,0,0,10);
@@ -859,7 +878,7 @@ static void _choose_seed(newgame_def& ng, newgame_def& choice,
         ui::set_focused_widget(seed_input.get());
         return true;
     });
-    seed_hbox->add_child(move(clear_btn));
+    seed_hbox->add_child(std::move(clear_btn));
 
     auto d_btn_label = make_shared<ui::Text>();
     d_btn_label->set_text(formatted_string("[d] Today's daily seed", BROWN));
@@ -867,7 +886,7 @@ static void _choose_seed(newgame_def& ng, newgame_def& choice,
     d_btn_label->set_margin_for_crt(0, 2, 0, 0);
 
     auto daily_seed_btn = make_shared<MenuButton>();
-    daily_seed_btn->set_child(move(d_btn_label));
+    daily_seed_btn->set_child(std::move(d_btn_label));
     daily_seed_btn->set_sync_id("btn-daily");
     daily_seed_btn->hotkey = 'd';
     daily_seed_btn->set_margin_for_sdl(0,0,0,10);
@@ -882,7 +901,7 @@ static void _choose_seed(newgame_def& ng, newgame_def& choice,
         ui::set_focused_widget(seed_input.get());
         return true;
     });
-    seed_hbox->add_child(move(daily_seed_btn));
+    seed_hbox->add_child(std::move(daily_seed_btn));
 
     const string footer_text =
 #ifdef USE_TILE_LOCAL
@@ -965,7 +984,7 @@ static void _choose_seed(newgame_def& ng, newgame_def& choice,
     popup->on_layout_pop([](){ tiles.pop_ui_layout(); });
 #endif
 
-    ui::run_layout(move(popup), done, seed_input);
+    ui::run_layout(std::move(popup), done, seed_input);
 
     string result = seed_input->get_text();
     uint64_t tmp_seed = 0;
@@ -1101,28 +1120,26 @@ static job_group jobs_order[] =
         { JOB_FIGHTER, JOB_GLADIATOR, JOB_MONK, JOB_HUNTER, JOB_BRIGAND }
     },
     {
-        "Adventurer",
-        coord_def(0, 6), 20,
-        { JOB_ARTIFICER, JOB_WANDERER, JOB_DELVER, }
+        "Zealot",
+        coord_def(0, 6), 25,
+        { JOB_BERSERKER, JOB_CINDER_ACOLYTE, JOB_CHAOS_KNIGHT }
     },
     {
-        "Zealot",
-        coord_def(1, 0), 25,
-        { JOB_BERSERKER, JOB_CINDER_ACOLYTE,
-          JOB_CHAOS_KNIGHT }
+        "Adventurer",
+        coord_def(1, 0), 20,
+        { JOB_ARTIFICER, JOB_SHAPESHIFTER, JOB_WANDERER, JOB_DELVER, }
     },
     {
         "Warrior-mage",
-        coord_def(1, 4), 26,
-        { JOB_TRANSMUTER, JOB_WARPER, JOB_HEXSLINGER,
-          JOB_ENCHANTER, JOB_REAVER }
+        coord_def(1, 5), 26,
+        { JOB_WARPER, JOB_HEXSLINGER, JOB_ENCHANTER, JOB_REAVER }
     },
     {
         "Mage",
         coord_def(2, 0), 22,
         { JOB_HEDGE_WIZARD, JOB_CONJURER, JOB_SUMMONER, JOB_NECROMANCER,
           JOB_FIRE_ELEMENTALIST, JOB_ICE_ELEMENTALIST,
-          JOB_AIR_ELEMENTALIST, JOB_EARTH_ELEMENTALIST, JOB_VENOM_MAGE }
+          JOB_AIR_ELEMENTALIST, JOB_EARTH_ELEMENTALIST, JOB_ALCHEMIST }
     }
 };
 
@@ -1269,7 +1286,7 @@ protected:
         tile->set_tile(item_tile);
         tile->set_margin_for_sdl(0, 6, 0, 0);
         tile->flex_grow = 0;
-        hbox->add_child(move(tile));
+        hbox->add_child(std::move(tile));
         hbox->add_child(label);
 #else
         UNUSED(item_tile);
@@ -1309,9 +1326,9 @@ protected:
         auto btn = make_shared<MenuButton>();
 #ifdef USE_TILE
         hbox->set_margin_for_sdl(2, 10, 2, 2);
-        btn->set_child(move(hbox));
+        btn->set_child(std::move(hbox));
 #else
-        btn->set_child(move(label));
+        btn->set_child(std::move(label));
 #endif
         btn->id = id;
         btn->description = desc;
@@ -1329,7 +1346,7 @@ protected:
     {
         auto text = make_shared<Text>(formatted_string(name, LIGHTBLUE));
         text->set_margin_for_sdl(7, 0, 7, 32+2+6);
-        m_main_items->add_label(move(text), position.x, position.y);
+        m_main_items->add_label(std::move(text), position.x, position.y);
     }
 
     void _add_choice_menu_option(int x, int y, const string& text, char letter,
@@ -1580,9 +1597,6 @@ void species_group::attach(const newgame_def& ng, const newgame_def& defaults,
         if (this_species == SP_UNKNOWN)
             break;
 
-        if (this_species == SP_METEORAN && ng.type == GAME_TYPE_SPRINT)
-            continue;
-
         if (ng.job == JOB_UNKNOWN && !species::is_starting_species(this_species))
             continue;
 
@@ -1640,7 +1654,7 @@ static void _prompt_choice(int choice_type, newgame_def& ng, newgame_def& ng_cho
     popup->on_layout_pop([](){ tiles.pop_ui_layout(); });
 #endif
 
-    ui::run_layout(move(popup), newgame_ui->done);
+    ui::run_layout(std::move(popup), newgame_ui->done);
 
     if (newgame_ui->end_game)
         end(0);
@@ -1648,14 +1662,14 @@ static void _prompt_choice(int choice_type, newgame_def& ng, newgame_def& ng_cho
         game_ended(game_exit::abort);
 }
 
-typedef pair<weapon_type, char_choice_restriction> weapon_choice;
+typedef pair<weapon_type, char_choice_restriction> weap_choice;
 
 static weapon_type _fixup_weapon(weapon_type wp,
-                                 const vector<weapon_choice>& weapons)
+                                 const vector<weap_choice>& weapons)
 {
     if (wp == WPN_UNKNOWN || wp == WPN_RANDOM || wp == WPN_VIABLE)
         return wp;
-    for (weapon_choice choice : weapons)
+    for (weap_choice choice : weapons)
         if (wp == choice.first)
             return wp;
     return WPN_UNKNOWN;
@@ -1663,7 +1677,7 @@ static weapon_type _fixup_weapon(weapon_type wp,
 
 static void _construct_weapon_menu(const newgame_def& ng,
                                    const weapon_type& defweapon,
-                                   const vector<weapon_choice>& weapons,
+                                   const vector<weap_choice>& weapons,
                                    shared_ptr<OuterMenu>& main_items,
                                    shared_ptr<OuterMenu>& sub_items)
 {
@@ -1672,9 +1686,9 @@ static void _construct_weapon_menu(const newgame_def& ng,
         string label;
         tileidx_t tile;
         weapon_menu_item(skill_type _skill, string _label, tileidx_t _tile)
-            : skill(move(_skill)), label(move(_label)), tile(move(_tile)) {};
+            : skill(std::move(_skill)), label(std::move(_label)), tile(std::move(_tile)) {};
         weapon_menu_item(skill_type _skill, string _label)
-            : skill(move(_skill)), label(move(_label)), tile(0) {};
+            : skill(std::move(_skill)), label(std::move(_label)), tile(0) {};
     };
     vector<weapon_menu_item> choices;
 
@@ -1755,7 +1769,7 @@ static void _construct_weapon_menu(const newgame_def& ng,
         hbox->add_child(suffix);
 
         auto btn = make_shared<MenuButton>();
-        btn->set_child(move(hbox));
+        btn->set_child(std::move(hbox));
         btn->id = wpn_type;
         btn->hotkey = letter;
         btn->highlight_colour = bg;
@@ -1763,7 +1777,7 @@ static void _construct_weapon_menu(const newgame_def& ng,
         // Is this item our default weapon?
         if (wpn_type == defweapon || (defweapon == WPN_UNKNOWN && i == 0))
             main_items->set_initial_focus(btn.get());
-        main_items->add_button(move(btn), 0, i);
+        main_items->add_button(std::move(btn), 0, i);
     }
 
     _add_menu_sub_item(sub_items, 0, 0, "+ - Recommended random choice",
@@ -1797,7 +1811,7 @@ static void _construct_weapon_menu(const newgame_def& ng,
  */
 static bool _prompt_weapon(const newgame_def& ng, newgame_def& ng_choice,
                            const newgame_def& defaults,
-                           const vector<weapon_choice>& weapons)
+                           const vector<weap_choice>& weapons)
 {
     weapon_type defweapon = _fixup_weapon(defaults.weapon, weapons);
 
@@ -1808,7 +1822,7 @@ static bool _prompt_weapon(const newgame_def& ng, newgame_def& ng_choice,
 #ifdef USE_TILE_LOCAL
     auto tile = make_shared<ui::PlayerDoll>(doll);
     tile->set_margin_for_sdl(0, 10, 0, 0);
-    title_hbox->add_child(move(tile));
+    title_hbox->add_child(std::move(tile));
 #endif
 #endif
     auto title = make_shared<Text>(formatted_string(_welcome(ng), BROWN));
@@ -1911,7 +1925,7 @@ static bool _prompt_weapon(const newgame_def& ng, newgame_def& ng_choice,
     tiles.push_ui_layout("newgame-choice", 1);
     popup->on_layout_pop([](){ tiles.pop_ui_layout(); });
 #endif
-    ui::run_layout(move(popup), done);
+    ui::run_layout(std::move(popup), done);
 
     return ret;
 }
@@ -1943,15 +1957,15 @@ weapon_type starting_weapon_upgrade(weapon_type wp, job_type job,
     }
 }
 
-static vector<weapon_choice> _get_weapons(const newgame_def& ng)
+static vector<weap_choice> _get_weapons(const newgame_def& ng)
 {
-    vector<weapon_choice> weapons;
+    vector<weap_choice> weapons;
     weapon_type startwep[7] = { WPN_SHORT_SWORD, WPN_MACE, WPN_HAND_AXE,
                                 WPN_SPEAR, WPN_FALCHION, WPN_QUARTERSTAFF,
                                 WPN_UNARMED };
     for (int i = 0; i < 7; ++i)
     {
-        weapon_choice wp;
+        weap_choice wp;
         wp.first = startwep[i];
         if (job_gets_good_weapons(ng.job))
         {
@@ -1967,7 +1981,7 @@ static vector<weapon_choice> _get_weapons(const newgame_def& ng)
 }
 
 static void _resolve_weapon(newgame_def& ng, newgame_def& ng_choice,
-                            const vector<weapon_choice>& weapons)
+                            const vector<weap_choice>& weapons)
 {
     int weapon = ng_choice.weapon;
 
@@ -1983,7 +1997,7 @@ static void _resolve_weapon(newgame_def& ng, newgame_def& ng_choice,
     case WPN_VIABLE:
     {
         int good_choices = 0;
-        for (weapon_choice choice : weapons)
+        for (weap_choice choice : weapons)
         {
             if (choice.second == CC_UNRESTRICTED
                 && one_chance_in(++good_choices))
@@ -2020,7 +2034,7 @@ static bool _choose_weapon(newgame_def& ng, newgame_def& ng_choice,
     if (!job_has_weapon_choice(ng.job))
         return true;
 
-    vector<weapon_choice> weapons = _get_weapons(ng);
+    vector<weap_choice> weapons = _get_weapons(ng);
 
     ASSERT(!weapons.empty());
     if (weapons.size() == 1)
@@ -2105,7 +2119,7 @@ static void _construct_gamemode_map_menu(const mapref_vector& maps,
         auto tile = make_shared<Image>();
         tile->set_tile(tile_for_map_name(map_name));
         tile->set_margin_for_sdl(0, 6, 0, 0);
-        hbox->add_child(move(tile));
+        hbox->add_child(std::move(tile));
         hbox->add_child(label);
 #endif
 
@@ -2114,9 +2128,9 @@ static void _construct_gamemode_map_menu(const mapref_vector& maps,
         auto btn = make_shared<MenuButton>();
 #ifdef USE_TILE
         hbox->set_margin_for_sdl(2, 10, 2, 2);
-        btn->set_child(move(hbox));
+        btn->set_child(std::move(hbox));
 #else
-        btn->set_child(move(label));
+        btn->set_child(std::move(label));
 #endif
         btn->id = i; // ID corresponds to location in vector
         btn->hotkey = letter;
@@ -2276,7 +2290,7 @@ static void _prompt_gamemode_map(newgame_def& ng, newgame_def& ng_choice,
     tiles.push_ui_layout("newgame-choice", 1);
     popup->on_layout_pop([](){ tiles.pop_ui_layout(); });
 #endif
-    ui::run_layout(move(popup), done);
+    ui::run_layout(std::move(popup), done);
 
     if (cancel || crawl_state.seen_hups)
         game_ended(game_exit::abort);
