@@ -27,6 +27,11 @@ int actor_to_death_source(const actor* agent);
 
 string morgue_name(string char_name, time_t when_crawl_got_even);
 
+int corrosion_chance(int sources);
+
+int outgoing_harm_amount(int levels);
+int incoming_harm_amount(int levels);
+
 void reset_damage_counters();
 void ouch(int dam, kill_method_type death_type, mid_t source = MID_NOBODY,
           const char *aux = nullptr, bool see_source = true,
@@ -34,7 +39,7 @@ void ouch(int dam, kill_method_type death_type, mid_t source = MID_NOBODY,
 
 void lose_level();
 bool drain_player(int power = 25, bool announce_full = true,
-                  bool ignore_protection = false);
+                  bool ignore_protection = false, bool quiet = false);
 
 void expose_player_to_element(beam_type flavour, int strength = 0,
                               bool slow_cold_blooded = true);
