@@ -171,8 +171,10 @@ bool unmeld_slot(equipment_type slot)
     return false;
 }
 
-static void _equip_weapon_effect(item_def& item, bool showMsgs, bool unmeld);
-static void _unequip_weapon_effect(item_def& item, bool showMsgs, bool meld);
+static void _equip_weapon_effect(item_def& item, bool showMsgs, bool unmeld,
+                                 equipment_type slot);
+static void _unequip_weapon_effect(item_def& item, bool showMsgs, bool meld,
+                                   equipment_type slot);
 static void _equip_armour_effect(item_def& arm, bool unmeld,
                                  equipment_type slot);
 static void _unequip_armour_effect(item_def& item, bool meld,
@@ -182,14 +184,16 @@ static void _equip_jewellery_effect(item_def &item, bool unmeld,
 static void _unequip_jewellery_effect(item_def &item, bool mesg, bool meld,
                                       equipment_type slot);
 static void _equip_use_warning(const item_def& item);
-static void _equip_regeneration_item(const item_def& item);
-static void _deactivate_regeneration_item(const item_def& item, bool meld);
+static void _handle_regen_item_equip(const item_def& item);
+static void _deactivate_item(const item_def& item);
 
 static void _assert_valid_slot(equipment_type eq, equipment_type slot)
 {
 #ifdef ASSERTS
     if (eq == slot)
         return;
+    if (slot == EQ_OFFHAND && you.has_mutation(MUT_WIELD_OFFHAND)) // hack for off-hand wielding
+        return;
     ASSERT(eq == EQ_RINGS); // all other slots are unique
     equipment_type r1 = EQ_LEFT_RING, r2 = EQ_RIGHT_RING;
     if (species::arm_count(you.species) > 2)
@@ -220,12 +224,15 @@ void equip_effect(equipment_type slot, int item_slot, bool unmeld, bool msg)
 
     identify_item(item);
 
-    if (slot == EQ_WEAPON)
-        _equip_weapon_effect(item, msg, unmeld);
+    if (slot == EQ_WEAPON || (slot == EQ_OFFHAND && is_weapon(item)))
+        _equip_weapon_effect(item, msg, unmeld, slot);
     else if (slot >= EQ_CLOAK && slot <= EQ_BODY_ARMOUR)
         _equip_armour_effect(item, unmeld, slot);
     else if (slot >= EQ_FIRST_JEWELLERY && slot <= EQ_LAST_JEWELLERY)
         _equip_jewellery_effect(item, unmeld, slot);
+
+    if (!unmeld)
+        _handle_regen_item_equip(item);
 }
 
 void unequip_effect(equipment_type slot, int item_slot, bool meld, bool msg)
@@ -240,8 +247,8 @@ void unequip_effect(equipment_type slot, int item_slot, bool meld, bool msg)
 
     const interrupt_block block_meld_interrupts(meld);
 
-    if (slot == EQ_WEAPON)
-        _unequip_weapon_effect(item, msg, meld);
+    if (slot == EQ_WEAPON || (slot == EQ_OFFHAND && is_weapon(item)))
+        _unequip_weapon_effect(item, msg, meld, slot);
     else if (slot >= EQ_CLOAK && slot <= EQ_BODY_ARMOUR)
         _unequip_armour_effect(item, meld, slot);
     else if (slot >= EQ_FIRST_JEWELLERY && slot <= EQ_LAST_JEWELLERY)
@@ -295,16 +302,6 @@ void equip_artefact_effect(item_def &item, bool *show_msgs, bool unmeld,
                                                   : MSG_MANA_DECREASE);
     }
 
-    if (proprt[ARTP_REGENERATION]
-        && !unmeld
-        // If regen is an intrinsic property too, don't double print messages.
-        && !item.is_type(OBJ_JEWELLERY, AMU_REGENERATION)
-        && (item.base_type != OBJ_ARMOUR
-            || !armour_type_prop(item.sub_type, ARMF_REGENERATION)))
-    {
-        _equip_regeneration_item(item);
-    }
-
     // Modify ability scores.
     notify_stat_change(STAT_STR, proprt[ARTP_STRENGTH],
                        !(msg && proprt[ARTP_STRENGTH] && !unmeld));
@@ -408,8 +405,8 @@ void unequip_artefact_effect(item_def &item,  bool *show_msgs, bool meld,
     if (proprt[ARTP_SEE_INVISIBLE])
         _mark_unseen_monsters();
 
-    if (proprt[ARTP_REGENERATION])
-        _deactivate_regeneration_item(item, meld);
+    if (proprt[ARTP_REGENERATION] && !meld)
+        _deactivate_item(item);
 
     if (is_unrandom_artefact(item))
     {
@@ -447,7 +444,8 @@ static void _equip_use_warning(const item_def& item)
 // Provide a function for handling initial wielding of 'special'
 // weapons, or those whose function is annoying to reproduce in
 // other places *cough* auto-butchering *cough*.    {gdl}
-static void _equip_weapon_effect(item_def& item, bool showMsgs, bool unmeld)
+static void _equip_weapon_effect(item_def& item, bool showMsgs, bool unmeld,
+                                 equipment_type slot)
 {
     you.wield_change = true;
     quiver::on_weapon_changed();
@@ -461,7 +459,7 @@ static void _equip_weapon_effect(item_def& item, bool showMsgs, bool unmeld)
     case OBJ_STAVES:
     {
         if (artefact)
-            equip_artefact_effect(item, &showMsgs, unmeld, EQ_STAFF);
+            equip_artefact_effect(item, &showMsgs, unmeld, slot);
         break;
     }
 
@@ -470,12 +468,9 @@ static void _equip_weapon_effect(item_def& item, bool showMsgs, bool unmeld)
         // Note that if the unrand equip prints a message, it will
         // generally set showMsgs to false.
         if (artefact)
-            equip_artefact_effect(item, &showMsgs, unmeld, EQ_WEAPON);
+            equip_artefact_effect(item, &showMsgs, unmeld, slot);
 
-        special = item.brand;
-
-        if (artefact)
-            special = artefact_property(item, ARTP_BRAND);
+        special = get_weapon_brand(item);
 
         if (special != SPWPN_NORMAL)
         {
@@ -497,8 +492,29 @@ static void _equip_weapon_effect(item_def& item, bool showMsgs, bool unmeld)
                     break;
 
                 case SPWPN_HOLY_WRATH:
-                    mprf("%s softly glows with a divine radiance!",
-                         item_name.c_str());
+                    if (you.undead_or_demonic())
+                    {
+                        mprf("%s sits dull and lifeless in your grasp.",
+                             item_name.c_str());
+                    }
+                    else
+                    {
+                        mprf("%s softly glows with a divine radiance!",
+                             item_name.c_str());
+                    }
+                    break;
+
+                case SPWPN_FOUL_FLAME:
+                    if (you.is_holy())
+                    {
+                        mprf("%s sits dull and lifeless in your grasp.",
+                             item_name.c_str());
+                    }
+                    else
+                    {
+                        mprf("%s glows horrifically with a foul blackness!",
+                             item_name.c_str());
+                    }
                     break;
 
                 case SPWPN_ELECTROCUTION:
@@ -619,7 +635,7 @@ static void _equip_weapon_effect(item_def& item, bool showMsgs, bool unmeld)
 }
 
 static void _unequip_weapon_effect(item_def& real_item, bool showMsgs,
-                                   bool meld)
+                                   bool meld, equipment_type slot)
 {
     you.wield_change = true;
     quiver::on_weapon_changed();
@@ -630,10 +646,7 @@ static void _unequip_weapon_effect(item_def& real_item, bool showMsgs,
     // Call this first, so that the unrandart func can set showMsgs to
     // false if it does its own message handling.
     if (is_artefact(item))
-    {
-        unequip_artefact_effect(real_item, &showMsgs, meld, EQ_WEAPON,
-                                true);
-    }
+        unequip_artefact_effect(real_item, &showMsgs, meld, slot, true);
 
     if (item.base_type == OBJ_WEAPONS)
     {
@@ -650,8 +663,17 @@ static void _unequip_weapon_effect(item_def& real_item, bool showMsgs,
                     mprf("%s stops flaming.", msg.c_str());
                 break;
 
-            case SPWPN_FREEZING:
             case SPWPN_HOLY_WRATH:
+                if (showMsgs && !you.undead_or_demonic())
+                    mprf("%s stops glowing.", msg.c_str());
+                break;
+
+            case SPWPN_FOUL_FLAME:
+                if (showMsgs && !you.is_holy())
+                    mprf("%s stops glowing.", msg.c_str());
+                break;
+
+            case SPWPN_FREEZING:
                 if (showMsgs)
                     mprf("%s stops glowing.", msg.c_str());
                 break;
@@ -728,7 +750,7 @@ static void _spirit_shield_message(bool unmeld)
 #if TAG_MAJOR_VERSION == 34
         if (you.species == SP_DEEP_DWARF)
         {
-            drain_mp(you.magic_points);
+            drain_mp(you.magic_points, true);
             mpr("Now linked to your health, your magic stops regenerating.");
         }
 #endif
@@ -866,9 +888,6 @@ static void _equip_armour_effect(item_def& arm, bool unmeld,
 
     }
 
-    if (armour_type_prop(arm.sub_type, ARMF_REGENERATION) && !unmeld)
-        _equip_regeneration_item(arm);
-
     if (is_artefact(arm))
     {
         bool show_msgs = true;
@@ -999,8 +1018,8 @@ static void _unequip_armour_effect(item_def& item, bool meld,
         break;
     }
 
-    if (armour_type_prop(item.sub_type, ARMF_REGENERATION))
-        _deactivate_regeneration_item(item, meld);
+    if (armour_type_prop(item.sub_type, ARMF_REGENERATION) && !meld)
+        _deactivate_item(item);
 
     if (is_artefact(item))
         unequip_artefact_effect(item, nullptr, meld, slot, false);
@@ -1026,13 +1045,6 @@ static void _remove_amulet_of_faith(item_def &item)
         return;
     }
 
-    if (you_worship(GOD_YREDELEMNUL))
-    {
-        mprf(MSGCH_GOD, "The black torch dims.");
-        yred_reclaim_souls();
-        return;
-    }
-
     simple_god_message(" seems less interested in you.");
 
     const int piety_loss = div_rand_round(you.piety, 3);
@@ -1042,10 +1054,18 @@ static void _remove_amulet_of_faith(item_def &item)
     lose_piety(piety_loss);
 }
 
-static void _equip_regeneration_item(const item_def &item)
+static void _handle_regen_item_equip(const item_def& item)
 {
+    const bool regen_hp = is_regen_item(item);
+    const bool regen_mp = is_mana_regen_item(item);
+
+    if (!regen_hp && !regen_mp)
+        return;
+
+    // The item is providing at least one of these two things, so let's check
+    // attunement status.
+
     equipment_type eq_slot = item_equip_slot(item);
-    // currently regen is only on amulets and armour
     bool plural = (eq_slot == EQ_BOOTS && item.sub_type != ARM_BARDING)
                   || eq_slot == EQ_GLOVES;
     string item_name = is_artefact(item) ? get_artefact_name(item)
@@ -1056,22 +1076,36 @@ static void _equip_regeneration_item(const item_def &item)
                                          : item_slot_name(eq_slot);
 
 #if TAG_MAJOR_VERSION == 34
-    if (you.get_mutation_level(MUT_NO_REGENERATION))
+    if (regen_hp && !regen_mp && you.get_mutation_level(MUT_NO_REGENERATION))
     {
         mprf("The %s feel%s cold and inert.", item_name.c_str(),
              plural ? "" : "s");
         return;
     }
 #endif
-    if (you.hp == you.hp_max)
+    if (regen_mp && !regen_hp && !player_regenerates_mp())
     {
-        mprf("The %s throb%s to your uninjured body.", item_name.c_str(),
-             plural ? " as they attune themselves" : "s as it attunes itself");
+        mprf("The %s feel%s cold and inert.", item_name.c_str(),
+             plural ? "" : "s");
+
+        return;
+    }
+
+    bool low_mp = regen_mp && you.magic_points < you.max_magic_points;
+    bool low_hp = regen_hp && you.hp < you.hp_max;
+
+    if (!low_mp && !low_hp)
+    {
+        mprf("The %s throb%s to your%s body.", item_name.c_str(),
+             plural ? " as they attune themselves" : "s as it attunes itself",
+             regen_hp ? " uninjured" : "");
         you.activated.set(eq_slot);
         return;
     }
-    mprf("The %s cannot attune %s to your injured body.", item_name.c_str(),
-         plural ? "themselves" : "itself");
+
+    mprf("The %s cannot attune %s to your%s body.", item_name.c_str(),
+         plural ? "themselves" : "itself", low_hp ? " injured" : " exhausted");
+
     you.activated.set(eq_slot, false);
     return;
 }
@@ -1084,19 +1118,6 @@ bool acrobat_boost_active()
            && (!you.is_constricted());
 }
 
-static void _equip_amulet_of_mana_regeneration()
-{
-    if (!player_regenerates_mp())
-        mpr("The amulet feels cold and inert.");
-    else if (you.magic_points == you.max_magic_points)
-        you.props[MANA_REGEN_AMULET_ACTIVE] = 1;
-    else
-    {
-        mpr("The amulet cannot attune itself to your exhausted body.");
-        you.props[MANA_REGEN_AMULET_ACTIVE] = 0;
-    }
-}
-
 static void _equip_amulet_of_reflection()
 {
     you.redraw_armour_class = true;
@@ -1173,11 +1194,6 @@ static void _equip_jewellery_effect(item_def &item, bool unmeld,
         const string ignore_reason = ignore_faith_reason();
         if (!ignore_reason.empty())
             simple_god_message(ignore_reason.c_str());
-        else if (you_worship(GOD_YREDELEMNUL))
-        {
-            mprf(MSGCH_GOD, "The black torch glows! You feel the dead"
-                            " draw near.");
-        }
         else
         {
             mprf(MSGCH_GOD, "You feel a %ssurge of divine interest.",
@@ -1187,11 +1203,6 @@ static void _equip_jewellery_effect(item_def &item, bool unmeld,
 
         break;
 
-    case AMU_REGENERATION:
-        if (!unmeld)
-            _equip_regeneration_item(item);
-        break;
-
     case AMU_ACROBAT:
         if (you.has_mutation(MUT_TENGU_FLIGHT))
             mpr("You feel no more acrobatic than usual.");
@@ -1199,11 +1210,6 @@ static void _equip_jewellery_effect(item_def &item, bool unmeld,
             mpr("You feel ready to tumble and roll out of harm's way.");
         break;
 
-    case AMU_MANA_REGENERATION:
-        if (!unmeld)
-            _equip_amulet_of_mana_regeneration();
-        break;
-
     case AMU_REFLECTION:
         _equip_amulet_of_reflection();
         break;
@@ -1211,6 +1217,8 @@ static void _equip_jewellery_effect(item_def &item, bool unmeld,
     case AMU_GUARDIAN_SPIRIT:
         _spirit_shield_message(unmeld);
         break;
+
+    // Regeneration and Magic Regeneration amulets handled elsewhere.
     }
 
     if (artefact)
@@ -1223,10 +1231,9 @@ static void _equip_jewellery_effect(item_def &item, bool unmeld,
         mprf_nocap("%s", item.name(DESC_INVENTORY_EQUIP).c_str());
 }
 
-static void _deactivate_regeneration_item(const item_def &item, bool meld)
+static void _deactivate_item(const item_def& item)
 {
-    if (!meld)
-        you.activated.set(get_item_slot(item), false);
+    you.activated.set(get_item_slot(item), false);
 }
 
 static void _unequip_jewellery_effect(item_def &item, bool mesg, bool meld,
@@ -1237,7 +1244,7 @@ static void _unequip_jewellery_effect(item_def &item, bool mesg, bool meld,
     {
     case RING_FIRE:
     case RING_ICE:
-    case RING_LIFE_PROTECTION:
+    case RING_POSITIVE_ENERGY:
     case RING_POISON_RESISTANCE:
     case RING_PROTECTION_FROM_COLD:
     case RING_PROTECTION_FROM_FIRE:
@@ -1248,7 +1255,8 @@ static void _unequip_jewellery_effect(item_def &item, bool mesg, bool meld,
         break;
 
     case AMU_REGENERATION:
-        _deactivate_regeneration_item(item, meld);
+        if (!meld)
+            _deactivate_item(item);
         break;
 
     case RING_SEE_INVISIBLE:
@@ -1305,7 +1313,7 @@ static void _unequip_jewellery_effect(item_def &item, bool mesg, bool meld,
 
     case AMU_MANA_REGENERATION:
         if (!meld)
-            you.props[MANA_REGEN_AMULET_ACTIVE] = 0;
+            _deactivate_item(item);
         break;
     }
 
@@ -1316,20 +1324,19 @@ static void _unequip_jewellery_effect(item_def &item, bool mesg, bool meld,
     calc_mp();
 }
 
-bool unwield_item(bool showMsgs)
+bool unwield_item(const item_def &item, bool showMsgs)
 {
-    if (!you.weapon())
+    if (is_weapon(item) && !safe_to_remove(item))
         return false;
 
-    item_def& item = *you.weapon();
-
-    const bool is_weapon = get_item_slot(item) == EQ_WEAPON;
-
-    if (is_weapon && !safe_to_remove(item))
-        return false;
+    if (you.has_mutation(MUT_SLOW_WIELD))
+    {
+        start_delay<EquipOffDelay>(ARMOUR_EQUIP_DELAY - 1, item,
+                                   you.weapon() == &item /*ew*/);
+        return true;
+    }
 
     unequip_item(EQ_WEAPON, showMsgs);
-
     you.wield_change     = true;
     quiver::set_needs_redraw();
 
