@@ -134,6 +134,9 @@ static bool _decrement_a_duration(duration_type dur, int delay,
             "expiration delay loss %d not less than duration expiration point %d",
             exploss * BASELINE_DELAY, exppoint);
 
+    if (dur == DUR_SENTINEL_MARK && aura_is_active_on_player(OPHAN_MARK_KEY))
+        return false;
+
     const int old_dur = you.duration[dur];
     you.duration[dur] -= delay;
 
@@ -370,7 +373,7 @@ static void _update_cowardice()
         mpr("You feel a twist of horror at the sight of this foe.");
 }
 
-// Uskawyaw piety decays incredibly fast, but only to a baseline level of *.
+// Uskayaw piety decays incredibly fast, but only to a baseline level of *.
 // Using Uskayaw abilities can still take you under *.
 static void _handle_uskayaw_piety(int time_taken)
 {
@@ -464,18 +467,58 @@ void player_reacts_to_monsters()
     if (_decrement_a_duration(DUR_GRASPING_ROOTS, you.time_taken)
         && you.is_constricted())
     {
-        // We handle the end-of-enchantment message here since the method
-        // of constriction is no longer detectable.
-        mprf("The grasping roots release their grip on you.");
+        actor* src = actor_by_mid(you.constricted_by);
+        mprf("%s grasping roots sink back into the ground.",
+             src ? src->name(DESC_ITS).c_str() : "The");
+        you.stop_being_constricted(true);
+    }
+    if (_decrement_a_duration(DUR_VILE_CLUTCH, you.time_taken)
+        && you.is_constricted())
+    {
+        actor* src = actor_by_mid(you.constricted_by);
+        mprf("%s zombie hands return to the earth.",
+             src ? src->name(DESC_ITS).c_str() : "The");
         you.stop_being_constricted(true);
     }
 
     _handle_jinxbite_interest();
 
+    // If you have signalled your allies to stop attacking, cancel this order
+    // once there are no longer any enemies in view for 50 consecutive aut
+    if (you.pet_target == MHITYOU)
+    {
+        // Reset the timer if there are hostiles in sight
+        if (there_are_monsters_nearby(true, true, false))
+            you.duration[DUR_ALLY_RESET_TIMER] = 0;
+        else
+        {
+            if (!you.duration[DUR_ALLY_RESET_TIMER])
+                you.duration[DUR_ALLY_RESET_TIMER] = 50;
+            else if (_decrement_a_duration(DUR_ALLY_RESET_TIMER, you.time_taken))
+                you.pet_target = MHITNOT;
+        }
+    }
+
+    // Expire Blood For Blood much faster when there are no enemies around for
+    // the horde to fight (but not at the tail end of its duration, which could
+    // otherwise make the expiry message misleading)
+    if (you.duration[DUR_BLOOD_FOR_BLOOD] > 40
+        && !there_are_monsters_nearby(true, true, false))
+    {
+        you.duration[DUR_BLOOD_FOR_BLOOD] -= you.time_taken * 3;
+        if (you.duration[DUR_BLOOD_FOR_BLOOD] < 1)
+            you.duration[DUR_BLOOD_FOR_BLOOD] = 1;
+    }
+
     _maybe_melt_armour();
     _update_cowardice();
     if (you_worship(GOD_USKAYAW))
         _handle_uskayaw_time(you.time_taken);
+
+    announce_beogh_conversion_offer();
+
+    if (player_in_branch(BRANCH_ARENA) && !okawaru_duel_active())
+        okawaru_end_duel();
 }
 
 static bool _check_recite()
@@ -566,7 +609,7 @@ static void _decrement_transform_duration(int delay)
         || you.transform_uncancellable)
     {
         if (form_can_fly()
-            || form_likes_water() && feat_is_water(env.grid(you.pos())))
+            || form_can_swim() && feat_is_water(env.grid(you.pos())))
         {
             // Disable emergency flight if it was active
             you.props.erase(EMERGENCY_FLIGHT_KEY);
@@ -791,6 +834,9 @@ static void _decrement_durations()
     if (you.duration[DUR_TOXIC_RADIANCE])
         toxic_radiance_effect(&you, min(delay, you.duration[DUR_TOXIC_RADIANCE]));
 
+    if (you.duration[DUR_FATHOMLESS_SHACKLES])
+        yred_fathomless_shackles_effect(min(delay, you.duration[DUR_FATHOMLESS_SHACKLES]));
+
     if (you.duration[DUR_RECITE] && _check_recite())
     {
         const int old_recite =
@@ -802,12 +848,12 @@ static void _decrement_durations()
             _handle_recitation(new_recite);
     }
 
-    if (you.attribute[ATTR_NEXT_RECALL_INDEX] > 0)
-        do_recall(delay);
-
     if (you.duration[DUR_DRAGON_CALL])
         do_dragon_call(delay);
 
+    if (you.duration[DUR_INFERNAL_LEGION])
+        makhleb_infernal_legion_tick(delay);
+
     if (you.duration[DUR_DOOM_HOWL])
         doom_howl(min(delay, you.duration[DUR_DOOM_HOWL]));
 
@@ -817,7 +863,7 @@ static void _decrement_durations()
     if (!you.cannot_act()
         && !you.confused())
     {
-        extract_manticore_spikes(
+        extract_barbs(
             make_stringf("You %s the barbed spikes from your body.",
                 you.berserk() ? "rip and tear" : "carefully extract").c_str());
     }
@@ -829,8 +875,6 @@ static void _decrement_durations()
         _try_to_respawn_ancestor();
     }
 
-    okawaru_handle_duel();
-
     const bool sanguine_armour_is_valid = sanguine_armour_valid();
     if (sanguine_armour_is_valid)
         activate_sanguine_armour();
@@ -847,6 +891,12 @@ static void _decrement_durations()
     if (you.duration[DUR_TEMP_CLOUD_IMMUNITY])
         _decrement_a_duration(DUR_TEMP_CLOUD_IMMUNITY, delay);
 
+    if (you.duration[DUR_BLOOD_FOR_BLOOD])
+        beogh_blood_for_blood_tick(delay);
+
+    if (you.duration[DUR_FUSILLADE] && you.time_taken > 0)
+        fire_fusillade();
+
     // these should be after decr_ambrosia, transforms, liquefying, etc.
     for (int i = 0; i < NUM_DURATIONS; ++i)
         if (duration_decrements_normally((duration_type) i))
@@ -870,31 +920,37 @@ static void _handle_emergency_flight()
     }
 }
 
-// Regen equipment only begins to function when full health is reached.
-static void _update_equipment_attunement_by_health()
+// Regeneration and Magic Regeneration items only work if the player has reached
+// max hp/mp while they are being worn. This scans and updates such items when
+// the player refills their hp/mp.
+static void _maybe_attune_items(bool attune_regen, bool attune_mana_regen)
 {
-    if (you.hp != you.hp_max
-#if TAG_MAJOR_VERSION == 34
-        || you.get_mutation_level(MUT_NO_REGENERATION)
-#endif
-        )
-    {
+    if (!attune_regen && !attune_mana_regen)
         return;
-    }
 
     vector<string> eq_list;
     bool plural = false;
 
+    bool gained_regen = false;
+    bool gained_mana_regen = false;
+
     for (int slot = EQ_MIN_ARMOUR; slot <= EQ_MAX_WORN; ++slot)
     {
         if (you.melded[slot] || you.equip[slot] == -1 || you.activated[slot])
             continue;
         const item_def &arm = you.inv[you.equip[slot]];
-        if (is_artefact(arm) && artefact_property(arm, ARTP_REGENERATION)
-            || arm.base_type == OBJ_ARMOUR
-               && armour_type_prop(arm.sub_type, ARMF_REGENERATION)
-            || arm.is_type(OBJ_JEWELLERY, AMU_REGENERATION))
+
+        if ((attune_regen && is_regen_item(arm)
+             && (you.magic_points == you.max_magic_points || !is_mana_regen_item(arm)))
+            || (attune_mana_regen && is_mana_regen_item(arm)
+                && (you.hp == you.hp_max || !is_regen_item(arm))))
         {
+            // Track which properties we should notify the player they have gained.
+            if (!gained_regen && is_regen_item(arm))
+                gained_regen = true;
+            if (!gained_mana_regen && is_mana_regen_item(arm))
+                gained_mana_regen = true;
+
             eq_list.push_back(is_artefact(arm) ? get_artefact_name(arm) :
                 slot == EQ_AMULET ? "amulet" :
                 slot != EQ_BODY_ARMOUR ?
@@ -913,29 +969,14 @@ static void _update_equipment_attunement_by_health()
     if (eq_list.empty())
         return;
 
+    const char* msg = (gained_regen && gained_mana_regen) ? " health and magic"
+                       : (gained_regen ? "" : " magic");
+
     plural = plural || eq_list.size() > 1;
     string eq_str = comma_separated_line(eq_list.begin(), eq_list.end());
-    mprf("Your %s attune%s to your body as you begin to regenerate "
-         "more quickly.", eq_str.c_str(), plural ? " themselves" : "s itself");
-}
-
-// Amulet of magic regeneration needs to be worn while at full magic before it
-// begins to function.
-static void _update_mana_regen_amulet_attunement()
-{
-    if (you.wearing(EQ_AMULET, AMU_MANA_REGENERATION)
-        && player_regenerates_mp())
-    {
-        if (you.magic_points == you.max_magic_points
-            && you.props[MANA_REGEN_AMULET_ACTIVE].get_int() == 0)
-        {
-            you.props[MANA_REGEN_AMULET_ACTIVE] = 1;
-            mpr("Your amulet attunes itself to your body and you begin to "
-                "regenerate magic more quickly.");
-        }
-    }
-    else if (!you.melded[EQ_AMULET])
-        you.props[MANA_REGEN_AMULET_ACTIVE] = 0;
+    mprf("Your %s attune%s to your body, and you begin to regenerate%s "
+         "more quickly.", eq_str.c_str(), plural ? " themselves" : "s itself",
+         msg);
 }
 
 // cjo: Handles player hp and mp regeneration. If the counter
@@ -950,6 +991,9 @@ static void _regenerate_hp_and_mp(int delay)
     if (crawl_state.disables[DIS_PLAYER_REGEN])
         return;
 
+    const int old_hp = you.hp;
+    const int old_mp = you.magic_points;
+
     // HP Regeneration
     if (!you.duration[DUR_DEATHS_DOOR])
     {
@@ -972,28 +1016,29 @@ static void _regenerate_hp_and_mp(int delay)
 
     ASSERT_RANGE(you.hit_points_regeneration, 0, 100);
 
-    _update_equipment_attunement_by_health();
-
     // MP Regeneration
-    if (!player_regenerates_mp())
-        return;
-
-    if (you.magic_points < you.max_magic_points)
+    if (player_regenerates_mp())
     {
-        const int base_val = player_mp_regen();
-        int mp_regen_countup = div_rand_round(base_val * delay, BASELINE_DELAY);
-        you.magic_points_regeneration += mp_regen_countup;
+        if (you.magic_points < you.max_magic_points)
+        {
+            const int base_val = player_mp_regen();
+            int mp_regen_countup = div_rand_round(base_val * delay, BASELINE_DELAY);
+            you.magic_points_regeneration += mp_regen_countup;
+        }
+
+        while (you.magic_points_regeneration >= 100)
+        {
+            inc_mp(1);
+            you.magic_points_regeneration -= 100;
+        }
+
+        ASSERT_RANGE(you.magic_points_regeneration, 0, 100);
     }
 
-    while (you.magic_points_regeneration >= 100)
-    {
-        inc_mp(1);
-        you.magic_points_regeneration -= 100;
-    }
-
-    ASSERT_RANGE(you.magic_points_regeneration, 0, 100);
-
-    _update_mana_regen_amulet_attunement();
+    // Update attunement of regeneration items if our hp/mp has refilled.
+    _maybe_attune_items(you.hp != old_hp && you.hp == you.hp_max,
+                        you.magic_points != old_mp
+                        && you.magic_points == you.max_magic_points);
 }
 
 static void _handle_fugue(int delay)
@@ -1005,7 +1050,7 @@ static void _handle_fugue(int delay)
     {
         // Keep the spam down
         if (you.props[FUGUE_KEY].get_int() < 3 || one_chance_in(5))
-            mprf("The wailing of tortured souls fills the air!");
+            mpr("The wailing of tortured souls fills the air!");
         noisy(spell_effect_noise(SPELL_FUGUE_OF_THE_FALLEN), you.pos());
     }
 }
@@ -1016,6 +1061,10 @@ void player_reacts()
     if (crawl_state.game_is_descent() && !env.properties.exists(DESCENT_STAIRS_KEY))
         return;
 
+    // This happens as close as possible after the player acts, for better messaging
+    if (you_worship(GOD_BEOGH))
+        beogh_ally_healing();
+
     //XXX: does this _need_ to be calculated up here?
     const int stealth = player_stealth();
 
@@ -1028,6 +1077,8 @@ void player_reacts()
         unrand_reacts();
 
     _handle_fugue(you.time_taken);
+    if (you.has_mutation(MUT_WARMUP_STRIKES))
+        you.rev_down(you.time_taken);
 
     if (x_chance_in_y(you.time_taken, 10 * BASELINE_DELAY))
     {
@@ -1051,11 +1102,8 @@ void player_reacts()
     abyss_maybe_spawn_xp_exit();
 
     actor_apply_cloud(&you);
-    // Miasma immunity from Dreadful Rot. Only lasts for one turn,
-    // so erase it just after we apply clouds for the turn (above).
-    if (you.props.exists(MIASMA_IMMUNE_KEY))
-        you.props.erase(MIASMA_IMMUNE_KEY);
-    // Ditto for blastmotes.
+    // Immunity due to just casting Volatile Blastmotes. Only lasts for one
+    // turn, so erase it just after we apply clouds for the turn (above).
     if (you.props.exists(BLASTMOTE_IMMUNE_KEY))
         you.props.erase(BLASTMOTE_IMMUNE_KEY);
 
@@ -1071,11 +1119,14 @@ void player_reacts()
 
     you.handle_constriction();
 
-    // increment constriction durations
-    you.accum_has_constricted();
-
     _regenerate_hp_and_mp(you.time_taken);
 
+    if (you.duration[DUR_CELEBRANT_COOLDOWN] && you.hp == you.hp_max)
+    {
+        mprf(MSGCH_DURATION, "You are ready to perform a blood rite again.");
+        you.duration[DUR_CELEBRANT_COOLDOWN] = 0;
+    }
+
     if (you.duration[DUR_POISONING])
         handle_player_poison(you.time_taken);
 
@@ -1097,11 +1148,14 @@ void player_reacts()
     if (you.props[EMERGENCY_FLIGHT_KEY].get_bool())
         _handle_emergency_flight();
 
+    if (you.duration[DUR_PRIMORDIAL_NIGHTFALL])
+        update_vision_range();
+
     incr_gem_clock();
     incr_zot_clock();
 }
 
-void extract_manticore_spikes(const char* endmsg)
+void extract_barbs(const char* endmsg)
 {
     if (_decrement_a_duration(DUR_BARBS, you.time_taken, endmsg))
     {
