@@ -4,7 +4,9 @@
 #include "mpr.h"
 #include "species.h"
 
+#include "branch.h"
 #include "item-prop.h"
+#include "items.h"
 #include "mutation.h"
 #include "output.h"
 #include "playable.h"
@@ -15,6 +17,7 @@
 #include "stringutil.h"
 #include "tag-version.h"
 #include "tiledoll.h"
+#include "travel.h"
 
 #include "species-data.h"
 
@@ -131,11 +134,6 @@ namespace species
         return species == SP_DEEP_ELF;
     }
 
-    bool is_orcish(species_type species)
-    {
-        return species == SP_HILL_ORC;
-    }
-
     bool is_undead(species_type species)
     {
         return undead_type(species) != US_ALIVE;
@@ -186,11 +184,11 @@ namespace species
             case SP_WHITE_DRACONIAN:
                 return "icy white";
             case SP_GREEN_DRACONIAN:
-                return "lurid green";
+                return "mossy green";
             case SP_YELLOW_DRACONIAN:
-                return "golden yellow";
+                return "lurid yellow";
             case SP_GREY_DRACONIAN:
-                return "dull iron-grey";
+                return "dull grey";
             case SP_BLACK_DRACONIAN:
                 return "glossy black";
             case SP_PURPLE_DRACONIAN:
@@ -213,7 +211,7 @@ namespace species
         case SP_GREEN_DRACONIAN:
             return MONS_SWAMP_DRAGON;
         case SP_YELLOW_DRACONIAN:
-            return MONS_GOLDEN_DRAGON;
+            return MONS_ACID_DRAGON;
         case SP_GREY_DRACONIAN:
             return MONS_IRON_DRAGON;
         case SP_BLACK_DRACONIAN:
@@ -232,14 +230,15 @@ namespace species
     {
         switch (species)
         {
-        case SP_GREEN_DRACONIAN:   return ABIL_BREATHE_MEPHITIC;
-        case SP_RED_DRACONIAN:     return ABIL_BREATHE_FIRE;
-        case SP_WHITE_DRACONIAN:   return ABIL_BREATHE_FROST;
-        case SP_YELLOW_DRACONIAN:  return ABIL_BREATHE_ACID;
-        case SP_BLACK_DRACONIAN:   return ABIL_BREATHE_LIGHTNING;
-        case SP_PURPLE_DRACONIAN:  return ABIL_BREATHE_POWER;
-        case SP_PALE_DRACONIAN:    return ABIL_BREATHE_STEAM;
-        case SP_BASE_DRACONIAN: case SP_GREY_DRACONIAN:
+        case SP_GREEN_DRACONIAN:   return ABIL_NOXIOUS_BREATH;
+        case SP_RED_DRACONIAN:     return ABIL_COMBUSTION_BREATH;
+        case SP_WHITE_DRACONIAN:   return ABIL_GLACIAL_BREATH;
+        case SP_YELLOW_DRACONIAN:  return ABIL_CAUSTIC_BREATH;
+        case SP_BLACK_DRACONIAN:   return ABIL_GALVANIC_BREATH;
+        case SP_PURPLE_DRACONIAN:  return ABIL_NULLIFYING_BREATH;
+        case SP_PALE_DRACONIAN:    return ABIL_STEAM_BREATH;
+        case SP_GREY_DRACONIAN:    return ABIL_MUD_BREATH;
+        case SP_BASE_DRACONIAN:
         default: return ABIL_NON_ABILITY;
         }
     }
@@ -268,9 +267,14 @@ namespace species
                      : get_species_def(species).verbose_fake_mutations;
     }
 
+    bool has_blood(species_type species)
+    {
+        return !bool(get_species_def(species).flags & SPF_NO_BLOOD);
+    }
+
     bool has_hair(species_type species)
     {
-        return !bool(get_species_def(species).flags & (SPF_NO_HAIR | SPF_DRACONIAN));
+        return !bool(get_species_def(species).flags & SPF_NO_HAIR);
     }
 
     bool has_bones(species_type species)
@@ -278,6 +282,16 @@ namespace species
         return !bool(get_species_def(species).flags & SPF_NO_BONES);
     }
 
+    bool has_feet(species_type species)
+    {
+        return !bool(get_species_def(species).flags & SPF_NO_FEET);
+    }
+
+    bool has_ears(species_type species)
+    {
+        return !bool(get_species_def(species).flags & SPF_NO_EARS);
+    }
+
     bool can_throw_large_rocks(species_type species)
     {
         return size(species) >= SIZE_LARGE;
@@ -358,6 +372,18 @@ namespace species
         return verb ? verb : "Child";
     }
 
+    /**
+     * What is an appropriate name for orcs of this species?
+     *
+     *  @param sp what kind of species to look at
+     *  @returns something with 'orc' in it.
+     */
+    string orc_name(species_type sp)
+    {
+        auto verb = get_species_def(sp).orc_name;
+        return verb ? verb : "Orc";
+    }
+
     /**
      * What message should be printed when a character of the specified species
      * prays at an altar, if not in some form?
@@ -425,6 +451,8 @@ namespace species
             return adj ? "furry" : "fur";
         else if (species == SP_MUMMY)
             return adj ? "bandage-wrapped" : "bandages";
+        else if (species == SP_GARGOYLE)
+            return adj ? "stony" : "stone";
         else
             return adj ? "fleshy" : "skin";
     }
@@ -448,6 +476,8 @@ namespace species
             return "tentacle";
         else if (mutation_level(species, MUT_CLAWS))
             return "claw"; // overridden for felids by first check
+        else if (species == SP_COGLIN)
+            return "grasper";
         else
             return "hand";
     }
@@ -512,7 +542,7 @@ namespace species
             break;
         }
         // remaining should be armour only
-        if (species == SP_OCTOPODE && eq != EQ_HELMET && eq != EQ_SHIELD)
+        if (species == SP_OCTOPODE && eq != EQ_HELMET && eq != EQ_OFFHAND)
             return true;
 
         if (is_draconian(species) && eq == EQ_BODY_ARMOUR)
@@ -532,6 +562,8 @@ namespace species
     vector<equipment_type> ring_slots(species_type species, bool missing_hand)
     {
         vector<equipment_type> result;
+        if (you.has_mutation(MUT_NO_JEWELLERY))
+            return result;
 
         const equipment_type missing = missing_hand
                             ? sacrificial_arm(species) : EQ_NONE;
@@ -651,6 +683,30 @@ namespace species
     }
 }
 
+int draconian_breath_uses_available()
+{
+    if (!species::is_draconian(you.species))
+        return 0;
+
+    if (!you.props.exists(DRACONIAN_BREATH_USES_KEY))
+        return 0;
+
+    return you.props[DRACONIAN_BREATH_USES_KEY].get_int();
+}
+
+// Attempts to gain num uses of our draconian breath weapon.
+// Returns true if any were gained (ie: we were not already capped)
+bool gain_draconian_breath_uses(int num)
+{
+    int cur = draconian_breath_uses_available();
+
+    if (cur >= MAX_DRACONIAN_BREATH)
+        return false;
+
+    you.props[DRACONIAN_BREATH_USES_KEY] = min(MAX_DRACONIAN_BREATH, cur + num);
+    return true;
+}
+
 void give_basic_mutations(species_type species)
 {
     // Don't perma_mutate since that gives messages.
@@ -773,16 +829,32 @@ void change_species_to(species_type sp)
     // FIXME: this checks only for valid slots, not for suitability of the
     // item in question. This is enough to make assertions happy, though.
     for (int i = EQ_FIRST_EQUIP; i < NUM_EQUIP; ++i)
-        if (bool(!you_can_wear(static_cast<equipment_type>(i)))
-            && you.equip[i] != -1)
+    {
+        if (you.equip[i] != -1)
         {
-            mprf("%s fall%s away.",
-                 you.inv[you.equip[i]].name(DESC_YOUR).c_str(),
-                 you.inv[you.equip[i]].quantity > 1 ? "" : "s");
-            // Unwear items without the usual processing.
-            you.equip[i] = -1;
-            you.melded.set(i, false);
+            if (i == EQ_WEAPON && you.inv[you.equip[i]].base_type == OBJ_WEAPONS
+                && sp == SP_COGLIN)
+            {
+                // Coglins' wielded non-offhand weapons need names. Without
+                // them, wielding a non-offhand weapon, changing species to
+                // coglin, and then trying to unwield that weapon will assert.
+                maybe_name_weapon(you.inv[you.equip[i]], true);
+            }
+
+            if (bool(!you_can_wear(static_cast<equipment_type>(i)))
+                // XXX: This can't be caught by you_can_wear as everyone has offhand slots
+                || (i == EQ_OFFHAND && you.inv[you.equip[i]].base_type == OBJ_WEAPONS
+                    && !you.has_mutation(MUT_WIELD_OFFHAND)))
+            {
+                mprf("%s fall%s away.",
+                     you.inv[you.equip[i]].name(DESC_YOUR).c_str(),
+                     you.inv[you.equip[i]].quantity > 1 ? "" : "s");
+                // Unwear items without the usual processing.
+                you.equip[i] = -1;
+                you.melded.set(i, false);
+            }
         }
+    }
 
     // Sanitize skills.
     fixup_skills();
