@@ -33,7 +33,6 @@ namespace species
                                                     bool initial_only = false);
 
     bool is_elven(species_type species);
-    bool is_orcish(species_type species);
     bool is_undead(species_type species);
     bool is_draconian(species_type species);
     undead_state_type undead_type(species_type species) PURE;
@@ -46,8 +45,13 @@ namespace species
 
     int mutation_level(species_type species, mutation_type mut, int mut_level=1);
     const vector<string>& fake_mutations(species_type species, bool terse);
+
+    bool has_blood(species_type species);
     bool has_hair(species_type species);
     bool has_bones(species_type species);
+    bool has_feet(species_type species);
+    bool has_ears(species_type species);
+
     bool can_throw_large_rocks(species_type species);
     bool wears_barding(species_type species);
     bool has_claws(species_type species);
@@ -59,6 +63,7 @@ namespace species
     string walking_verb(species_type sp);
     string walking_title(species_type sp);
     string child_name(species_type species);
+    string orc_name(species_type species);
     string prayer_action(species_type species);
     string shout_verb(species_type sp, int screaminess, bool directed);
     string skin_name(species_type sp, bool adj=false);
@@ -84,6 +89,13 @@ namespace species
     vector<species_type> get_all_species();
 }
 
+#define DRACONIAN_BREATH_USES_KEY "drac_breath_uses"
+#define DRACONIAN_BREATH_RECHARGE_KEY "drac_breath_recharge"
+const int MAX_DRACONIAN_BREATH = 3;
+
+int draconian_breath_uses_available();
+bool gain_draconian_breath_uses(int num);
+
 void species_stat_init(species_type species);
 void species_stat_gain(species_type species);
 
