@@ -34,6 +34,7 @@
 #include "random-pick.h"
 #include "religion.h"
 #include "spl-cast.h"
+#include "spl-summoning.h"
 #include "spl-util.h"
 #include "state.h"
 #include "stringutil.h"
@@ -100,6 +101,7 @@ static const map<wand_type, spell_type> _wand_spells =
     { WAND_QUICKSILVER, SPELL_QUICKSILVER_BOLT },
     { WAND_MINDBURST, SPELL_MINDBURST },
     { WAND_ROOTS, SPELL_FASTROOT },
+    { WAND_WARPING, SPELL_WARP_SPACE },
 };
 
 
@@ -235,14 +237,25 @@ static unordered_set<int> _player_nonbook_spells =
                           // least uses the enum value.
     SPELL_TREMORSTONE,    // not cast directly, but the spell type is used for
                           // damage and noise display.
+    SPELL_GRAVITAS,
     SPELL_SONIC_WAVE,
     // religion
     SPELL_SMITING,
-    SPELL_MINOR_DESTRUCTION,
+    SPELL_UNLEASH_DESTRUCTION,
+    SPELL_HURL_TORCHLIGHT,
     // Ds powers
     SPELL_HURL_DAMNATION,
-    // Green Draconian breath
-    SPELL_MEPHITIC_BREATH,
+    // Draconian breaths
+    SPELL_NOXIOUS_BREATH,
+    SPELL_COMBUSTION_BREATH,
+    SPELL_GLACIAL_BREATH,
+    SPELL_NULLIFYING_BREATH,
+    SPELL_STEAM_BREATH,
+    SPELL_CAUSTIC_BREATH,
+    SPELL_GALVANIC_BREATH,
+    SPELL_MUD_BREATH,
+    // Activate component of Seismic Cannonade
+    SPELL_SEISMIC_SHOCKWAVE,
 };
 
 bool is_player_spell(spell_type which_spell)
@@ -414,10 +427,9 @@ bool library_add_spells(vector<spell_type> spells, bool quiet)
         if (!you.spell_library[st])
         {
             you.spell_library.set(st, true);
-            bool memorise = you_can_memorise(st);
-            if (memorise)
+            if (you_can_memorise(st))
                 new_spells.push_back(st);
-            if (!memorise || Options.auto_hide_spells)
+            if (Options.auto_hide_spells)
                 you.hidden_spells.set(st, true);
         }
     }
@@ -509,7 +521,8 @@ vector<spell_type> get_sorted_spell_list(bool silent, bool memorise_only)
 class SpellLibraryMenu : public Menu
 {
 public:
-    enum class action { cast, memorise, describe, hide, unhide } current_action;
+    enum class action { cast, memorise, imbue, describe, hide, unhide };
+    action current_action, default_action;
 
 protected:
     virtual formatted_string calc_title() override
@@ -520,6 +533,7 @@ protected:
                         : current_action == action::memorise ? "(Memorise)"
                         : current_action == action::describe ? "(Describe)"
                         : current_action == action::hide ? "(Hide)    "
+                        : current_action == action::imbue ? "(Imbue)   "
                         : "(Show)    ",
                         you.divine_exegesis ? "         " : "Failure  "));
     }
@@ -573,13 +587,16 @@ private:
 
         desc << "\n";
 
-        const string act = you.divine_exegesis ? "Cast" : "Memorise";
+        const string act = default_action == action::memorise ? "Memorise"
+                           : default_action == action::imbue ? "Imbue" : "Cast";
         // line 2
         desc << menu_keyhelp_cmd(CMD_MENU_CYCLE_MODE) << " ";
         desc << ( current_action == action::cast
                             ? "<w>Cast</w>|Describe|Hide|Show"
                  : current_action == action::memorise
                             ? "<w>Memorise</w>|Describe|Hide|Show"
+                 : current_action == action::imbue
+                            ? "<w>Imbue</w>|Describe|Hide|Show"
                  : current_action == action::describe
                             ? act + "|<w>Describe</w>|Hide|Show"
                  : current_action == action::hide
@@ -604,6 +621,7 @@ private:
             {
                 case action::cast:
                 case action::memorise:
+                case action::imbue:
                     current_action = action::describe;
                     entries_changed = true; // need to add hotkeys
                     break;
@@ -617,8 +635,7 @@ private:
                         last_hovered = 0;
                     break;
                 case action::unhide:
-                    current_action = you.divine_exegesis ? action::cast
-                                                          : action::memorise;
+                    current_action = default_action;
                     entries_changed = true;
                     if (last_hovered >= 0 && is_set(MF_ARROWS_SELECT))
                         last_hovered = 0;
@@ -631,14 +648,14 @@ private:
             {
                 case action::cast:
                 case action::memorise:
+                case action::imbue:
                     current_action = action::unhide;
                     entries_changed = true;
                     if (last_hovered >= 0 && is_set(MF_ARROWS_SELECT))
                         last_hovered = 0;
                     break;
                 case action::describe:
-                    current_action = you.divine_exegesis ? action::cast
-                                                         : action::memorise;
+                    current_action = default_action;
                     entries_changed = true; // may need to remove hotkeys
                     break;
                 case action::hide:
@@ -812,12 +829,13 @@ private:
     }
 
 public:
-    SpellLibraryMenu(spell_list& list)
+    SpellLibraryMenu(spell_list& list, action _default_action)
         : Menu(MF_SINGLESELECT | MF_ALLOW_FORMATTING
                 | MF_ARROWS_SELECT | MF_INIT_HOVER | MF_SHOW_EMPTY
                 // To have the ctrl-f menu show up in webtiles
                 | MF_ALLOW_FILTER, "spell"),
-        current_action(you.divine_exegesis ? action::cast : action::memorise),
+        current_action(_default_action),
+        default_action(_default_action),
         spells(list),
         hidden_count(0)
     {
@@ -831,6 +849,8 @@ public:
                 "<lightgreen>Select a spell to cast with Divine Exegesis: %d MP available</lightgreen>",
                 you.magic_points);
         }
+        else if (default_action == action::imbue)
+            spell_levels_str = "<lightgreen>Select a spell to imbue your Spellforged Servitor with:</lightgreen>";
         else
         {
             spell_levels_str = make_stringf("<lightgreen>%d spell level%s"
@@ -868,6 +888,7 @@ public:
             {
             case action::memorise:
             case action::cast:
+            case action::imbue:
                 return false;
             case action::describe:
                 // n.b. skip superclass handling of ACT_EXAMINE, since we
@@ -894,7 +915,7 @@ static spell_type _choose_mem_spell(spell_list &spells)
     // If we've gotten this far, we know that at least one spell here is
     // memorisable, which is enough.
 
-    SpellLibraryMenu spell_menu(spells);
+    SpellLibraryMenu spell_menu(spells, SpellLibraryMenu::action::memorise);
 
     const vector<MenuEntry*> sel = spell_menu.show();
     if (!crawl_state.doing_prev_cmd_again)
@@ -1127,7 +1148,7 @@ spret divine_exegesis(bool fail)
     sort(spells.begin(), spells.end(), _sort_divine_spells);
     // If we've gotten this far, we know at least one useful spell.
 
-    SpellLibraryMenu spell_menu(spells);
+    SpellLibraryMenu spell_menu(spells, SpellLibraryMenu::action::cast);
 
     const vector<MenuEntry*> sel = spell_menu.show();
     if (!crawl_state.doing_prev_cmd_again)
@@ -1148,6 +1169,50 @@ spret divine_exegesis(bool fail)
     return cast_a_spell(false, spell, nullptr, fail);
 }
 
+static spell_list _get_player_servitor_spells()
+{
+    spell_list spells;
+    for (const spell_type spell : you.spells)
+        if (spell_servitorable(spell) && failure_rate_to_int(raw_spell_fail(spell)) <= 20)
+            spells.push_back(spell);
+
+    return spells;
+}
+
+spret imbue_servitor()
+{
+    spell_list spells(_get_player_servitor_spells());
+    if (spells.empty())
+    {
+        mpr("You don't know any spells that your servitor could cast!");
+        return spret::abort;
+    }
+
+    SpellLibraryMenu spell_menu(spells, SpellLibraryMenu::action::imbue);
+
+    const vector<MenuEntry*> sel = spell_menu.show();
+    if (!crawl_state.doing_prev_cmd_again)
+    {
+        redraw_screen();
+        update_screen();
+    }
+
+    if (sel.empty())
+        return spret::abort;
+
+    const spell_type spell = *static_cast<spell_type*>(sel[0]->data);
+    if (spell == SPELL_NO_SPELL)
+        return spret::abort;
+
+    ASSERT(is_valid_spell(spell));
+
+    // Remove any servitors that currently exist and start imbuing.
+    remove_player_servitor();
+    start_delay<ImbueDelay>(5, spell);
+
+    return spret::success;
+}
+
 /// For a given dungeon depth (or item level), how much weight should we give
 /// to a spellbook for having a spell of the given level in it?
 /// We add a flat 10 to all these weights later.
