@@ -31,7 +31,7 @@ spret cast_sublimation_of_blood(int pow, bool fail)
 
     if (you.duration[DUR_DEATHS_DOOR])
         mpr("You can't draw power from your own body while in death's door.");
-    else if (!you.can_bleed())
+    else if (!you.has_blood())
     {
         if (you.has_mutation(MUT_VAMPIRISM))
             mpr("You don't have enough blood to draw power from your own body.");
@@ -126,13 +126,11 @@ spret cast_animate_dead(int pow, bool fail)
     return spret::success;
 }
 
-void start_recall(recall_t type)
+void do_player_recall(recall_t type)
 {
-    // Assemble the recall list.
-    typedef pair<mid_t, int> mid_hd;
-    vector<mid_hd> rlist;
+    bool did_recall = false;
 
-    you.recall_list.clear();
+    // Search for recallable allies on your current floor
     for (monster_iterator mi; mi; ++mi)
     {
         if (!mons_is_recallable(&you, **mi))
@@ -145,33 +143,26 @@ void start_recall(recall_t type)
         }
         else if (type == recall_t::beogh)
         {
-            if (!is_orcish_follower(**mi))
+            if (!is_apostle_follower(**mi))
                 continue;
         }
 
-        mid_hd m(mi->mid, mi->get_experience_level());
-        rlist.push_back(m);
+        if (try_recall(mi->mid))
+            did_recall = true;
     }
 
-    if (branch_allows_followers(you.where_are_you))
-        populate_offlevel_recall_list(rlist);
-
-    if (!rlist.empty())
+    // Then search for recallable companions on any floor
+    for (auto &entry : companion_list)
     {
-        // Sort the recall list roughly
-        for (mid_hd &entry : rlist)
-            entry.second += random2(10);
-        sort(rlist.begin(), rlist.end(), greater_second<mid_hd>());
-
-        you.recall_list.clear();
-        for (mid_hd &entry : rlist)
-            you.recall_list.push_back(entry.first);
-
-        you.attribute[ATTR_NEXT_RECALL_INDEX] = 1;
-        you.attribute[ATTR_NEXT_RECALL_TIME] = 0;
-        mpr("You begin recalling your allies.");
+        const int mid = entry.first;
+        if (companion_is_elsewhere(mid, true))
+        {
+            if (try_recall(mid))
+                did_recall = true;
+        }
     }
-    else
+
+    if (!did_recall)
         mpr("Nothing appears to have answered your call.");
 }
 
@@ -184,6 +175,7 @@ void recall_orders(monster *mons)
 
     // Don't patrol
     mons->patrol_point = coord_def(0, 0);
+    mons->travel_path.clear();
 
     // Don't wander
     mons->behaviour = BEH_SEEK;
@@ -225,41 +217,6 @@ bool try_recall(mid_t mid)
     return true;
 }
 
-// Attempt to recall a number of allies proportional to how much time
-// has passed. Once the list has been fully processed, terminate the
-// status.
-void do_recall(int time)
-{
-    while (time > you.attribute[ATTR_NEXT_RECALL_TIME])
-    {
-        // Try to recall an ally.
-        mid_t mid = you.recall_list[you.attribute[ATTR_NEXT_RECALL_INDEX]-1];
-        you.attribute[ATTR_NEXT_RECALL_INDEX]++;
-        if (try_recall(mid))
-        {
-            time -= you.attribute[ATTR_NEXT_RECALL_TIME];
-            you.attribute[ATTR_NEXT_RECALL_TIME] = 3 + random2(4);
-        }
-        if ((unsigned int)you.attribute[ATTR_NEXT_RECALL_INDEX] >
-             you.recall_list.size())
-        {
-            end_recall();
-            mpr("You finish recalling your allies.");
-            return;
-        }
-    }
-
-    you.attribute[ATTR_NEXT_RECALL_TIME] -= time;
-    return;
-}
-
-void end_recall()
-{
-    you.attribute[ATTR_NEXT_RECALL_INDEX] = 0;
-    you.attribute[ATTR_NEXT_RECALL_TIME] = 0;
-    you.recall_list.clear();
-}
-
 static bool _feat_is_passwallable(dungeon_feature_type feat)
 {
     // Worked stone walls are out, they're not diggable and
@@ -275,10 +232,16 @@ static bool _feat_is_passwallable(dungeon_feature_type feat)
     }
 }
 
+static bool _grid_is_passwallable(coord_def pos)
+{
+    return _feat_is_passwallable(env.grid(pos))
+            && !is_temp_terrain(pos);
+}
+
 bool passwall_simplified_check(const actor &act)
 {
     for (adjacent_iterator ai(act.pos(), true); ai; ++ai)
-        if (_feat_is_passwallable(env.grid(*ai)))
+        if (_grid_is_passwallable(*ai))
             return true;
     return false;
 }
@@ -300,7 +263,7 @@ passwall_path::passwall_path(const actor &act, const coord_def& dir, int max_ran
         path.emplace_back(pos);
         if (in_bounds(pos))
         {
-            if (!_feat_is_passwallable(env.grid(pos)))
+            if (!_grid_is_passwallable(pos))
             {
                 if (!dest_found)
                 {
@@ -631,22 +594,22 @@ spret cast_sigil_of_binding(int pow, bool fail, bool tracer)
 
 void trigger_binding_sigil(actor& actor)
 {
+    if (actor.is_binding_sigil_immune())
+    {
+        mprf("%s cannot be bound by the sigil due to %s high momentum!",
+             actor.name(DESC_THE).c_str(), actor.pronoun(PRONOUN_POSSESSIVE).c_str());
+        return;
+    }
+
     if (actor.is_player())
     {
-        mprf(MSGCH_WARN, "You move over your own binding sigil and are bound in place!");
+        mprf(MSGCH_WARN, "You move over the binding sigil and are bound in place!");
         you.increase_duration(DUR_NO_MOMENTUM, random_range(3, 6));
         revert_terrain_change(you.pos(), TERRAIN_CHANGE_BINDING_SIGIL);
         return;
     }
 
     monster* m = actor.as_monster();
-    if (m->has_ench(ENCH_SWIFT))
-    {
-        mprf("%s has too much momentum for your sigil to bind %s!",
-             m->name(DESC_THE).c_str(), m->pronoun(PRONOUN_OBJECTIVE).c_str());
-        return;
-    }
-
     const int pow = calc_spell_power(SPELL_SIGIL_OF_BINDING);
     const int dur = max(2, random_range(4 + div_rand_round(pow, 12),
                                         7 + div_rand_round(pow, 8))
@@ -656,8 +619,8 @@ void trigger_binding_sigil(actor& actor)
     if (m->add_ench(mon_enchant(ENCH_BOUND, 0, &you, dur)))
     {
         simple_monster_message(*m,
-            " moves over your binding sigil and is bound in place!",
-            MSGCH_FRIEND_SPELL);
+            " moves over the binding sigil and is bound in place!",
+            false, MSGCH_FRIEND_SPELL);
 
         // The enemy will gain swift for twice as long as it was bound
         m->props[BINDING_SIGIL_DURATION_KEY] = dur * 2;
