@@ -41,6 +41,7 @@
 #include "mgen-data.h"
 #include "mon-abil.h"
 #include "mon-act.h"
+#include "mon-aura.h"
 #include "mon-behv.h"
 #include "mon-book.h" // MON_SPELL_WIZARD
 #include "mon-cast.h"
@@ -54,6 +55,7 @@
 #include "prompt.h"
 #include "religion.h"
 #include "shout.h"
+#include "spl-book.h"
 #include "spl-util.h"
 #include "spl-zap.h"
 #include "state.h"
@@ -201,6 +203,14 @@ spret cast_call_canine_familiar(int pow, god_type god, bool fail)
         old_dog->heal(random_range(5, 9) + div_rand_round(pow, 5));
         old_dog->lose_ench_levels(ENCH_POISON, 1);
         old_dog->add_ench(mon_enchant(ENCH_INSTANT_CLEAVE, 1, &you, 50));
+
+        // Give our familiar a small amount of extra duration, if its duration
+        // is currently low, to avoid imbuing it and then having it immediately
+        // poof before it can even do anything with the buff.
+        mon_enchant abj = old_dog->get_ench(ENCH_ABJ);
+        if (abj.duration < 110)
+            abj.duration += random_range(60, 90);
+        old_dog->update_ench(abj);
     }
 
     return spret::success;
@@ -697,7 +707,7 @@ bool summon_holy_warrior(int pow, bool punish)
  **/
 static bool _fail_tukimas()
 {
-    mprf("You can't see a target there!");
+    mpr("You can't see a target there!");
     return false; // Waste the turn - no anti-invis tech
 }
 
@@ -716,7 +726,8 @@ bool tukima_affects(const actor &target)
            && !is_special_unrandom_artefact(*wpn)
            && !mons_class_is_animated_weapon(target.type)
            // XX use god_protects here. But, need to know the caster too...
-           && !mons_is_hepliaklqana_ancestor(target.type);
+           && !mons_is_hepliaklqana_ancestor(target.type)
+           && !(target.is_monster() && target.as_monster()->type == MONS_ORC_APOSTLE);
 }
 
 /**
@@ -897,6 +908,19 @@ spret cast_conjure_ball_lightning(int pow, god_type god, bool fail)
     return spret::success;
 }
 
+static int _lightning_spire_hd(int pow, bool random = true)
+{
+    if (random)
+        return max(1, div_rand_round(pow, 10));
+    else
+        return max(1, pow / 10);
+}
+
+dice_def lightning_spire_damage(int pow)
+{
+    return zap_damage(ZAP_ELECTRICAL_BOLT, _lightning_spire_hd(pow, false) * 12, true, false);
+}
+
 spret cast_summon_lightning_spire(int pow, god_type god, bool fail)
 {
     if (stop_summoning_prompt(MR_RES_POISON))
@@ -906,7 +930,7 @@ spret cast_summon_lightning_spire(int pow, god_type god, bool fail)
 
     mgen_data spire = _pal_data(MONS_LIGHTNING_SPIRE, 2, god,
                                 SPELL_SUMMON_LIGHTNING_SPIRE);
-    spire.hd = max(1, div_rand_round(pow, 10));
+    spire.hd = _lightning_spire_hd(pow);
 
     monster* mons = create_monster(spire);
 
@@ -928,7 +952,7 @@ spret cast_summon_blazeheart_golem(int pow, god_type god, bool fail)
     mgen_data golem = _pal_data(MONS_BLAZEHEART_GOLEM, 3, god,
                                 SPELL_SUMMON_BLAZEHEART_GOLEM);
     golem.flags &= ~MG_AUTOFOE; // !!!
-    golem.hd = 6 + div_rand_round(pow, 12);
+    golem.hd = 6 + div_rand_round(pow, 10);
 
     monster* mons = (create_monster(golem));
 
@@ -973,96 +997,6 @@ spret cast_call_imp(int pow, god_type god, bool fail)
     return spret::success;
 }
 
-static bool _summon_demon_wrapper(int pow, god_type god, int spell,
-                                  monster_type mon, int dur, bool friendly,
-                                  bool charmed)
-{
-    bool success = false;
-
-    if (monster *demon = create_monster(
-            mgen_data(mon,
-                      friendly ? BEH_FRIENDLY :
-                       charmed ? BEH_CHARMED
-                               : BEH_HOSTILE,
-                      you.pos(), MHITYOU, MG_FORCE_BEH | MG_AUTOFOE)
-            .set_summoned(&you, dur, spell, god)))
-    {
-        success = true;
-
-        mpr("A demon appears!");
-
-        if (!friendly)
-        {
-            mpr(charmed ? "You don't feel so good about this..."
-                        : "It doesn't seem very happy.");
-        }
-        else if (mon == MONS_CRIMSON_IMP || mon == MONS_WHITE_IMP
-                || mon == MONS_IRON_IMP || mon == MONS_SHADOW_IMP)
-        {
-            _monster_greeting(demon, "_friendly_imp_greeting");
-        }
-
-        if (charmed && !friendly)
-        {
-            int charm_dur = random_range(15 + pow / 14, 27 + pow / 11)
-                            * BASELINE_DELAY;
-
-            mon_enchant charm = demon->get_ench(ENCH_CHARM);
-            charm.duration = charm_dur;
-            demon->update_ench(charm);
-
-            // Ensure that temporarily-charmed demons will outlast their charm
-            mon_enchant abj = demon->get_ench(ENCH_ABJ);
-            if (charm.duration + 100 > abj.duration)
-            {
-                abj.duration = charm.duration + 100;
-                demon->update_ench(abj);
-            }
-
-            // Affects messaging, and stuns demon a turn upon charm wearing off
-            demon->props[CHARMED_DEMON_KEY].get_bool() = true;
-        }
-    }
-
-    return success;
-}
-
-static bool _summon_common_demon(int pow, god_type god, int spell)
-{
-    const int chance = 70 - (pow / 3);
-    monster_type type = MONS_PROGRAM_BUG;
-
-    if (x_chance_in_y(chance, 100))
-        type = random_demon_by_tier(4);
-    else
-        type = random_demon_by_tier(3);
-
-    return _summon_demon_wrapper(pow, god, spell, type,
-                                 min(2 + (random2(pow) / 4), 6),
-                                 random2(pow) > 3, false);
-}
-
-bool summon_demon_type(monster_type mon, int pow, god_type god,
-                       int spell, bool friendly)
-{
-    return _summon_demon_wrapper(pow, god, spell, mon,
-                                 min(2 + (random2(pow) / 4), 6),
-                                 friendly, false);
-}
-
-spret cast_summon_demon(int pow)
-{
-    // Don't prompt here, since this is invoked automatically by the
-    // obsidian axe. The player shouldn't have control.
-
-    mpr("You open a gate to Pandemonium!");
-
-    if (!_summon_common_demon(pow, GOD_NO_GOD, SPELL_SUMMON_DEMON))
-        canned_msg(MSG_NOTHING_HAPPENS);
-
-    return spret::success;
-}
-
 static bool _butterfly_knockback(coord_def p)
 {
     monster* mon = monster_at(p);
@@ -1070,7 +1004,7 @@ static bool _butterfly_knockback(coord_def p)
         return false;
 
     const int dist = random_range(2, 4);
-    if (!mon->knockback(you, dist, -1, "sudden gust"))
+    if (!mon->knockback(you, dist, 0, "sudden gust"))
         return false;
 
     behaviour_event(mon, ME_ALERT, &you);
@@ -1175,8 +1109,7 @@ spret summon_shadow_creatures()
             // summon IDs
             for (monster_iterator mi; mi; ++mi)
             {
-                if (testbits(mi->flags, MF_BAND_MEMBER)
-                    && (mid_t) mi->props[BAND_LEADER_KEY].get_int() == mons->mid)
+                if (mi->is_band_follower_of(*mons))
                 {
                     if (god_hates_monster(**mi))
                         monster_die(**mi, KILL_RESET, NON_MONSTER);
@@ -1541,7 +1474,7 @@ spret cast_martyrs_knell(const actor* caster, int pow, god_type god, bool fail)
                  caster->name(DESC_THE).c_str());
         }
 
-        martyr_injury_bond(*shade);
+        mons_update_aura(*shade);
     }
     else if (caster->is_player())
         canned_msg(MSG_NOTHING_HAPPENS);
@@ -1556,6 +1489,7 @@ static spell_type servitor_spells[] =
     SPELL_IOOD,
     SPELL_BOMBARD,
     SPELL_PLASMA_BEAM, // maybe should be higher?
+    SPELL_PERMAFROST_ERUPTION,
     SPELL_FIREBALL,
     SPELL_ARCJOLT,
     SPELL_STONE_ARROW,
@@ -1576,9 +1510,20 @@ static spell_type servitor_spells[] =
  */
 spell_type player_servitor_spell()
 {
+    if (you.props.exists(SERVITOR_SPELL_KEY))
+    {
+        spell_type spell = (spell_type)you.props[SERVITOR_SPELL_KEY].get_int();
+        // Double-check that we know the spell and can cast it well enough, since
+        // both things may have changed since it was saved.
+        if (you.has_spell(spell) && failure_rate_to_int(raw_spell_fail(spell)) <= 20)
+            return spell;
+    }
+
+    // Fallback using default list if none was specified.
     for (const spell_type spell : servitor_spells)
         if (you.has_spell(spell) && failure_rate_to_int(raw_spell_fail(spell)) <= 20)
             return spell;
+
     return SPELL_NO_SPELL;
 }
 
@@ -1607,17 +1552,21 @@ static void _init_servitor_monster(monster &mon, const actor& caster, int pow)
                                             // mhp doesn't vary with HD
     int spell_levels = 0;
 
-    for (const spell_type spell : servitor_spells)
+    if (caster.is_player())
     {
-        if (caster.has_spell(spell)
-            && (caster_mon || failure_rate_to_int(raw_spell_fail(spell)) <= 20))
+        const spell_type spell = player_servitor_spell();
+        mon.spells.emplace_back(spell, 0, MON_SPELL_WIZARD);
+        spell_levels += spell_difficulty(spell);
+    }
+    else
+    {
+        for (const spell_type spell : servitor_spells)
         {
-            mon.spells.emplace_back(spell, 0, MON_SPELL_WIZARD);
-            spell_levels += spell_difficulty(spell);
-
-            // Player servitors take a single spell
-            if (!caster_mon)
-                break;
+            if (caster.has_spell(spell))
+            {
+                mon.spells.emplace_back(spell, 0, MON_SPELL_WIZARD);
+                spell_levels += spell_difficulty(spell);
+            }
         }
     }
 
@@ -1679,6 +1628,18 @@ spret cast_spellforged_servitor(int pow, god_type god, bool fail)
     return spret::success;
 }
 
+void remove_player_servitor()
+{
+    for (monster_iterator mi; mi; ++mi)
+    {
+        if (mi->type == MONS_SPELLFORGED_SERVITOR && mi->summoner == MID_PLAYER)
+        {
+            monster_die(**mi, KILL_RESET, NON_MONSTER);
+            return;
+        }
+    }
+}
+
 monster* find_battlesphere(const actor* agent)
 {
     if (agent->props.exists(BATTLESPHERE_KEY))
@@ -1772,7 +1733,9 @@ spret cast_battlesphere(actor* agent, int pow, god_type god, bool fail)
                 }
                 else if (you.can_see(*battlesphere))
                     simple_monster_message(*battlesphere, " appears!");
-                battlesphere->props[BAND_LEADER_KEY].get_int() = agent->mid;
+
+                if (agent->is_monster())
+                    battlesphere->set_band_leader(*(agent->as_monster()));
             }
             battlesphere->battlecharge = 4 + random2(pow + 10) / 10;
             battlesphere->foe = agent->mindex();
@@ -2079,8 +2042,8 @@ int prism_hd(int pow, bool random)
     return pow / 10;
 }
 
-spret cast_fulminating_prism(actor* caster, int pow,
-                                  const coord_def& where, bool fail)
+spret cast_fulminating_prism(actor* caster, int pow, const coord_def& where,
+                             bool fail, bool is_shadow)
 {
     if (grid_distance(where, caster->pos())
         > spell_range(SPELL_FULMINANT_PRISM, pow))
@@ -2128,12 +2091,15 @@ spret cast_fulminating_prism(actor* caster, int pow,
 
     const int hd = prism_hd(pow);
 
-    mgen_data prism_data = mgen_data(MONS_FULMINANT_PRISM,
+    mgen_data prism_data = mgen_data(is_shadow
+                                        ? MONS_SHADOW_PRISM
+                                        : MONS_FULMINANT_PRISM,
                                      caster->is_player()
-                                     ? BEH_FRIENDLY
-                                     : SAME_ATTITUDE(caster->as_monster()),
+                                        ? BEH_FRIENDLY
+                                        : SAME_ATTITUDE(caster->as_monster()),
                                      where, MHITNOT, MG_FORCE_PLACE);
-    prism_data.set_summoned(caster, 0, SPELL_FULMINANT_PRISM);
+    prism_data.set_summoned(caster, 0, is_shadow ? SPELL_SHADOW_PRISM
+                                                 : SPELL_FULMINANT_PRISM);
     prism_data.hd = hd;
     monster *prism = create_monster(prism_data);
 
@@ -2141,12 +2107,25 @@ spret cast_fulminating_prism(actor* caster, int pow,
     {
         if (caster->observable())
         {
-            mprf("%s %s a prism of explosive energy!",
+            mprf("%s %s a prism of %s energy!",
                  caster->name(DESC_THE).c_str(),
-                 caster->conj_verb("conjure").c_str());
+                 caster->conj_verb("conjure").c_str(),
+                 is_shadow ? "shadowy" : "explosive");
         }
         else if (you.can_see(*prism))
-            mprf("A prism of explosive energy appears from nowhere!");
+        {
+            mprf("A prism of %s energy appears from nowhere!",
+                 is_shadow ? "shadowy" : "explosive");
+        }
+
+        // This looks silly, but prevents the even sillier-looking situation of
+        // monster-cast prisms displaying as 'unaware of you'.
+        if (caster->is_monster())
+        {
+            prism->foe = caster->as_monster()->foe;
+            prism->behaviour = BEH_SEEK;
+            prism->flags |= MF_WAS_IN_VIEW;
+        }
     }
     else if (you.can_see(*caster))
         canned_msg(MSG_NOTHING_HAPPENS);
@@ -2245,11 +2224,12 @@ static const map<spell_type, summon_cap> summonsdata =
     { SPELL_MARTYRS_KNELL,            { 1, 1 } },
     { SPELL_HAUNT,                    { 8, 8 } },
     { SPELL_SUMMON_CACTUS,            { 1, 1 } },
+    { SPELL_SOUL_SPLINTER,            { 1, 1 } },
     // Monster-only spells
     { SPELL_SHADOW_CREATURES,         { 0, 4 } },
     { SPELL_SUMMON_SPIDERS,           { 0, 5 } },
     { SPELL_SUMMON_UFETUBUS,          { 0, 8 } },
-    { SPELL_SUMMON_HELL_BEAST,        { 0, 8 } },
+    { SPELL_SUMMON_SIN_BEAST,         { 0, 5 } },
     { SPELL_SUMMON_UNDEAD,            { 0, 8 } },
     { SPELL_SUMMON_DRAKES,            { 0, 4 } },
     { SPELL_SUMMON_MUSHROOMS,         { 0, 8 } },
@@ -2258,7 +2238,8 @@ static const map<spell_type, summon_cap> summonsdata =
     { SPELL_FIRE_ELEMENTALS,          { 0, 3 } },
     { SPELL_EARTH_ELEMENTALS,         { 0, 3 } },
     { SPELL_AIR_ELEMENTALS,           { 0, 3 } },
-    { SPELL_SUMMON_SPECTRAL_ORCS,     { 0, 3 } },
+    { SPELL_STICKS_TO_SNAKES,         { 0, 2 } },
+    { SPELL_VANQUISHED_VANGUARD,      { 0, 3 } },
     { SPELL_FIRE_SUMMON,              { 0, 4 } },
     { SPELL_SUMMON_MINOR_DEMON,       { 0, 3 } },
     { SPELL_CALL_LOST_SOULS,          { 0, 3 } },
@@ -2279,6 +2260,8 @@ static const map<spell_type, summon_cap> summonsdata =
     { SPELL_SUMMON_HELL_SENTINEL,     { 0, 3 } },
     { SPELL_CONJURE_LIVING_SPELLS,    { 0, 4 } },
     { SPELL_SHEZAS_DANCE,             { 0, 6 } },
+    { SPELL_DIVINE_ARMAMENT,          { 0, 1 } },
+    { SPELL_FLASHING_BALESTRA,        { 0, 2 } },
 };
 
 bool summons_are_capped(spell_type spell)
@@ -2488,7 +2471,7 @@ static void _overgrow_wall(const coord_def &pos)
     destroy_wall(pos);
 
     const monster_type mon = random_choose_weighted(4, MONS_OKLOB_SAPLING,
-                                                    4, MONS_BURNING_BUSH,
+                                                    4, MONS_SCRUB_NETTLE,
                                                     4, MONS_WANDERING_MUSHROOM,
                                                     1, MONS_BALLISTOMYCETE,
                                                     1, MONS_OKLOB_PLANT);
@@ -2753,7 +2736,7 @@ spret foxfire_swarm()
     bool unknown_unseen = false;
     for (radius_iterator ri(you.pos(), 2, C_SQUARE, LOS_NO_TRANS); ri; ++ri)
     {
-        if (_create_foxfire(you, *ri, GOD_NO_GOD, 10))
+        if (_create_foxfire(you, *ri, GOD_NO_GOD, 20))
         {
             created = true;
             continue;
@@ -2776,6 +2759,65 @@ spret foxfire_swarm()
     return spret::fail; // don't spend piety, do spend a turn
 }
 
+bool summon_hell_out_of_bat(const actor &agent, coord_def pos)
+{
+    // Since this isn't really used as for a spell: count how many creatures
+    // this has already summoned, and abort at our max defined here (four).
+    int count = 0;
+    for (monster_iterator mi; mi; ++mi)
+    {
+       if (mi->summoner == agent.mid)
+           ++count;
+
+        if (count > 3)
+           return false;
+    }
+
+    monster_type mon = coinflip() ? MONS_HELL_HOUND : MONS_HELL_RAT;
+
+    monster *mons = create_monster(
+            mgen_data(mon, BEH_COPY, pos, _auto_autofoe(&agent), MG_AUTOFOE)
+                      .set_summoned(&agent, 1, SPELL_NO_SPELL, GOD_NO_GOD));
+    if (mons)
+        return true;
+
+    return false;
+}
+
+bool summon_swarm_clone(const monster& agent, coord_def target_pos)
+{
+    // Go up the summon chain to find the highest-level version of ourselves
+    const monster* parent = &agent;
+    while (parent->summoner && monster_by_mid(parent->summoner)
+           && monster_by_mid(parent->summoner)->type == agent.type)
+    {
+        parent = monster_by_mid(parent->summoner);
+    }
+
+    // Apply pseudo-summon cap
+    int count = 0;
+    for (monster_iterator mi; mi; ++mi)
+    {
+       if (mi->summoner == parent->mid)
+           ++count;
+
+        if (count > 8)
+           return false;
+    }
+
+    mgen_data mg(agent.type, BEH_COPY, target_pos, _auto_autofoe(parent), MG_AUTOFOE);
+    mg.set_summoned(parent, 2, SPELL_NO_SPELL, GOD_NO_GOD);
+
+    if (monster* spawn = create_monster(mg))
+    {
+        if (you.can_see(*spawn))
+            mprf("Another %s is drawn to the feast!", spawn->name(DESC_PLAIN).c_str());
+        return true;
+    }
+
+    return false;
+}
+
 bool summon_spider(const actor &agent, coord_def pos, god_type god,
                         spell_type spell, int pow)
 {
@@ -2824,6 +2866,11 @@ spret summon_spiders(actor &agent, int pow, god_type god, bool fail)
     return spret::success;
 }
 
+int barrelling_boulder_hp(int pow)
+{
+    return 15 + pow / 3;
+}
+
 spret cast_broms_barrelling_boulder(actor& agent, coord_def targ, int pow, bool fail)
 {
     fail_check();
@@ -2849,6 +2896,7 @@ spret cast_broms_barrelling_boulder(actor& agent, coord_def targ, int pow, bool
                                 : SAME_ATTITUDE(agent.as_monster()),
                              pos, MHITNOT, MG_FORCE_PLACE);
     mg.set_summoned(&agent, 0, SPELL_BOULDER);
+    mg.hp = barrelling_boulder_hp(pow);
     monster *boulder = create_monster(mg);
 
     // If some other reason prevents this from working (I'm not sure what?)
@@ -2864,6 +2912,10 @@ spret cast_broms_barrelling_boulder(actor& agent, coord_def targ, int pow, bool
     boulder->props[BOULDER_DIRECTION_KEY] = pos - agent.pos();
 
     mpr("You send a boulder barrelling forward!");
+
+    // Let the boulder roll one space immediately.
+    boulder->speed_increment = 80;
+
     return spret::success;
 }
 
@@ -2875,6 +2927,9 @@ string mons_simulacrum_immune_reason(const monster *mons)
     if (!mons_can_be_spectralised(*mons))
         return "You can't make a simulacrum of that!";
 
+    if (mons->friendly() || mons->neutral())
+        return "That would be terribly rude!";
+
     return "";
 }
 
@@ -2892,7 +2947,7 @@ spret cast_simulacrum(coord_def target, int pow, bool fail)
         fail_check();
         canned_msg(MSG_NOTHING_CLOSE_ENOUGH);
         // If there's no monster there, you still pay the costs in
-        // order to prevent locating invisible/submerged monsters.
+        // order to prevent locating invisible monsters.
         return spret::success;
     }
 
@@ -2934,3 +2989,289 @@ spret cast_simulacrum(coord_def target, int pow, bool fail)
 
     return spret::success;
 }
+
+static int _hoarfrost_cannon_hd(int pow, bool random = true)
+{
+    if (random)
+        return 4 + div_rand_round(pow, 20);
+    return 4 + pow / 20;
+}
+
+dice_def hoarfrost_cannonade_damage(int pow, bool finale)
+{
+    if (finale)
+        return zap_damage(ZAP_HOARFROST_BULLET_FINALE, _hoarfrost_cannon_hd(pow, false) * 12, true, false);
+    else
+        return zap_damage(ZAP_HOARFROST_BULLET, _hoarfrost_cannon_hd(pow, false) * 12, true, false);
+}
+
+spret cast_hoarfrost_cannonade(const actor& agent, int pow, bool fail)
+{
+    fail_check();
+
+    // Remove any existing cannons we may have first
+    for (monster_iterator mi; mi; ++mi)
+    {
+        if (mi->type == MONS_HOARFROST_CANNON && mi->summoner == agent.mid)
+            monster_die(**mi, KILL_MISC, NON_MONSTER);
+    }
+
+    mgen_data cannon = _summon_data(agent, MONS_HOARFROST_CANNON, 0, GOD_NO_GOD,
+                                    SPELL_HOARFROST_CANNONADE);
+    cannon.flags |= MG_FORCE_PLACE;
+    cannon.hd = _hoarfrost_cannon_hd(pow);
+
+    // Make both cannons share the same duration
+    const int dur = random_range(16, 22) * BASELINE_DELAY;
+
+    int num_seen = 0;
+
+    // Monsters summon cannons around their foe instead of themselves (both to
+    // let them operate with other monsters in the way and because it's too easy
+    // for the player to back out of range otherwise)
+    const coord_def center = (agent.is_player() ? you.pos()
+                                                : agent.as_monster()->get_foe()->pos());
+    for (int i = 0; i < 2; ++i)
+    {
+        // Find a spot for each cannon (at a somewhat larger distance than
+        // normal summons)
+        find_habitable_spot_near(center, MONS_HOARFROST_CANNON, 3, false,
+                                 cannon.pos);
+
+        monster* mons = create_monster(cannon);
+        if (mons)
+        {
+            // Give a bit of instant energy so the slow cannons don't take
+            // multiple turns to fire their first shot.
+            mons->speed_increment = 70;
+            mons->add_ench(mon_enchant(ENCH_FAKE_ABJURATION, 0, &agent, dur));
+            mons->mark_summoned(0, false, SPELL_HOARFROST_CANNONADE, false);
+
+            if (you.can_see(*mons))
+                ++num_seen;
+        }
+    }
+
+    if (num_seen > 1)
+    {
+        mprf("%s sculpt%s a pair of cannons out of ice!",
+             agent.name(DESC_THE).c_str(),
+             agent.is_player() ? ""  : "s");
+    }
+    else if (num_seen == 1)
+    {
+        mprf("%s sculpt%s a cannon out of ice!",
+             agent.name(DESC_THE).c_str(),
+             agent.is_player() ? ""  : "s");
+    }
+    else
+        canned_msg(MSG_NOTHING_HAPPENS);
+
+    return spret::success;
+}
+
+static int _hellfire_mortar_hd(int pow, bool random = true)
+{
+    if (random)
+        return 3 + div_rand_round(pow, 28);
+    return 3 + pow / 28;
+}
+
+dice_def hellfire_mortar_damage(int pow)
+{
+    return zap_damage(ZAP_BOLT_OF_MAGMA, _hellfire_mortar_hd(pow, false) * 12, true, false);
+}
+
+static bool _hellfire_stops_here(bolt& beam, coord_def pos)
+{
+    return actor_at(pos) && !beam.ignores_monster(monster_at(pos))
+           || cell_is_solid(pos) && !beam.can_affect_wall(pos);
+}
+
+spret cast_hellfire_mortar(const actor& agent, bolt& beam, int pow, bool fail)
+{
+    // Determine path by firing digging tracer
+    zappy(ZAP_HELLFIRE_MORTAR_DIG, pow, agent.is_monster(), beam);
+    beam.source = agent.pos();
+    beam.source_id = agent.mid;
+    beam.origin_spell = SPELL_HELLFIRE_MORTAR;
+    beam.is_tracer = true;
+    beam.fire();
+
+    // Check whether the path ends because it reached max range, or because it
+    // hit something invalid.
+    const int len = _hellfire_stops_here(beam, beam.path_taken.back())
+                    ? beam.path_taken.size() - 1
+                    : beam.path_taken.size();
+
+    // Abort if the player knows this path is blocked. (Invisible monsters at
+    // close range need to be handled later, alas)
+    if (agent.is_player())
+    {
+        monster* mon = monster_at(beam.path_taken[0]);
+        if (mon && you.can_see(*mon))
+        {
+            mprf("%s is in the way!", mon->name(DESC_THE).c_str());
+            return spret::abort;
+        }
+        else if (len == 0 && !mon)
+        {
+            mpr("There's no room!");
+            return spret::abort;
+        }
+    }
+
+    fail_check();
+
+    // Likely because of an invisible monster standing in our first cannon spot
+    if (actor_at(beam.path_taken[0]))
+    {
+        if (agent.is_player())
+            mpr("Something prevents your mortar from forming!");
+        return spret::success;
+    }
+
+    // Make the lava
+    int dur = random_range(15, 19) * BASELINE_DELAY;
+    for (unsigned int i = 0; i < beam.path_taken.size(); ++i)
+    {
+        const coord_def pos = beam.path_taken[i];
+
+        if (!in_bounds(pos))
+            break;
+
+        // Don't make lava under things that can't survive there.
+        if (monster_at(pos) && !beam.ignores_monster(monster_at(pos))
+            || pos == you.pos() && !you.airborne())
+        {
+            break;
+        }
+
+        if (feat_is_solid(env.grid(pos)) && !feat_is_diggable(env.grid(pos))
+            && !feat_is_tree(env.grid(pos)))
+        {
+            break;
+        }
+
+        temp_change_terrain(beam.path_taken[i], DNGN_LAVA,
+                            dur - (i * BASELINE_DELAY),
+                            TERRAIN_CHANGE_HELLFIRE_MORTAR);
+
+        flash_tile(pos, RED, 5);
+    }
+
+    noisy(spell_effect_noise(SPELL_HELLFIRE_MORTAR), agent.pos(), agent.mid);
+
+    mgen_data mg = _summon_data(agent, MONS_HELLFIRE_MORTAR, 0,
+                                GOD_NO_GOD, SPELL_HELLFIRE_MORTAR);
+    mg.flags |= MG_FORCE_PLACE;
+    mg.pos = beam.path_taken[0];
+    mg.hd = _hellfire_mortar_hd(pow);
+    monster* cannon = create_monster(mg);
+
+    // Unclear why this could happen (we've already checked that the spot is
+    // empty), but let's guard against it anyway.
+    if (!cannon)
+    {
+        if (agent.is_player())
+            mpr("Something prevents your mortar from forming!");
+        return spret::success;
+    }
+
+    // Store the cannon's movement path
+    CrawlVector& path = cannon->props[HELLFIRE_PATH_KEY].get_vector();
+    for (unsigned int i = 0; i < beam.path_taken.size(); ++i)
+    {
+        const coord_def pos = beam.path_taken[i];
+        path.push_back(pos);
+    }
+
+    cannon->add_ench(mon_enchant(ENCH_SHORT_LIVED, 1, &agent, dur));
+
+    mprf("With a deafening crack, the ground splits apart in the path of %s "
+        "chthonic artillery!", agent.name(DESC_ITS).c_str());
+
+    return spret::success;
+}
+
+bool hellfire_mortar_active(const actor& agent)
+{
+    // XXX: Really hate to put marionette-specific code in individual spells,
+    //      but this one doesn't have an obvious way to avoid it yet.
+    //
+    //      I'd really like to generalize 'count_summons' to count all things
+    //      with a given summoner, rather than just abjurable things with a
+    //      given summoner.
+    const mid_t agent_mid = (agent.real_attitude() == ATT_MARIONETTE
+                                ? MID_PLAYER : agent.mid);
+
+    for (monster_iterator mi; mi; ++mi)
+    {
+        if (mi->type == MONS_HELLFIRE_MORTAR && mi->summoner == agent_mid)
+            return true;
+    }
+
+    return false;
+}
+
+bool make_soul_wisp(const actor& agent, actor& victim)
+{
+    if (victim.is_monster() && !mons_can_be_spectralised(*victim.as_monster()))
+        return false;
+
+    // Don't try to create a wisp from a monster who's already had one made from
+    // them. (This causes weird messaging and removes the Weak effect).
+    if (victim.is_monster() && victim.props.exists(SOUL_SPLINTERED_KEY))
+    {
+        if (agent.is_player() && victim.observable())
+        {
+            mprf("A fragment of %s is already outside their body!",
+                    victim.name(DESC_THE).c_str());
+        }
+        return false;
+    }
+
+    vector<coord_def> spots;
+    for (adjacent_iterator ai(victim.pos()); ai; ++ai)
+    {
+        if (monster_habitable_grid(MONS_SOUL_WISP, env.grid(*ai))
+            && !actor_at(*ai) && agent.see_cell_no_trans(*ai))
+        {
+            spots.push_back(*ai);
+        }
+    }
+    if (spots.size() <= 0)
+    {
+        if (agent.is_player())
+            mpr("There's no room for the soul wisp to form!");
+        return false;
+    }
+
+    if (you.see_cell(victim.pos()))
+    {
+        mprf("A fragment of %s soul is dislodged from %s body.",
+                victim.name(DESC_ITS).c_str(),
+                victim.pronoun(PRONOUN_POSSESSIVE).c_str());
+
+    }
+
+    mgen_data mg = _summon_data(agent, MONS_SOUL_WISP, 2, GOD_NO_GOD, SPELL_SOUL_SPLINTER);
+    mg.pos = spots[random2(spots.size())];
+    mg.flags |= MG_FORCE_PLACE;
+
+    // Damage improves when extracted from stronger enemies, but they are always fragile.
+    mg.hd = 2 + div_rand_round(victim.get_experience_level(), 2);
+    mg.hp = random_range(5, 8);
+
+    monster* wisp = create_monster(mg);
+
+    wisp->add_ench(mon_enchant(ENCH_HAUNTING, 1, &victim, INFINITE_DURATION));
+    victim.weaken(&agent, wisp->get_ench(ENCH_ABJ).duration / 10);
+    victim.props[SOUL_SPLINTERED_KEY]= true;
+
+    // Let wisp act immediately (so that if it appears behind the enemy, the
+    // enemy won't simply move out of range first).
+    wisp->speed_increment = 80;
+
+    return true;
+}
