@@ -25,6 +25,7 @@
 #include "item-prop.h"
 #include "level-state-type.h"
 #include "libutil.h"
+#include "macro.h"
 #include "message.h"
 #include "notes.h"
 #include "options.h"
@@ -347,6 +348,19 @@ bool add_spell_to_memory(spell_type spell)
     if (you.num_turns)
         mprf("Spell assigned to '%c'.", index_to_letter(letter_j));
 
+    // A hint, for those who may not be aware.
+    if (spell == SPELL_SPELLFORGED_SERVITOR)
+    {
+        mprf(MSGCH_TUTORIAL,
+             "(You may use Imbue Servitor from the <w>%s</w>bility menu to change "
+             "which spell your servitor casts)",
+                command_to_string(CMD_USE_ABILITY).c_str());
+    }
+    // Give a free charge upon learning this spell for the first time, so the
+    // player can actually use it immediately.
+    else if (spell == SPELL_GRAVE_CLAW)
+        gain_grave_claw_soul(true);
+
     // Swapping with an existing spell.
     if (you.spell_letter_table[letter_j] != -1)
     {
@@ -432,7 +446,7 @@ bool spell_harms_target(spell_type spell)
     if (flags & spflag::targeting_mask)
         return true;
 
-    // n.b. this excludes various untargeted attack spells like hailstorm, abs 0
+    // n.b. this excludes various untargeted attack spells like hailstorm, MCC
 
     return false;
 }
@@ -462,7 +476,6 @@ bool spell_is_direct_attack(spell_type spell)
         // spell school exceptions
         if (spell == SPELL_VIOLENT_UNRAVELLING  // hex
             || spell == SPELL_FORCE_LANCE // transloc
-            || spell == SPELL_GRAVITAS
             || spell == SPELL_BLINKBOLT
             || spell == SPELL_BANISHMENT)
         {
@@ -501,6 +514,7 @@ bool spell_is_direct_attack(spell_type spell)
         || spell == SPELL_IGNITION
         || spell == SPELL_STARBURST
         || spell == SPELL_HAILSTORM
+        || spell == SPELL_PERMAFROST_ERUPTION
         || spell == SPELL_MANIFOLD_ASSAULT
         || spell == SPELL_MAXWELLS_COUPLING) // n.b. not an area spell
     {
@@ -516,10 +530,17 @@ bool spell_is_direct_attack(spell_type spell)
 int spell_mana(spell_type which_spell, bool real_spell)
 {
     const int level = _seekspell(which_spell)->level;
-    if (real_spell && (you.duration[DUR_BRILLIANCE]
-                       || player_equip_unrand(UNRAND_FOLLY)))
+
+    if (real_spell)
     {
-        return level/2 + level%2; // round up
+        int cost = level;
+        if (you.wearing_ego(EQ_GIZMO, SPGIZMO_SPELLMOTOR))
+            cost = max(1, cost - you.rev_tier());
+
+        if (you.duration[DUR_BRILLIANCE] || player_equip_unrand(UNRAND_FOLLY))
+            cost = cost/2 + cost%2; // round up
+
+        return cost;
     }
     return level;
 }
@@ -1014,7 +1035,6 @@ int spell_range(spell_type spell, int pow,
         && vehumet_supports_spell(spell)
         && have_passive(passive_t::spells_range)
         && maxrange > 1
-        && spell != SPELL_HAILSTORM // uses a special system
         && spell != SPELL_THUNDERBOLT) // lightning rod only
     {
         maxrange++;
@@ -1170,6 +1190,8 @@ string casting_uselessness_reason(spell_type spell, bool temp)
     case SPELL_SIMULACRUM:
     case SPELL_INFESTATION:
     case SPELL_TUKIMAS_DANCE:
+    case SPELL_HOARFROST_CANNONADE:
+    case SPELL_SOUL_SPLINTER:
         if (you.allies_forbidden())
             return "you cannot coerce anything to obey you.";
         break;
@@ -1302,7 +1324,7 @@ string spell_uselessness_reason(spell_type spell, bool temp, bool prevent,
         break;
 
     case SPELL_SUBLIMATION_OF_BLOOD:
-        if (!you.can_bleed(temp))
+        if (!you.has_blood(temp))
             return "you have no blood to sublime.";
         break;
 
@@ -1356,18 +1378,12 @@ string spell_uselessness_reason(spell_type spell, bool temp, bool prevent,
 
     case SPELL_DEATH_CHANNEL:
         if (temp && you.duration[DUR_DEATH_CHANNEL])
-            return "you are already channeling the dead.";
+            return "you are already channelling the dead.";
         if (have_passive(passive_t::reaping))
             return "you are already reaping souls!";
         break;
 
-    case SPELL_ROT:
-        {
-            const mon_holy_type holiness = you.holiness(temp, false);
-            if (holiness != MH_NATURAL && holiness != MH_UNDEAD)
-                return "you have no flesh to rot.";
-        }
-        // fallthrough to cloud spells
+    case SPELL_PUTREFACTION:
     case SPELL_BLASTMOTE:
     case SPELL_POISONOUS_CLOUD:
     case SPELL_FREEZING_CLOUD:
@@ -1412,7 +1428,7 @@ string spell_uselessness_reason(spell_type spell, bool temp, bool prevent,
     case SPELL_MANIFOLD_ASSAULT:
         if (temp)
         {
-            const string unproj_reason = weapon_unprojectability_reason();
+            const string unproj_reason = weapon_unprojectability_reason(you.weapon());
             if (unproj_reason != "")
                 return unproj_reason;
         }
@@ -1423,6 +1439,20 @@ string spell_uselessness_reason(spell_type spell, bool temp, bool prevent,
             return "you cannot redirect your momentum while unable to move.";
         break;
 
+    case SPELL_PILEDRIVER:
+        if (you.stasis())
+            return "your stasis prevents you from slingshotting yourself.";
+        if (temp)
+        {
+            if (!you.is_motile())
+                return "you cannot slingshot yourself while unable to move.";
+            if (you.no_tele(true))
+                return lowercase_first(you.no_tele_reason(true));
+            if (possible_piledriver_targets(false).empty())
+                return "you cannot see anything nearby that you can slingshot.";
+        }
+        break;
+
     case SPELL_ELECTRIC_CHARGE:
         // XXX: this is a little redundant with you_no_tele_reason()
         // but trying to sort out temp and so on is a mess
@@ -1430,9 +1460,6 @@ string spell_uselessness_reason(spell_type spell, bool temp, bool prevent,
             return "your stasis prevents you from teleporting.";
         if (temp)
         {
-            const string no_move_reason = movement_impossible_reason();
-            if (!no_move_reason.empty())
-                return no_move_reason;
             if (you.no_tele(true))
                 return lowercase_first(you.no_tele_reason(true));
             const string no_charge_reason = electric_charge_impossible_reason(true);
@@ -1449,6 +1476,32 @@ string spell_uselessness_reason(spell_type spell, bool temp, bool prevent,
     case SPELL_CALL_CANINE_FAMILIAR:
         if (temp && you.duration[DUR_CANINE_FAMILIAR_DEAD])
             return "your canine familiar is too injured to answer your call.";
+        break;
+
+    case SPELL_GELLS_GAVOTTE:
+        if (temp && you.duration[DUR_GAVOTTE_COOLDOWN])
+            return "local gravity is still too unstable to reorient.";
+        break;
+
+    case SPELL_FULSOME_FUSILLADE:
+        if (temp && you.duration[DUR_FUSILLADE])
+            return "you are already unleashing a barrage of alchemical concoctions!";
+        break;
+
+    case SPELL_HELLFIRE_MORTAR:
+        if (temp && hellfire_mortar_active(you))
+            return "you already have an active mortar!";
+        break;
+
+    case SPELL_STARBURST:
+        if (temp && you.current_vision == 0)
+            return "you cannot see far enough to hit anything with this spell.";
+        break;
+
+    case SPELL_GRAVE_CLAW:
+        if (temp && you.props[GRAVE_CLAW_CHARGES_KEY].get_int() == 0)
+            return "you must harvest more living souls to recharge this spell.";
+        break;
 
     default:
         break;
@@ -1510,6 +1563,7 @@ bool spell_no_hostile_in_range(spell_type spell)
     case SPELL_SUMMON_LIGHTNING_SPIRE:
     case SPELL_NOXIOUS_BOG:
     case SPELL_BOULDER:
+    case SPELL_GELLS_GAVOTTE:
     // This can always potentially hit out-of-LOS, although this is conditional
     // on spell-power.
     case SPELL_FIRE_STORM:
@@ -1518,6 +1572,8 @@ bool spell_no_hostile_in_range(spell_type spell)
     case SPELL_OLGREBS_TOXIC_RADIANCE:
     case SPELL_IGNITION:
     case SPELL_FROZEN_RAMPARTS:
+    case SPELL_FULSOME_FUSILLADE:
+    case SPELL_HELLFIRE_MORTAR:
         return minRange > you.current_vision;
 
     // Special handling for cloud spells.
@@ -1557,7 +1613,7 @@ bool spell_no_hostile_in_range(spell_type spell)
         return cast_hailstorm(-1, false, true) == spret::abort;
 
     case SPELL_DAZZLING_FLASH:
-        return cast_dazzling_flash(pow, false, true) == spret::abort;
+        return cast_dazzling_flash(&you, pow, false, true) == spret::abort;
 
      case SPELL_MAXWELLS_COUPLING:
          return cast_maxwells_coupling(pow, false, true) == spret::abort;
@@ -1566,7 +1622,7 @@ bool spell_no_hostile_in_range(spell_type spell)
          return cast_intoxicate(-1, false, true) == spret::abort;
 
     case SPELL_MANIFOLD_ASSAULT:
-         return cast_manifold_assault(-1, false, false) == spret::abort;
+         return cast_manifold_assault(you, -1, false, false) == spret::abort;
 
     case SPELL_OZOCUBUS_REFRIGERATION:
          return trace_los_attack_spell(SPELL_OZOCUBUS_REFRIGERATION, pow, &you)
@@ -1593,10 +1649,10 @@ bool spell_no_hostile_in_range(spell_type spell)
     // Check slightly beyond our target range, in case someone wants to catch
     // something in the AoE at the edge of range.
     case SPELL_MERCURY_VAPOURS:
-        return find_near_hostiles(range + 1, false).empty();
+        return find_near_hostiles(range + 1, false, you).empty();
 
     case SPELL_SCORCH:
-        return find_near_hostiles(range, false).empty();
+        return find_near_hostiles(range, false, you).empty();
 
     case SPELL_ANGUISH:
         for (monster_near_iterator mi(you.pos(), LOS_NO_TRANS); mi; ++mi)
@@ -1614,6 +1670,9 @@ bool spell_no_hostile_in_range(spell_type spell)
         }
         return true; // TODO
 
+    case SPELL_PERMAFROST_ERUPTION:
+        return permafrost_targets(you, pow, false).empty();
+
     default:
         break;
     }
@@ -1794,20 +1853,20 @@ const vector<spell_type> *soh_breath_spells(spell_type spell)
     static const map<spell_type, vector<spell_type>> soh_breaths = {
         { SPELL_SERPENT_OF_HELL_GEH_BREATH,
             { SPELL_FIRE_BREATH,
-              SPELL_FLAMING_CLOUD,
+              SPELL_BOLT_OF_MAGMA,
               SPELL_FIREBALL } },
         { SPELL_SERPENT_OF_HELL_COC_BREATH,
             { SPELL_COLD_BREATH,
               SPELL_FREEZING_CLOUD,
               SPELL_FLASH_FREEZE } },
         { SPELL_SERPENT_OF_HELL_DIS_BREATH,
-            { SPELL_METAL_SPLINTERS,
+            { SPELL_IRON_SHOT,
               SPELL_QUICKSILVER_BOLT,
-              SPELL_LEHUDIBS_CRYSTAL_SPEAR } },
-        { SPELL_SERPENT_OF_HELL_TAR_BREATH,
-            { SPELL_BOLT_OF_DRAINING,
-              SPELL_MIASMA_BREATH,
               SPELL_CORROSIVE_BOLT } },
+        { SPELL_SERPENT_OF_HELL_TAR_BREATH,
+            { SPELL_GHOSTLY_FIREBALL,
+              SPELL_MIASMA_BREATH,
+              SPELL_POISON_ARROW } },
     };
 
     return map_find(soh_breaths, spell);
@@ -1823,38 +1882,51 @@ const set<spell_type> removed_spells =
 {
 #if TAG_MAJOR_VERSION == 34
     SPELL_AURA_OF_ABJURATION,
+    SPELL_AWAKEN_EARTH,
+    SPELL_BEASTLY_APPENDAGE,
+    SPELL_BLADE_HANDS,
     SPELL_BOLT_OF_INACCURACY,
     SPELL_CHANT_FIRE_STORM,
     SPELL_CIGOTUVIS_DEGENERATION,
     SPELL_CIGOTUVIS_EMBRACE,
+    SPELL_CLOUD_CONE,
     SPELL_CONDENSATION_SHIELD,
+    SPELL_CONJURE_FLAME,
     SPELL_CONTROLLED_BLINK,
     SPELL_CONTROL_TELEPORT,
     SPELL_CONTROL_UNDEAD,
     SPELL_CONTROL_WINDS,
+    SPELL_CORPSE_ROT,
     SPELL_CORRUPT_BODY,
     SPELL_CURE_POISON,
+    SPELL_DARKNESS,
     SPELL_DEFLECT_MISSILES,
     SPELL_DELAYED_FIREBALL,
     SPELL_DEMONIC_HORDE,
     SPELL_DRACONIAN_BREATH,
+    SPELL_DRAGON_FORM,
     SPELL_EPHEMERAL_INFUSION,
     SPELL_EVAPORATE,
+    SPELL_EXCRUCIATING_WOUNDS,
     SPELL_EXPLOSIVE_BOLT,
     SPELL_FAKE_RAKSHASA_SUMMON,
     SPELL_FIRE_BRAND,
     SPELL_FIRE_CLOUD,
+    SPELL_FLAME_TONGUE,
     SPELL_FLY,
     SPELL_FORCEFUL_DISMISSAL,
     SPELL_FREEZING_AURA,
     SPELL_FRENZY,
     SPELL_FULSOME_DISTILLATION,
+    SPELL_GOAD_BEASTS,
     SPELL_GRAND_AVATAR,
     SPELL_HASTE_PLANTS,
     SPELL_HOLY_LIGHT,
     SPELL_HOLY_WORD,
     SPELL_HOMUNCULUS,
     SPELL_HUNTING_CRY,
+    SPELL_HYDRA_FORM,
+    SPELL_ICE_FORM,
     SPELL_IGNITE_POISON_SINGLE,
     SPELL_INFUSION,
     SPELL_INSULATION,
@@ -1863,15 +1935,18 @@ const set<spell_type> removed_spells =
     SPELL_MELEE,
     SPELL_MIASMA_CLOUD,
     SPELL_MISLEAD,
+    SPELL_NECROMUTATION,
     SPELL_PHASE_SHIFT,
     SPELL_POISON_CLOUD,
     SPELL_POISON_WEAPON,
     SPELL_RANDOM_BOLT,
+    SPELL_RANDOM_EFFECTS,
     SPELL_REARRANGE_PIECES,
     SPELL_RECALL,
     SPELL_REGENERATION,
     SPELL_RESURRECT,
     SPELL_RING_OF_FLAMES,
+    SPELL_RING_OF_THUNDER,
     SPELL_SACRIFICE,
     SPELL_SCATTERSHOT,
     SPELL_SEE_INVISIBLE,
@@ -1882,9 +1957,11 @@ const set<spell_type> removed_spells =
     SPELL_SINGULARITY,
     SPELL_SONG_OF_SHIELDING,
     SPELL_SPECTRAL_WEAPON,
+    SPELL_SPIDER_FORM,
+    SPELL_STATUE_FORM,
     SPELL_STEAM_CLOUD,
-    SPELL_STICKS_TO_SNAKES,
     SPELL_STONESKIN,
+    SPELL_STORM_FORM,
     SPELL_STRIKING,
     SPELL_SUMMON_BUTTERFLIES,
     SPELL_SUMMON_ELEMENTAL,
@@ -1893,33 +1970,14 @@ const set<spell_type> removed_spells =
     SPELL_SUMMON_TWISTER,
     SPELL_SUNRAY,
     SPELL_SURE_BLADE,
+    SPELL_TELEPORT_SELF,
     SPELL_THROW,
     SPELL_TOMB_OF_DOROKLOHE,
+    SPELL_TWISTED_RESURRECTION,
     SPELL_VAMPIRE_SUMMON,
+    SPELL_VORTEX,
     SPELL_WARP_BRAND,
     SPELL_WEAVE_SHADOWS,
-    SPELL_DARKNESS,
-    SPELL_CLOUD_CONE,
-    SPELL_RING_OF_THUNDER,
-    SPELL_TWISTED_RESURRECTION,
-    SPELL_RANDOM_EFFECTS,
-    SPELL_HYDRA_FORM,
-    SPELL_VORTEX,
-    SPELL_GOAD_BEASTS,
-    SPELL_TELEPORT_SELF,
-    SPELL_EXCRUCIATING_WOUNDS,
-    SPELL_CONJURE_FLAME,
-    SPELL_CORPSE_ROT,
-    SPELL_FLAME_TONGUE,
-    SPELL_BEASTLY_APPENDAGE,
-    SPELL_SPIDER_FORM,
-    SPELL_ICE_FORM,
-    SPELL_BLADE_HANDS,
-    SPELL_STATUE_FORM,
-    SPELL_STORM_FORM,
-    SPELL_DRAGON_FORM,
-    SPELL_NECROMUTATION,
-    SPELL_AWAKEN_EARTH,
 #endif
 };
 
@@ -1948,7 +2006,7 @@ bool spell_was_form(spell_type spell)
 
 void end_wait_spells(bool quiet)
 {
-    end_searing_ray();
+    end_searing_ray(you);
     end_maxwells_coupling(quiet);
     end_flame_wave();
 }
