@@ -8,6 +8,7 @@
 #include "abyss.h"
 #include "act-iter.h"
 #include "areas.h"
+#include "art-enum.h"
 #include "artefact.h"
 #include "bloodspatter.h"
 #include "branch.h"
@@ -22,6 +23,7 @@
 #include "env.h"
 #include "files.h"
 #include "god-abil.h"
+#include "god-companions.h"
 #include "god-passive.h" // passive_t::slow_abyss
 #include "hints.h"
 #include "hiscores.h"
@@ -214,7 +216,10 @@ static void _climb_message(dungeon_feature_type stair, bool going_up,
         return;
 
     if (feat_is_portal(stair))
-        mpr("The world spins around you as you enter the gateway.");
+    {
+        if (stair != DNGN_ENTER_CRUCIBLE)
+            mpr("The world spins around you as you enter the gateway.");
+    }
     else if (feat_is_escape_hatch(stair))
     {
         if (going_up)
@@ -250,11 +255,17 @@ static void _clear_golubria_traps()
     }
 }
 
-static void _clear_prisms()
+static void _clear_constructs()
 {
     for (auto &mons : menv_real)
-        if (mons.type == MONS_FULMINANT_PRISM)
+        if (mons.type == MONS_FULMINANT_PRISM
+            || mons.type == MONS_SHADOW_PRISM
+            || mons.type == MONS_HELLFIRE_MORTAR
+            || mons.type == MONS_BOULDER
+            || mons.type == MONS_BALLISTOMYCETE_SPORE)
+        {
             mons.reset();
+        }
 }
 
 static void _complete_zig()
@@ -287,9 +298,7 @@ void leaving_level_now(dungeon_feature_type stair_used)
     dungeon_events.fire_event(DET_LEAVING_LEVEL);
 
     _clear_golubria_traps();
-    _clear_prisms();
-
-    end_recall();
+    _clear_constructs();
 }
 
 static void _update_travel_cache(const level_id& old_level,
@@ -489,7 +498,7 @@ static void _hell_effects()
     if (have_passive(passive_t::resist_hell_effects)
         && x_chance_in_y(you.piety, MAX_PIETY * 2) || is_sanctuary(you.pos()))
     {
-        simple_god_message("'s power protects you from the chaos of Hell!");
+        simple_god_message(" power protects you from the chaos of Hell!", true);
         return;
     }
 
@@ -706,6 +715,14 @@ level_id level_above()
 void rise_through_ceiling()
 {
     const level_id whither = level_above();
+    if (you.where_are_you == BRANCH_DUNGEON
+        && you.depth == 1
+        && player_has_orb())
+    {
+        mpr("With a burst of heat and light, you rocket upward!");
+        floor_transition(DNGN_EXIT_DUNGEON, DNGN_EXIT_DUNGEON,
+                         level_id(BRANCH_DUNGEON, 0), true, true, false, false);
+    }
     if (!whither.is_valid())
     {
         mpr("In a burst of heat and light, you rocket briefly upward... "
@@ -715,6 +732,7 @@ void rise_through_ceiling()
 
     mpr("With a burst of heat and light, you rocket upward!");
     untag_followers(); // XXX: is this needed?
+    stop_delay(true);
     floor_transition(DNGN_ALTAR_IGNIS /*hack*/, DNGN_ALTAR_IGNIS,
                      whither, true, true, false, false);
     you.clear_far_engulf();
@@ -769,6 +787,25 @@ void floor_transition(dungeon_feature_type how,
         if (targ)
             targ->del_ench(ENCH_BULLSEYE_TARGET);
     }
+    if (yred_torch_is_raised())
+        yred_end_conquest();
+    if (you.duration[DUR_FATHOMLESS_SHACKLES])
+    {
+        you.duration[DUR_FATHOMLESS_SHACKLES] = 0;
+        yred_end_blasphemy();
+    }
+
+    if (you.duration[DUR_BEOGH_DIVINE_CHALLENGE])
+        flee_apostle_challenge();
+
+    if (you.duration[DUR_BLOOD_FOR_BLOOD])
+        beogh_end_blood_for_blood();
+
+    if (you.duration[DUR_BEOGH_CAN_RECRUIT])
+    {
+        you.duration[DUR_BEOGH_CAN_RECRUIT] = 0;
+        end_beogh_recruit_window();
+    }
 
     // Fire level-leaving trigger.
     leaving_level_now(how);
@@ -971,6 +1008,9 @@ void floor_transition(dungeon_feature_type how,
         if (branch == BRANCH_GAUNTLET)
             _gauntlet_effect();
 
+        if (branch == BRANCH_ARENA)
+            okawaru_duel_healing();
+
         const set<branch_type> boring_branch_exits = {
             BRANCH_TEMPLE,
             BRANCH_BAZAAR,
@@ -1285,7 +1325,7 @@ static void _update_level_state()
 
     for (monster_iterator mon_it; mon_it; ++mon_it)
     {
-        if (mons_allows_beogh(**mon_it))
+        if (mons_offers_beogh_conversion(**mon_it))
             env.level_state |= LSTATE_BEOGH;
         if (mon_it->has_ench(ENCH_STILL_WINDS))
             env.level_state |= LSTATE_STILL_WINDS;
@@ -1329,9 +1369,10 @@ static void _update_level_state()
     env.orb_pos = coord_def();
     if (item_def* orb = find_floor_item(OBJ_ORBS, ORB_ZOT))
         env.orb_pos = orb->pos;
-    else if (player_has_orb())
+    else if (player_has_orb() || player_equip_unrand(UNRAND_CHARLATANS_ORB))
     {
-        env.orb_pos = you.pos();
+        if (player_has_orb())
+            env.orb_pos = you.pos();
         invalidate_agrid(true);
     }
 }
