@@ -11,5 +11,7 @@ enum terrain_change_type
     TERRAIN_CHANGE_BOG,
     TERRAIN_CHANGE_SLIME,
     TERRAIN_CHANGE_BINDING_SIGIL,
+    TERRAIN_CHANGE_HELLFIRE_MORTAR,
+    TERRAIN_CHANGE_WEBS,
     NUM_TERRAIN_CHANGE_TYPES
 };
