@@ -15,6 +15,7 @@
 #include "chardump.h"
 #include "command.h"
 #include "coordit.h"
+#include "describe.h"
 #include "directn.h"
 #include "english.h"
 #include "env.h"
@@ -194,18 +195,14 @@ vector<string> fire_target_behaviour::get_monster_desc(const monster_info& mi)
 {
     vector<string> descs;
     item_def* item = active_item();
-    item_def fake_proj;
     const item_def *launcher = action.get_launcher();
-    if (launcher && is_range_weapon(*launcher))
-    {
-        populate_fake_projectile(*launcher, fake_proj);
-        item = &fake_proj;
-    }
-    if (!targeted() || !item || item->base_type != OBJ_MISSILES)
+    const bool ranged = launcher && is_range_weapon(*launcher);
+    if (!targeted() || !(ranged || (item && item->base_type == OBJ_MISSILES)))
         return descs;
 
-    ranged_attack attk(&you, nullptr, item, false);
-    descs.emplace_back(make_stringf("%d%% to hit", to_hit_pct(mi, attk, false)));
+    ostringstream result;
+    describe_to_hit(mi, result, ranged ? launcher : item);
+    descs.emplace_back(result.str());
 
     if (get_ammo_brand(*item) == SPMSL_SILVER && mi.is(MB_CHAOTIC))
         descs.emplace_back("chaotic");
@@ -456,17 +453,14 @@ static bool _returning(const item_def &item)
     return item.is_type(OBJ_MISSILES, MI_BOOMERANG);
 }
 
-static void _setup_missile_beam(const actor *agent, bolt &beam, item_def &item)
+static void _setup_missile_beam(const actor *agent, bolt &beam,
+                                item_def &item, item_def const *launcher)
 {
     const auto cglyph = get_item_glyph(item);
     beam.glyph  = cglyph.ch;
     beam.colour = cglyph.col;
     beam.was_missile = true;
 
-    item_def *launcher = nullptr;
-    if (!is_throwable(agent, item))
-        launcher = agent->weapon(0);
-
     if (agent->is_player())
     {
         beam.attitude      = ATT_FRIENDLY;
@@ -482,6 +476,7 @@ static void _setup_missile_beam(const actor *agent, bolt &beam, item_def &item)
 
     beam.range        = you.current_vision;
     beam.source_id    = agent->mid;
+    beam.launcher     = launcher;
     beam.item         = &item;
     beam.source       = agent->pos();
     beam.flavour      = BEAM_MISSILE;
@@ -532,12 +527,11 @@ static void _handle_cannon_fx(actor &act, const item_def &weapon, coord_def targ
         return;
 
     // blast smoke
-    for (fair_adjacent_iterator ai(act.pos(), false); ai; ++ai)
+    for (fair_adjacent_iterator ai(act.pos()); ai; ++ai)
     {
         if (!in_bounds(*ai)
             || cell_is_solid(*ai)
-            || cloud_at(*ai)
-            || !one_chance_in(3))
+            || cloud_at(*ai))
         {
             continue;
         }
@@ -602,8 +596,7 @@ static void _throw_noise(actor* act, const item_def &ammo)
     noisy(noise, act->pos(), msg, act->mid);
 }
 
-static void _player_shoot(bolt &pbolt, item_def &item, item_def const *launcher,
-                          int ammo_slot);
+static void _player_shoot(bolt &pbolt, item_def &item, item_def const *launcher);
 
 // throw_it - handles player throwing/firing only. Monster throwing is handled
 // in mons_throw().
@@ -611,7 +604,15 @@ static void _player_shoot(bolt &pbolt, item_def &item, item_def const *launcher,
 // refactored to be a method of quiver::ammo_action.
 void throw_it(quiver::action &a)
 {
-    const item_def *launcher = a.get_launcher();
+    const item_def *primary = a.get_launcher();
+    const item_def *offhand = you.offhand_weapon();
+    const item_def *launcher = primary;
+    const item_def *alt_launcher = offhand;
+    if (primary && offhand && is_range_weapon(*offhand) && coinflip())
+    {
+        launcher = offhand;
+        alt_launcher = primary;
+    }
     // launchers have get_item set to the launcher. But, if we are tossing
     // the launcher itself, get_launcher() will be nullptr.
     // XX can this api be simplified now that projectiles and launchers are
@@ -633,6 +634,7 @@ void throw_it(quiver::action &a)
         direction_chooser_args args;
         args.behaviour = &beh;
         args.mode = TARG_HOSTILE;
+        args.self = confirm_prompt_type::cancel;
         direction(a.target, args);
     }
     if (!a.target.isValid || a.target.isCancel)
@@ -658,11 +660,12 @@ void throw_it(quiver::action &a)
     if (ammo_slot != -1)
         item.slot     = index_to_letter(item.link);
 
-    _setup_missile_beam(&you, pbolt, item);
+    _setup_missile_beam(&you, pbolt, item, launcher);
 
     // Don't trace at all when confused.
     // Give the player a chance to be warned about helpless targets when using
     // Portaled Projectile, but obviously don't trace a path.
+    bool aimed_at_foe = false;
     if (!you.confused())
     {
         // Set values absurdly high to make sure the tracer will
@@ -682,6 +685,8 @@ void throw_it(quiver::action &a)
         pbolt.damage = dice_def();
         if (pbolt.friendly_past_target)
             pbolt.aimed_at_spot = true;
+        if (pbolt.foe_info.count)
+            aimed_at_foe = true; // dubious
 
         // Should only happen if the player answered 'n' to one of those
         // "Fire through friendly?" prompts.
@@ -701,6 +706,51 @@ void throw_it(quiver::action &a)
     // want to use tx, ty to make the missile fly to map edge.
     pbolt.set_target(a.target);
 
+    you.time_taken = you.attack_delay(&item).roll();
+    _player_shoot(pbolt, item, launcher);
+    if (ammo_slot != -1 && (pbolt.item_mulches || !_returning(item)))
+        dec_inv_item_quantity(ammo_slot, 1);
+
+    if (launcher && alt_launcher && is_range_weapon(*alt_launcher))
+    {
+        item_def alt_fake_proj;
+        populate_fake_projectile(*alt_launcher, alt_fake_proj);
+
+        bolt alt_pbolt;
+        alt_pbolt.set_target(a.target);
+        if (pbolt.friendly_past_target)
+            alt_pbolt.aimed_at_spot = true;
+        _setup_missile_beam(&you, alt_pbolt, alt_fake_proj, alt_launcher);
+        _player_shoot(alt_pbolt, alt_fake_proj, alt_launcher);
+    }
+
+    // ...any monster nearby can see that something has been thrown, even
+    // if it didn't make any noise.
+    alert_nearby_monsters();
+
+    you.turn_is_over = true;
+    if (aimed_at_foe && launcher && you.has_mutation(MUT_WARMUP_STRIKES))
+        you.rev_up(you.time_taken);
+
+    if ((launcher || is_thrown)
+        && will_have_passive(passive_t::shadow_attacks)
+        && item.base_type == OBJ_MISSILES
+        && item.sub_type != MI_DART
+        && item.sub_type != MI_THROWING_NET)
+    {
+        dithmenos_shadow_shoot(a.target, item);
+    }
+}
+
+// Once the player has committed to a target, shoot/throw/toss at it.
+static void _player_shoot(bolt &pbolt, item_def &item, item_def const *launcher)
+{
+    const int bow_brand = launcher ? get_weapon_brand(*launcher) : SPWPN_NORMAL;
+    const int ammo_brand = get_ammo_brand(item);
+    const bool returning = _returning(item);
+    const bool is_thrown = is_throwable(&you, item);
+    const bool tossing = !launcher && !is_thrown;
+
     if (launcher)
     {
         practise_launching(*launcher);
@@ -711,45 +761,18 @@ void throw_it(quiver::action &a)
         }
         else
             count_action(CACT_FIRE, launcher->sub_type);
-    } else if (is_thrown)
+    }
+    else if (is_thrown)
     {
         practise_throwing((missile_type)item.sub_type);
         count_action(CACT_THROW, item.sub_type, OBJ_MISSILES);
     }
 
-    _player_shoot(pbolt, item, launcher, ammo_slot);
-
-    // ...any monster nearby can see that something has been thrown, even
-    // if it didn't make any noise.
-    alert_nearby_monsters();
-
-    you.turn_is_over = true;
-
-    if ((launcher || is_thrown)
-        && will_have_passive(passive_t::shadow_attacks)
-        && item.base_type == OBJ_MISSILES
-        && item.sub_type != MI_DART)
-    {
-        dithmenos_shadow_throw(a.target, item);
-    }
-}
-
-// Once the player has committed to a target, shoot/throw/toss at it.
-static void _player_shoot(bolt &pbolt, item_def &item, item_def const *launcher,
-                          int ammo_slot)
-{
-    const int bow_brand = launcher ? get_weapon_brand(*launcher) : SPWPN_NORMAL;
-    const int ammo_brand = get_ammo_brand(item);
-    const bool returning = _returning(item);
-    const bool is_thrown = is_throwable(&you, item);
-    const bool tossing = !launcher && !is_thrown;
-
-    you.time_taken = you.attack_delay(&item).roll();
-
     // Create message.
-    mprf("You %s %s.",
+    mprf("You %s %s%s.",
           is_thrown ? "throw" : launcher ? "shoot" : "toss away",
-          _ammo_name(item, launcher).c_str());
+          _ammo_name(item, launcher).c_str(),
+          you.current_vision == 0 ? " into the darkness" : "");
 
     // Ensure we're firing a 'missile'-type beam.
     pbolt.pierce    = false;
@@ -771,7 +794,21 @@ static void _player_shoot(bolt &pbolt, item_def &item, item_def const *launcher,
         Hints.hints_throw_counter++;
 
     const coord_def target = pbolt.target;
-    pbolt.fire();
+
+    // XXX: Firing via beam will never hit a target outside our vision range,
+    //      so create the ranged_attack manually when bumping into something
+    //      during the start of Primordial Nightfall
+    if (you.current_vision == 0)
+    {
+        monster* mon = monster_at(target);
+        if (mon && mon->alive())
+        {
+            ranged_attack attk(&you, mon, launcher, pbolt.item, false, &you, false);
+            attk.attack();
+        }
+    }
+    else
+        pbolt.fire();
 
     if (bow_brand == SPWPN_CHAOS || ammo_brand == SPMSL_CHAOS)
         did_god_conduct(DID_CHAOS, 2 + random2(3), bow_brand == SPWPN_CHAOS);
@@ -790,8 +827,6 @@ static void _player_shoot(bolt &pbolt, item_def &item, item_def const *launcher,
         update_screen();
         pbolt.fire();
     }
-    else if (ammo_slot != -1)
-        dec_inv_item_quantity(ammo_slot, 1);
 
     _throw_noise(&you, item);
 
@@ -817,7 +852,7 @@ void setup_monster_throw_beam(monster* mons, bolt &beam)
 bool mons_throw(monster* mons, bolt &beam, bool teleport)
 {
     ASSERT(beam.item);
-    item_def &missile = *beam.item;
+    const item_def &missile = *beam.item;
     ASSERT(missile.base_type == OBJ_MISSILES);
 
     // Energy is already deducted for the spell cast, if using portal projectile
@@ -835,7 +870,10 @@ bool mons_throw(monster* mons, bolt &beam, bool teleport)
     item_def item = missile;
     item.quantity = 1;
 
-    _setup_missile_beam(mons, beam, item);
+    item_def *launcher = nullptr;
+    if (!is_throwable(mons, item))
+        launcher = mons->weapon(0);
+    _setup_missile_beam(mons, beam, item, launcher);
     beam.aimed_at_spot |= _returning(item);
     // Avoid overshooting and potentially hitting the player.
     // Piercing beams' tracers already account for this.
