@@ -247,69 +247,19 @@ mcache_monster::mcache_monster(const monster_info& mon)
 
     const item_def* mon_weapon = mon.inv[MSLOT_WEAPON].get();
     m_equ_tile = (mon_weapon != nullptr) ? tilep_equ_weapon(*mon_weapon) : 0;
-    if (mons_class_wields_two_weapons(mon.type))
-    {
-        const item_def* mon_weapon2 = mon.inv[MSLOT_ALT_WEAPON].get();
-        if (mon_weapon2)
-        {
-            switch (tilep_equ_weapon(*mon_weapon2))
-            {
-                case TILEP_HAND1_DAGGER:
-                    m_shd_tile = TILEP_HAND2_DAGGER;
-                    break;
-                case TILEP_DAGGER_1:
-                    m_shd_tile = TILEP_HAND2_DAGGER_1;
-                    break;
-                case TILEP_HAND1_RAPIER:
-                    m_shd_tile = TILEP_HAND2_RAPIER;
-                    break;
-                case TILEP_RAPIER_1:
-                    m_shd_tile = TILEP_HAND2_RAPIER_1;
-                    break;
-                default:
-                case TILEP_HAND1_SHORT_SWORD_SLANT:
-                    m_shd_tile = TILEP_HAND2_SHORT_SWORD_SLANT;
-                    break;
-                case TILEP_SHORT_SWORD_SLANT_1:
-                    m_shd_tile = TILEP_HAND2_SHORT_SWORD_SLANT_1;
-                    break;
-                case TILEP_HAND1_GREAT_FLAIL:
-                    m_shd_tile = TILEP_HAND2_GREAT_FLAIL;
-                    break;
-                case TILEP_GREAT_FLAIL_1:
-                    m_shd_tile = TILEP_HAND2_GREAT_FLAIL_1;
-                    break;
-                case TILEP_HAND1_GREAT_MACE:
-                    m_shd_tile = TILEP_HAND2_GREAT_MACE;
-                    break;
-                case TILEP_GREAT_MACE_1:
-                    m_shd_tile = TILEP_HAND2_GREAT_MACE_1;
-                    break;
-                case TILEP_HAND1_GIANT_CLUB:
-                    m_shd_tile = TILEP_HAND2_GIANT_CLUB;
-                    break;
-                case TILEP_HAND1_GIANT_CLUB_SLANT:
-                    m_shd_tile = TILEP_HAND2_GIANT_CLUB_SLANT;
-                    break;
-                case TILEP_HAND1_GIANT_CLUB_SPIKE:
-                    m_shd_tile = TILEP_HAND2_GIANT_CLUB_SPIKE;
-                    break;
-                case TILEP_HAND1_GIANT_CLUB_SPIKE_SLANT:
-                    m_shd_tile = TILEP_HAND2_GIANT_CLUB_SPIKE_SLANT;
-                    break;
-                case TILEP_HAND1_GIANT_CLUB_PLAIN:
-                    m_shd_tile = TILEP_HAND2_GIANT_CLUB_PLAIN;
-                    break;
-            };
-        }
-        else
-            m_shd_tile = 0;
-    }
-    else
+    if (!mons_class_wields_two_weapons(mon.type))
     {
         const item_def* mon_shield = mon.inv[MSLOT_SHIELD].get();
         m_shd_tile = (mon_shield != nullptr) ? tilep_equ_shield(*mon_shield) : 0;
+        return;
     }
+    const item_def* mon_weapon2 = mon.inv[MSLOT_ALT_WEAPON].get();
+    if (!mon_weapon2)
+    {
+        m_shd_tile = 0;
+        return;
+    }
+    m_shd_tile = mirror_weapon(*mon_weapon2);
 }
 
 // Returns the amount of pixels necessary to shift a wielded weapon
@@ -338,6 +288,7 @@ bool mcache_monster::get_weapon_offset(tileidx_t mon_tile,
     case TILEP_MONS_MENNAS:
     case TILEP_MONS_PROFANE_SERVITOR:
     case TILEP_MONS_OCTOPODE:
+    case TILEP_MONS_PLAYER_SHADOW_OCTOPODE:
     case TILEP_MONS_ZOMBIE_OCTOPODE:
     case TILEP_MONS_ANCESTOR:
     case TILEP_MONS_ANCESTOR_KNIGHT:
@@ -346,16 +297,20 @@ bool mcache_monster::get_weapon_offset(tileidx_t mon_tile,
     case TILEP_MONS_VAMPIRE_MAGE:
     case TILEP_MONS_NORRIS:
     case TILEP_MONS_BURIAL_ACOLYTE:
+    case TILEP_MONS_WIGLAF:
+    case TILEP_MONS_PLAYER_SHADOW_DRACONIAN:
+    case TILEP_MONS_PLAYER_SHADOW_ONI:
         *ofs_x = 0;
         *ofs_y = 0;
         break;
     // Shift to the left.
     case TILEP_MONS_DEEP_ELF_DEATH_MAGE:
     case TILEP_MONS_DEMONSPAWN:
+    case TILEP_MONS_PLAYER_SHADOW_DEMONSPAWN:
     case TILEP_MONS_DEMONSPAWN_BLOOD_SAINT:
     case TILEP_MONS_DEMONSPAWN_WARMONGER:
     case TILEP_MONS_DEMONSPAWN_CORRUPTER:
-    case TILEP_MONS_DEMONSPAWN_BLACK_SUN:
+    case TILEP_MONS_DEMONSPAWN_SOUL_SCHOLAR:
     case TILEP_MONS_VAMPIRE_KNIGHT:
     case TILEP_MONS_ZOMBIE_ORC:
     case TILEP_MONS_JORGRUN:
@@ -366,6 +321,7 @@ bool mcache_monster::get_weapon_offset(tileidx_t mon_tile,
     case TILEP_MONS_ROBIN:
     case TILEP_MONS_GNOLL_SERGEANT:
     case TILEP_MONS_TENGU:
+    case TILEP_MONS_PLAYER_SHADOW_TENGU:
     case TILEP_MONS_TENGU_CONJURER:
     case TILEP_MONS_TENGU_WARRIOR:
     case TILEP_MONS_TENGU_REAVER:
@@ -377,16 +333,21 @@ bool mcache_monster::get_weapon_offset(tileidx_t mon_tile,
         break;
     case TILEP_MONS_SKELETON_LARGE:
     case TILEP_MONS_ORC_WARLORD:
+    case TILEP_MONS_FALLEN_ORC_WARLORD:
     case TILEP_MONS_KOBOLD_BRIGAND:
     case TILEP_MONS_KOBOLD:
+    case TILEP_MONS_PLAYER_SHADOW_KOBOLD:
     case TILEP_MONS_EFREET:
+    case TILEP_MONS_PLAYER_SHADOW_DJINN:
     case TILEP_MONS_GNOLL:
+    case TILEP_MONS_PLAYER_SHADOW_GNOLL:
     case TILEP_MONS_CRAZY_YIUF:
     case TILEP_MONS_GRUM:
     case TILEP_MONS_GRUNN:
     case TILEP_MONS_GNOLL_BOUDA:
     case TILEP_MONS_NESSOS_BOWLESS:
     case TILEP_MONS_ARMATAUR:
+    case TILEP_MONS_PLAYER_SHADOW_ARMATAUR:
     case TILEP_MONS_FENSTRIDER_WITCH:
         *ofs_x = -3;
         *ofs_y = 0;
@@ -406,6 +367,7 @@ bool mcache_monster::get_weapon_offset(tileidx_t mon_tile,
     case TILEP_MONS_IRONBOUND_CONVOKER:
     case TILEP_MONS_IRONBOUND_PRESERVER:
     case TILEP_MONS_GARGOYLE:
+    case TILEP_MONS_PLAYER_SHADOW_GARGOYLE:
     case TILEP_MONS_MOLTEN_GARGOYLE:
     case TILEP_MONS_WAR_GARGOYLE:
     case TILEP_MONS_SERVANT_OF_WHISPERS:
@@ -431,6 +393,7 @@ bool mcache_monster::get_weapon_offset(tileidx_t mon_tile,
     case TILEP_MONS_HELL_KNIGHT:
     case TILEP_MONS_DEATH_KNIGHT:
     case TILEP_MONS_MUMMY:
+    case TILEP_MONS_PLAYER_SHADOW_MUMMY:
     case TILEP_MONS_MUMMY_PRIEST:
     case TILEP_MONS_GUARDIAN_MUMMY:
     case TILEP_MONS_SKELETON_SMALL:
@@ -454,6 +417,7 @@ bool mcache_monster::get_weapon_offset(tileidx_t mon_tile,
         *ofs_y = -3;
         break;
     case TILEP_MONS_NAGA:
+    case TILEP_MONS_PLAYER_SHADOW_NAGA:
     case TILEP_MONS_NAGA_WARRIOR:
     case TILEP_MONS_NAGA_MAGE:
     case TILEP_MONS_NAGA_RITUALIST:
@@ -466,6 +430,7 @@ bool mcache_monster::get_weapon_offset(tileidx_t mon_tile,
     case TILEP_MONS_DEEP_ELF_KNIGHT:
     case TILEP_MONS_GUARDIAN_SERPENT:
     case TILEP_MONS_DEEP_DWARF:
+    case TILEP_MONS_PLAYER_SHADOW_DWARF:
     case TILEP_MONS_RUST_DEVIL:
         *ofs_x = 0;
         *ofs_y = 1;
@@ -483,7 +448,9 @@ bool mcache_monster::get_weapon_offset(tileidx_t mon_tile,
     case TILEP_MONS_DEEP_ELF_DEMONOLOGIST:
     case TILEP_MONS_DEEP_ELF_ANNIHILATOR:
     case TILEP_MONS_MINOTAUR:
+    case TILEP_MONS_PLAYER_SHADOW_MINOTAUR:
     case TILEP_MONS_VAMPIRE:
+    case TILEP_MONS_PLAYER_SHADOW_VAMPIRE:
         *ofs_x = -1;
         *ofs_y = -2;
         break;
@@ -493,6 +460,7 @@ bool mcache_monster::get_weapon_offset(tileidx_t mon_tile,
         *ofs_y = -3;
         break;
     case TILEP_MONS_DEMIGOD:
+    case TILEP_MONS_PLAYER_SHADOW_DEMIGOD:
     case TILEP_MONS_KIRKE:
     case TILEP_MONS_FREDERICK:
         *ofs_x = -1;
@@ -504,6 +472,7 @@ bool mcache_monster::get_weapon_offset(tileidx_t mon_tile,
     case TILEP_MONS_SKELETAL_WARRIOR:
     case TILEP_MONS_SALAMANDER:
     case TILEP_MONS_VINE_STALKER:
+    case TILEP_MONS_PLAYER_SHADOW_VINE_STALKER:
     case TILEP_MONS_NECROPHAGE:
     case TILEP_MONS_WIGHT:
         *ofs_x = -2;
@@ -511,6 +480,7 @@ bool mcache_monster::get_weapon_offset(tileidx_t mon_tile,
         break;
     case TILEP_MONS_SPRIGGAN_BERSERKER:
     case TILEP_MONS_BARACHI:
+    case TILEP_MONS_PLAYER_SHADOW_BARACHI:
         *ofs_x = -3;
         *ofs_y = -2;
         break;
@@ -532,6 +502,8 @@ bool mcache_monster::get_weapon_offset(tileidx_t mon_tile,
     case TILEP_MONS_DEEP_ELF_ELEMENTALIST_2:
     case TILEP_MONS_DEEP_ELF_ELEMENTALIST_3:
     case TILEP_MONS_FORMICID:
+    case TILEP_MONS_PLAYER_SHADOW_FORMICID:
+    case TILEP_MONS_XAKKRIXIS:
         *ofs_x = -2;
         *ofs_y = -1;
         break;
@@ -607,12 +579,14 @@ bool mcache_monster::get_weapon_offset(tileidx_t mon_tile,
         *ofs_y = -2;
         break;
     case TILEP_MONS_ELF:
+    case TILEP_MONS_PLAYER_SHADOW_ELF:
         *ofs_x = 2;
         *ofs_y = -3;
         break;
     case TILEP_MONS_HUMAN:
     case TILEP_MONS_HUMAN_1:
     case TILEP_MONS_HUMAN_2:
+    case TILEP_MONS_PLAYER_SHADOW_HUMAN:
     case TILEP_MONS_ZENATA:
     case TILEP_MONS_METEORAN:
         *ofs_x = -1;
@@ -623,6 +597,7 @@ bool mcache_monster::get_weapon_offset(tileidx_t mon_tile,
         *ofs_y = -4;
         break;
     case TILEP_MONS_MERFOLK:
+    case TILEP_MONS_PLAYER_SHADOW_MERFOLK:
     case TILEP_MONS_MERFOLK_AQUAMANCER:
     case TILEP_MONS_MERFOLK_IMPALER:
     case TILEP_MONS_MERFOLK_JAVELINEER:
@@ -633,11 +608,13 @@ bool mcache_monster::get_weapon_offset(tileidx_t mon_tile,
         *ofs_y = -5;
         break;
     case TILEP_MONS_FRANCES: // left-handed
+                             // TODO: use a mirrored tile instead
         *ofs_x = 20;
         *ofs_y = -1;
         break;
     // Shift downwards and to the left.
     case TILEP_MONS_GOBLIN:
+    case TILEP_MONS_PLAYER_SHADOW_COGLIN:
     case TILEP_MONS_IJYB:
         *ofs_x = -2;
         *ofs_y = 3;
@@ -646,20 +623,27 @@ bool mcache_monster::get_weapon_offset(tileidx_t mon_tile,
     case TILEP_MONS_ORC_PRIEST:
     case TILEP_MONS_ORC_HIGH_PRIEST:
     case TILEP_MONS_SPRIGGAN:
+    case TILEP_MONS_PLAYER_SHADOW_SPRIGGAN:
     case TILEP_MONS_SPRIGGAN_DEFENDER:
         *ofs_x = -1;
         *ofs_y = 1;
         break;
     case TILEP_MONS_ORC_KNIGHT:
+    case TILEP_MONS_FALLEN_ORC_KNIGHT:
     case TILEP_MONS_ORC_WIZARD:
     case TILEP_MONS_ORC_SORCERER:
     case TILEP_MONS_NERGALLE:
     case TILEP_MONS_ETTIN:
+    case TILEP_MONS_STONE_GIANT:
+    case TILEP_MONS_TITAN:
     case TILEP_MONS_FROST_GIANT:
     case TILEP_MONS_FIRE_GIANT:
     case TILEP_MONS_IRON_GIANT:
     case TILEP_MONS_CACTUS_GIANT:
-    case TILEP_MONS_THE_ENCHANTRESS:
+    case TILEP_MONS_ENCHANTRESS:
+    case TILEP_MONS_ORC_APOSTLE:
+    case TILEP_MONS_ORC_APOSTLE_1:
+    case TILEP_MONS_ORC_APOSTLE_2:
         *ofs_x = -2;
         *ofs_y = 1;
         break;
@@ -693,10 +677,12 @@ bool mcache_monster::get_weapon_offset(tileidx_t mon_tile,
         *ofs_y = 3;
         break;
     case TILEP_MONS_TROLL:
+    case TILEP_MONS_PLAYER_SHADOW_TROLL:
         *ofs_x = -3;
         *ofs_y = 11;
         break;
     case TILEP_MONS_GHOUL:
+    case TILEP_MONS_PLAYER_SHADOW_GHOUL:
         *ofs_x = -3;
         *ofs_y = -3;
         break;
@@ -742,10 +728,6 @@ bool mcache_monster::get_weapon_offset(tileidx_t mon_tile,
         *ofs_x = 3;
         *ofs_y = -4;
         break;
-    case TILEP_MONS_LOROCYPROCA:
-        *ofs_x = -4;
-        *ofs_y = 6;
-        break;
     case TILEP_MONS_HELLION:
         *ofs_x = -1;
         *ofs_y = 4;
@@ -792,7 +774,7 @@ bool mcache_monster::get_weapon_offset(tileidx_t mon_tile,
         *ofs_y = -8;
         break;
     case TILEP_MONS_ABOMINATION_LARGE_2:
-    case TILEP_MONS_BLORK_THE_ORC:
+    case TILEP_MONS_BLORKULA_THE_ORCULA:
         *ofs_x = -3;
         *ofs_y = 0;
         break;
@@ -819,6 +801,7 @@ bool mcache_monster::get_weapon_offset(tileidx_t mon_tile,
     case TILEP_MONS_CHAOS_SPAWN_3:
     case TILEP_MONS_ORC:
     case TILEP_MONS_ORC_WARRIOR:
+    case TILEP_MONS_FALLEN_ORC_WARRIOR:
         *ofs_x = -2;
         *ofs_y = -1;
         break;
@@ -889,7 +872,7 @@ bool mcache_monster::get_shield_offset(tileidx_t mon_tile,
     {
     case TILEP_MONS_ORC:
     case TILEP_MONS_URUG:
-    case TILEP_MONS_BLORK_THE_ORC:
+    case TILEP_MONS_BLORKULA_THE_ORCULA:
     case TILEP_MONS_ORC_PRIEST:
     case TILEP_MONS_ORC_WARRIOR:
     case TILEP_MONS_ORC_KNIGHT:
@@ -898,6 +881,7 @@ bool mcache_monster::get_shield_offset(tileidx_t mon_tile,
     case TILEP_MONS_DEEP_ELF_KNIGHT:
     case TILEP_MONS_KIRKE:
     case TILEP_MONS_DIMME:
+    case TILEP_MONS_ORC_APOSTLE_2:
         *ofs_x = 1;
         *ofs_y = 0;
         break;
@@ -908,7 +892,7 @@ bool mcache_monster::get_shield_offset(tileidx_t mon_tile,
     case TILEP_MONS_DEMONSPAWN_BLOOD_SAINT:
     case TILEP_MONS_DEMONSPAWN_WARMONGER:
     case TILEP_MONS_DEMONSPAWN_CORRUPTER:
-    case TILEP_MONS_DEMONSPAWN_BLACK_SUN:
+    case TILEP_MONS_DEMONSPAWN_SOUL_SCHOLAR:
     case TILEP_MONS_NORRIS:
         *ofs_x = 2;
         *ofs_y = 0;
@@ -924,6 +908,7 @@ bool mcache_monster::get_shield_offset(tileidx_t mon_tile,
         *ofs_y = 2;
         break;
 
+    case TILEP_MONS_WIGLAF:
     case TILEP_MONS_FENSTRIDER_WITCH:
         *ofs_x = -1;
         *ofs_y = 1;
@@ -952,6 +937,7 @@ bool mcache_monster::get_shield_offset(tileidx_t mon_tile,
     case TILEP_MONS_TENGU_REAVER:
     case TILEP_MONS_SOJOBO:
     case TILEP_MONS_FORMICID:
+    case TILEP_MONS_XAKKRIXIS:
     case TILEP_MONS_VINE_STALKER:
     case TILEP_MONS_OCTOPODE:
     case TILEP_MONS_CHERUB:
@@ -1002,7 +988,7 @@ bool mcache_monster::get_shield_offset(tileidx_t mon_tile,
         *ofs_y = -1;
         break;
 
-    case TILEP_MONS_THE_ENCHANTRESS:
+    case TILEP_MONS_ENCHANTRESS:
         *ofs_x = 6;
         *ofs_y = 3;
         break;
@@ -1159,11 +1145,6 @@ bool mcache_monster::get_shield_offset(tileidx_t mon_tile,
         *ofs_y = -3;
         break;
 
-    case TILEP_MONS_LOROCYPROCA:
-        *ofs_x = -3;
-        *ofs_y = 0;
-        break;
-
     case TILEP_MONS_HELLION:
         *ofs_x = 0;
         *ofs_y = 4;
