@@ -92,8 +92,6 @@ TilesFramework::TilesFramework() :
       m_next_view(coord_def(GXM, GYM)),
       m_next_view_tl(0, 0),
       m_next_view_br(-1, -1),
-      m_current_flash_colour(BLACK),
-      m_next_flash_colour(BLACK),
       m_need_full_map(true),
       m_text_menu("menu_txt"),
       m_print_fg(15)
@@ -205,13 +203,17 @@ void TilesFramework::finish_message()
     m_msg_buf.append("\n");
     const char* fragment_start = m_msg_buf.data();
     const char* data_end = m_msg_buf.data() + m_msg_buf.size();
+#ifdef DEBUG_WEBSOCKETS
     int fragments = 0;
+#endif
     while (fragment_start < data_end)
     {
         int fragment_size = data_end - fragment_start;
         if (fragment_size > m_max_msg_size)
             fragment_size = m_max_msg_size;
+#ifdef DEBUG_WEBSOCKETS
         fragments++;
+#endif
 
         for (unsigned int i = 0; i < m_dest_addrs.size(); ++i)
         {
@@ -341,6 +343,7 @@ wint_t TilesFramework::_receive_control_message()
 
     srcaddr_len = sizeof(srcaddr);
 
+    // XX currently this is not interrupted by HUP
     int len = recvfrom(m_sock, buf, sizeof(buf),
                        0,
                        (sockaddr *) &srcaddr, &srcaddr_len);
@@ -989,14 +992,7 @@ static bool _update_statuses(player_info& c)
     status_info inf;
     for (unsigned int status = 0; status <= STATUS_LAST_STATUS; ++status)
     {
-        if (status == DUR_DIVINE_SHIELD)
-        {
-            inf = status_info();
-            if (!you.duration[status])
-                continue;
-            inf.short_text = "divinely shielded";
-        }
-        else if (status == DUR_ICEMAIL_DEPLETED)
+        if (status == DUR_ICEMAIL_DEPLETED)
         {
             inf = status_info();
             if (you.duration[status] <= ICEMAIL_TIME / ICEMAIL_MAX)
@@ -1093,7 +1089,7 @@ void TilesFramework::_send_player(bool force_full)
                    "title");
     _update_int(force_full, c.wizard, you.wizard, "wizard");
     _update_int(force_full, c.explore, you.explore, "explore");
-    _update_string(force_full, c.species, species::name(you.species),
+    _update_string(force_full, c.species, player_species_name(),
                    "species");
     string god = "";
     if (you_worship(GOD_JIYVA))
@@ -1135,8 +1131,8 @@ void TilesFramework::_send_player(bool force_full)
     _update_int(force_full, c.poison_survival, max(0, poison_survival()),
                 "poison_survival");
 
-    _update_int(force_full, c.armour_class, you.armour_class(), "ac");
-    _update_int(force_full, c.evasion, you.evasion(), "ev");
+    _update_int(force_full, c.armour_class, you.armour_class_scaled(1), "ac");
+    _update_int(force_full, c.evasion, you.evasion_scaled(1), "ev");
     _update_int(force_full, c.shield_class, player_displayed_shield_class(),
                 "sh");
 
@@ -1235,8 +1231,12 @@ void TilesFramework::_send_player(bool force_full)
     {
         json_open_object(to_string(i));
         item_def item = get_item_known_info(you.inv[i]);
-        if ((char)i == you.equip[EQ_WEAPON] && is_weapon(item) && you.corrosion_amount())
-            item.plus -= 4 * you.corrosion_amount();
+        if (((char)i == you.equip[EQ_WEAPON] && is_weapon(item)
+             || (char)i == you.equip[EQ_OFFHAND] && you.offhand_weapon())
+            && you.corrosion_amount())
+        {
+            item.plus -= 1 * you.corrosion_amount();
+        }
         _send_item(c.inv[i], item, c.inv_uselessness[i], force_full);
         json_close_object(true);
     }
@@ -1250,6 +1250,9 @@ void TilesFramework::_send_player(bool force_full)
     }
     json_close_object(true);
 
+    _update_int(force_full, c.offhand_weapon, (bool) you.offhand_weapon(),
+                "offhand_weapon");
+
     _update_int(force_full, c.quiver_item,
                 (int8_t) you.quiver_action.get()->get_item(), "quiver_item");
 
@@ -1421,51 +1424,10 @@ void TilesFramework::_send_item(item_def& current, const item_def& next,
 
 void TilesFramework::send_doll(const dolls_data &doll, bool submerged, bool ghost)
 {
-    // Ordered from back to front.
-    // FIXME: Implement this logic in one place in e.g. pack_doll_buf().
-    int p_order[TILEP_PART_MAX] =
-    {
-        // background
-        TILEP_PART_SHADOW,
-        TILEP_PART_HALO,
-        TILEP_PART_ENCH,
-        TILEP_PART_DRCWING,
-        TILEP_PART_CLOAK,
-        // player
-        TILEP_PART_BASE,
-        TILEP_PART_BOOTS,
-        TILEP_PART_LEG,
-        TILEP_PART_BODY,
-        TILEP_PART_ARM,
-        TILEP_PART_HAIR,
-        TILEP_PART_BEARD,
-        TILEP_PART_HELM,
-        TILEP_PART_HAND1,
-        TILEP_PART_HAND2,
-    };
+    static int p_order[TILEP_PART_MAX];
+    static int flags[TILEP_PART_MAX];
 
-    int flags[TILEP_PART_MAX];
-    tilep_calc_flags(doll, flags);
-
-    // For skirts, boots go under the leg armour. For pants, they go over.
-    if (doll.parts[TILEP_PART_LEG] < TILEP_LEG_SKIRT_OFS)
-    {
-        p_order[7] = TILEP_PART_BOOTS;
-        p_order[6] = TILEP_PART_LEG;
-    }
-
-    // Draw scarves above other clothing.
-    if (doll.parts[TILEP_PART_CLOAK] >= TILEP_CLOAK_SCARF_FIRST_NORM)
-    {
-        p_order[4] = p_order[5];
-        p_order[5] = p_order[6];
-        p_order[6] = p_order[7];
-        p_order[7] = p_order[8];
-        p_order[8] = p_order[9];
-        p_order[9] = TILEP_PART_CLOAK;
-    }
-
-    reveal_bardings(doll.parts, flags);
+    tilep_fill_order_and_flags(doll, p_order, flags);
 
     tiles.json_open_array("doll");
 
@@ -1594,6 +1556,10 @@ void TilesFramework::_send_cell(const coord_def &gc,
         col = (_get_highlight(col) << 4) | macro_colour(col & 0xF);
         json_write_int("col", col);
     }
+    if (current_sc.flash_colour != next_sc.flash_colour)
+        json_write_int("flc", next_sc.flash_colour);
+    if (current_sc.flash_alpha != next_sc.flash_alpha)
+        json_write_int("fla", next_sc.flash_alpha);
 
     json_open_object("t");
     {
@@ -1654,6 +1620,11 @@ void TilesFramework::_send_cell(const coord_def &gc,
 
         if (next_pc.is_sanctuary != current_pc.is_sanctuary)
             json_write_bool("sanctuary", next_pc.is_sanctuary);
+        if (next_pc.is_blasphemy != current_pc.is_blasphemy)
+            json_write_bool("blasphemy", next_pc.is_blasphemy);
+
+        if (next_pc.has_bfb_corpse != current_pc.has_bfb_corpse)
+            json_write_bool("has_bfb_corpse", next_pc.has_bfb_corpse);
 
         if (next_pc.is_liquefied != current_pc.is_liquefied)
             json_write_bool("liquefied", next_pc.is_liquefied);
@@ -1720,7 +1691,7 @@ void TilesFramework::_send_cell(const coord_def &gc,
             if (fg_changed || player_doll_changed)
             {
                 send_doll(last_player_doll, in_water, false);
-                if (Options.tile_use_monster != MONS_0)
+                if (player_uses_monster_tile())
                 {
                     monster_info minfo(MONS_PLAYER, MONS_PLAYER);
                     minfo.props[MONSTER_TILE_KEY] =
@@ -1732,10 +1703,10 @@ void TilesFramework::_send_cell(const coord_def &gc,
                             get_item_known_info(*you.slot_item(EQ_WEAPON)));
                         minfo.inv[MSLOT_WEAPON].reset(item);
                     }
-                    if (you.slot_item(EQ_SHIELD))
+                    if (you.slot_item(EQ_OFFHAND))
                     {
                         item = new item_def(
-                            get_item_known_info(*you.slot_item(EQ_SHIELD)));
+                            get_item_known_info(*you.slot_item(EQ_OFFHAND)));
                         minfo.inv[MSLOT_SHIELD].reset(item);
                     }
                     tileidx_t mcache_idx = mcache.register_monster(minfo);
@@ -1879,6 +1850,10 @@ void TilesFramework::_send_map(bool force_full)
     coord_def last_gc(0, 0);
     bool send_gc = true;
 
+    int flash_colour = you.flash_colour;
+    if (flash_colour == BLACK)
+        flash_colour = viewmap_flash_colour();
+
     json_open_array("cells");
     for (int y = 0; y < GYM; y++)
         for (int x = 0; x < GXM; x++)
@@ -1892,7 +1867,11 @@ void TilesFramework::_send_map(bool force_full)
             {
                 screen_cell_t *cell = &m_next_view(gc);
 
-                draw_cell(cell, gc, false, m_current_flash_colour);
+                if (you.flash_where && you.flash_where->is_affected(gc) <= 0)
+                    draw_cell(cell, gc, false, 0);
+                else
+                    draw_cell(cell, gc, false, flash_colour);
+
                 pack_cell_overlays(gc, m_next_view);
             }
 
@@ -2026,10 +2005,6 @@ void TilesFramework::load_dungeon(const crawl_view_buffer &vbuf,
     if (m_ui_state == UI_CRT)
         set_ui_state(UI_NORMAL);
 
-    m_next_flash_colour = you.flash_colour;
-    if (m_next_flash_colour == BLACK)
-        m_next_flash_colour = viewmap_flash_colour();
-
     // First re-render the area that was covered by vbuf the last time
     for (int y = m_next_view_tl.y; y <= m_next_view_br.y; y++)
         for (int x = m_next_view_tl.x; x <= m_next_view_br.x; x++)
@@ -2122,8 +2097,6 @@ void TilesFramework::_send_everything()
     _send_ui_state(m_ui_state);
     m_last_ui_state = m_ui_state;
 
-    send_message("{\"msg\":\"flash\",\"col\":%d}", m_current_flash_colour);
-
     _send_cursor(CURSOR_MOUSE);
     _send_cursor(CURSOR_TUTORIAL);
 
@@ -2237,15 +2210,7 @@ void TilesFramework::redraw()
     _send_messages();
 
     if (m_need_redraw && m_view_loaded)
-    {
-        if (m_current_flash_colour != m_next_flash_colour)
-        {
-            send_message("{\"msg\":\"flash\",\"col\":%d}",
-                         m_next_flash_colour);
-            m_current_flash_colour = m_next_flash_colour;
-        }
         _send_map(false);
-    }
 
     m_need_redraw = false;
     m_last_tick_redraw = get_milliseconds();
