@@ -16,6 +16,7 @@ my %found_funcs   = ();
 
 my %field_type = (
     AC       => "num",
+    ACROBAT  => "bool",
     ANGRY    => "num",
     APPEAR   => "str",
     ARCHMAGI => "bool",
@@ -71,6 +72,7 @@ my %field_type = (
     RANDAPP  => "bool",
     RCORR    => "bool",
     REGEN    => "num",
+    REGEN_MP => "num",
     RMSL     => "bool",
     RMUT     => "bool",
     SEEINV   => "bool",
@@ -549,6 +551,7 @@ my @art_order = (
     "SH", "HARM", "RAMPAGE", "ARCHMAGI", "ENH_CONJ", "ENH_HEXES", "\n",
     "ENH_SUMM", "ENH_NECRO", "ENH_TLOC", "unused", "ENH_FIRE", "\n",
     "ENH_ICE", "ENH_AIR", "ENH_EARTH", "ENH_ALCH", "\n",
+    "ACROBAT", "REGEN_MP",
     "}",
 # end TAG_MAJOR_VERSION
 # start TAG_MAJOR_VERSION == 35
@@ -563,6 +566,7 @@ my @art_order = (
 #     "SH", "HARM", "RAMPAGE", "ARCHMAGI", "ENH_CONJ", "ENH_HEXES", "\n",
 #     "ENH_SUMM", "ENH_NECRO", "ENH_TLOC", "ENH_FIRE", "\n",
 #     "ENH_ICE", "ENH_AIR", "ENH_EARTH", "ENH_ALCH", "\n",
+#     "ACROBAT", "REGEN_MP",
 #     "}",
 # end TAG_MAJOR_VERSION
 
