@@ -113,6 +113,22 @@ static const zap_info zap_data[] =
     false,
 },
 
+{
+    ZAP_CRYSTALLIZING_SHOT,
+    "crystallizing shot",
+    200,
+    nullptr,
+    nullptr,
+    new dicedef_calculator<2, 5, 1, 10>,
+    new tohit_calculator<15, 1, 30>,
+    LIGHTCYAN,
+    false,
+    BEAM_CRYSTALLIZING,
+    DCHAR_FIRED_MISSILE,
+    false,
+    false,
+},
+
 {
     ZAP_SPLINTERSPRAY,
     "spray of wooden splinters",
@@ -258,7 +274,7 @@ static const zap_info zap_data[] =
 },
 
 {
-    ZAP_ICY_FLASH_FREEZE,
+    ZAP_FLASH_FREEZE,
     "icy flash freeze",
     200,
     nullptr,
@@ -267,14 +283,14 @@ static const zap_info zap_data[] =
     new tohit_calculator<5, 1, 3>,
     WHITE,
     false,
-    BEAM_ICE, // rC capped at 50%, use ZAP_FLASH_FREEZE if you don't want this
+    BEAM_ICE,
     DCHAR_FIRED_ZAP,
     false,
     false,
 },
 
 {
-    ZAP_FLASH_FREEZE,
+    ZAP_CREEPING_FROST,
     "flash freeze",
     200,
     nullptr,
@@ -283,12 +299,44 @@ static const zap_info zap_data[] =
     new tohit_calculator<5, 1, 3>,
     WHITE,
     false,
-    BEAM_COLD, // normal rC (e.g. fully resisted at rC+++)
+    BEAM_COLD,
     DCHAR_FIRED_ZAP,
     false,
     false,
 },
 
+{
+    ZAP_PERMAFROST_ERUPTION_COLD,
+    "piercing cold",
+    200,
+    new calcdice_calculator<4, 7, 2, 5>,
+    new tohit_calculator<AUTOMATIC_HIT>,
+    new dicedef_calculator<3, 7, 1, 20>,
+    new tohit_calculator<AUTOMATIC_HIT>,
+    WHITE,
+    false,
+    BEAM_COLD,
+    DCHAR_FIRED_ZAP,
+    false,
+    true,
+},
+
+{
+    ZAP_PERMAFROST_ERUPTION_EARTH,
+    "mass of rock",
+    200,
+    new calcdice_calculator<4, 7, 2, 5>,
+    new tohit_calculator<AUTOMATIC_HIT>,
+    new dicedef_calculator<3, 7, 1, 20>,
+    new tohit_calculator<AUTOMATIC_HIT>,
+    LIGHTGREY,
+    false,
+    BEAM_MMISSILE,
+    DCHAR_FIRED_MISSILE,
+    false,
+    false,
+},
+
 {
     ZAP_SPIT_LAVA,
     "glob of lava",
@@ -433,6 +481,22 @@ static const zap_info zap_data[] =
     false,
 },
 
+{
+    ZAP_BREATHE_ACID,
+    "glob of acid",
+    50,
+    new dicedef_calculator<5, 4, 1, 3>,
+    new tohit_calculator<7, 1, 6>,
+    new dicedef_calculator<3, 14, 0, 1>,
+    new tohit_calculator<20, 1, 4>,
+    YELLOW,
+    false,
+    BEAM_ACID,
+    DCHAR_FIRED_ZAP,
+    false,
+    false,
+},
+
 {
     ZAP_SLOW,
     "",
@@ -539,7 +603,7 @@ _mon_hex_zap(ZAP_PARALYSE, BEAM_PARALYSIS),
     new tohit_calculator<10, 1, 25>,
     // Water attack is weaker than the pure elemental damage attacks, but also
     // less resistible.
-    new dicedef_calculator<3, 6, 1, 12>,
+    new dicedef_calculator<3, 6, 1, 10>,
     // Huge wave of water is hard to dodge.
     new tohit_calculator<14, 1, 35>,
     LIGHTBLUE,
@@ -585,6 +649,22 @@ _mon_hex_zap(ZAP_TUKIMAS_DANCE, BEAM_TUKIMAS_DANCE, 100),
     false,
 },
 
+{
+    ZAP_HELLFIRE_MORTAR_DIG,
+    "",
+    200,
+    nullptr,
+    nullptr,
+    nullptr,
+    nullptr,
+    RED,
+    true,
+    BEAM_DIGGING,
+    DCHAR_FIRED_ZAP,
+    false,
+    false,
+},
+
 {
     ZAP_FASTROOT,
     "",
@@ -734,12 +814,12 @@ _mon_hex_zap(ZAP_POLYMORPH, BEAM_POLYMORPH),
 
 {
     ZAP_KISS_OF_DEATH,
-    "negative energy",
+    "fragment of death",
     25,
     new calcdice_calculator<2, 11, 3, 5>, // caps at 2d13... spooky
     new tohit_calculator<AUTOMATIC_HIT>, // XXX: should we let this miss?
-    nullptr,
-    nullptr,
+    new calcdice_calculator<2, 13, 2, 7>,
+    new tohit_calculator<AUTOMATIC_HIT>,
     DARKGREY,
     false,
     BEAM_NEG,
@@ -855,8 +935,8 @@ _mon_hex_zap(ZAP_BANISHMENT, BEAM_BANISH, 150),
     25,
     new dicedef_calculator<1, 3, 1, 4>,
     new tohit_calculator<8, 1, 5>,
-    new dicedef_calculator<1, 6, 1, 25>,
-    new tohit_calculator<60, 0, 1>,
+    new dicedef_calculator<2, 4, 1, 10>,
+    new tohit_calculator<12, 1, 5>,
     GREEN,
     false,
     BEAM_POISON_ARROW,
@@ -935,7 +1015,7 @@ _mon_hex_zap(ZAP_BANISHMENT, BEAM_BANISH, 150),
     25,
     new dicedef_calculator<1, 3, 1, 4>,
     new tohit_calculator<8, 1, 7>,
-    new dicedef_calculator<1, 8, 1, 20>,
+    new dicedef_calculator<1, 8, 1, 6>,
     new tohit_calculator<17, 1, 20>,
     LIGHTCYAN,
     false,
@@ -1026,13 +1106,13 @@ _mon_hex_zap(ZAP_BANISHMENT, BEAM_BANISH, 150),
 },
 
 {
-    ZAP_BREATHE_FROST,
-    "freezing breath",
+    ZAP_GLACIAL_BREATH,
+    "glacial breath",
     50,
-    new dicedef_calculator<3, 4, 1, 3>,
-    new tohit_calculator<8, 1, 6>,
-    new dicedef_calculator<3, 0, 1, 9>,
-    new tohit_calculator<30>,
+    new dicedef_calculator<3, 6, 4, 3>,
+    new tohit_calculator<12, 1, 1>,
+    nullptr,
+    nullptr,
     WHITE,
     false,
     BEAM_COLD,
@@ -1042,18 +1122,18 @@ _mon_hex_zap(ZAP_BANISHMENT, BEAM_BANISH, 150),
 },
 
 {
-    ZAP_BREATHE_ACID,
-    "glob of acid",
+    ZAP_COMBUSTION_BREATH,
+    "explosive embers",
     50,
-    new dicedef_calculator<5, 4, 1, 3>,
-    new tohit_calculator<7, 1, 6>,
-    new dicedef_calculator<3, 14, 0, 1>,
-    new tohit_calculator<20, 1, 4>,
-    YELLOW,
+    new dicedef_calculator<0, 0, 0, 1>, // deals damage through explosions
+    new tohit_calculator<AUTOMATIC_HIT>,
+    nullptr,
+    nullptr,
+    RED,
     false,
-    BEAM_ACID,
+    BEAM_MISSILE, // To avoid printing needless messages for non-damaging hits
     DCHAR_FIRED_ZAP,
-    false,
+    true,
     false,
 },
 
@@ -1073,22 +1153,6 @@ _mon_hex_zap(ZAP_BANISHMENT, BEAM_BANISH, 150),
     false,
 },
 
-{
-    ZAP_BREATHE_POWER,
-    "bolt of dispelling energy",
-    50,
-    new dicedef_calculator<3, 3, 1, 3>,
-    new tohit_calculator<5, 1, 6>,
-    nullptr,
-    nullptr,
-    BLUE,
-    false,
-    BEAM_MMISSILE,
-    DCHAR_FIRED_ZAP,
-    true,
-    false,
-},
-
 {
     ZAP_SPIT_ACID,
     "splash of acid",
@@ -1173,7 +1237,7 @@ _mon_hex_zap(ZAP_BANISHMENT, BEAM_BANISH, 150),
     ZAP_BREATHE_STEAM,
     "ball of steam",
     50,
-    new dicedef_calculator<3, 4, 1, 5>,
+    new dicedef_calculator<3, 4, 2, 3>,
     new tohit_calculator<10, 1, 10>,
     new dicedef_calculator<3, 7, 1, 15>,
     new tohit_calculator<20, 1, 20>,
@@ -1226,7 +1290,7 @@ _mon_hex_zap(ZAP_HIBERNATION, BEAM_HIBERNATION),
     50,
     new dicedef_calculator<2, 4, 1, 3>,
     new tohit_calculator<15, 1, 10>,
-    new dicedef_calculator<3, 5, 1, 40>,
+    new dicedef_calculator<3, 7, 1, 18>,
     new tohit_calculator<20, 1, 40>,
     BROWN,
     false,
@@ -1306,11 +1370,11 @@ _mon_hex_zap(ZAP_SLEEP, BEAM_SLEEP),
 },
 
 {
-    ZAP_BREATHE_MEPHITIC,
+    ZAP_NOXIOUS_BREATH,
     "blast of choking fumes",
     50,
-    new dicedef_calculator<3, 2, 1, 6>,
-    new tohit_calculator<6, 1, 6>,
+    new dicedef_calculator<0, 1, 1, 1>,
+    new tohit_calculator<AUTOMATIC_HIT>,
     nullptr,
     nullptr,
     GREEN,
@@ -1359,8 +1423,8 @@ _mon_hex_zap(ZAP_SLEEP, BEAM_SLEEP),
     50,
     new calcdice_calculator<2, 9, 1, 7>,
     new tohit_calculator<11, 1, 8>,
-    nullptr,
-    nullptr,
+    new calcdice_calculator<2, 7, 1, 4>,
+    new tohit_calculator<11, 1, 7>,
     WHITE,
     false,
     BEAM_MMISSILE,
@@ -1370,21 +1434,38 @@ _mon_hex_zap(ZAP_SLEEP, BEAM_SLEEP),
 },
 
 {
-    ZAP_CRYSTAL_BOLT,
-    "crystal bolt",
+    ZAP_REBOUNDING_BLAZE,
+    "rebounding bolt of fire",
     200,
     new calcdice_calculator<6, 18, 2, 3>,
     new tohit_calculator<10, 1, 25>,
-    new dicedef_calculator<3, 8, 1, 11>,
+    new dicedef_calculator<3, 11, 1, 11>,
     new tohit_calculator<17, 1, 25>,
-    GREEN,
+    RED,
     false,
-    BEAM_CRYSTAL,
+    BEAM_FIRE,
     DCHAR_FIRED_ZAP,
     true,
     false,
 },
 
+{
+    ZAP_REBOUNDING_CHILL,
+    "rebounding bolt of cold",
+    200,
+    new calcdice_calculator<6, 18, 2, 3>,
+    new tohit_calculator<10, 1, 25>,
+    new dicedef_calculator<3, 11, 1, 11>,
+    new tohit_calculator<17, 1, 25>,
+    WHITE,
+    false,
+    BEAM_COLD,
+    DCHAR_FIRED_ZAP,
+    true,
+    false,
+},
+
+
 {
     ZAP_QUICKSILVER_BOLT,
     "bolt of dispelling energy",
@@ -1637,9 +1718,9 @@ _mon_hex_zap(ZAP_VITRIFY, BEAM_VITRIFY),
     ZAP_DRAIN_LIFE,
     "drain life",
     200,
-    new calcdice_calculator<1, 10, 1, 1>,
-    new tohit_calculator<40>,
-    new calcdice_calculator<1, 10, 1, 1>,
+    nullptr,
+    nullptr,
+    new dicedef_calculator<2, 6, 7, 10>,
     new tohit_calculator<40>,
     DARKGREY,
     false,
@@ -1794,16 +1875,16 @@ _mon_hex_zap(ZAP_VITRIFY, BEAM_VITRIFY),
 },
 
 {
-    ZAP_NECROTISE,
+    ZAP_SOUL_SPLINTER,
     "",
-    100,
-    new dicedef_calculator<1, 4, 1, 5>,
-    new tohit_calculator<0, 7, 2>,
+    25,
     nullptr,
+    new tohit_calculator<6, 1, 2>,
     nullptr,
+    new tohit_calculator<0, 4, 1>,
     BLACK,
     true,
-    BEAM_NECROTISE,
+    BEAM_SOUL_SPLINTER,
     NUM_DCHAR_TYPES,
     false,
     false,
@@ -1857,4 +1938,420 @@ _mon_hex_zap(ZAP_VITRIFY, BEAM_VITRIFY),
     false,
 },
 
+{
+    ZAP_HURL_TORCHLIGHT,
+    "umbral torchlight",
+    200,
+    new dicedef_calculator<3, 7, 1, 5>,
+    new tohit_calculator<40>,
+    new dicedef_calculator<3, 6, 1, 13>,
+    new tohit_calculator<40>,
+    CYAN,
+    false,
+    BEAM_UMBRAL_TORCHLIGHT,
+    DCHAR_FIRED_ZAP,
+    false,
+    true,
+},
+
+{
+    ZAP_WARP_SPACE,
+    "rupture",
+    200,
+    new calcdice_calculator<3, 12, 3, 5>,
+    new tohit_calculator<40>,
+    new dicedef_calculator<3, 4, 1, 12>,
+    new tohit_calculator<40>,
+    LIGHTMAGENTA,
+    false,
+    BEAM_WARPING,
+    DCHAR_FIRED_ZAP,
+    false,
+    true,
+},
+
+{
+    ZAP_CAUSTIC_BREATH,
+    "acid spray",
+    200,
+    new dicedef_calculator<3, 3, 1, 1>,
+    new tohit_calculator<30>,
+    new dicedef_calculator<3, 3, 1, 1>,
+    new tohit_calculator<30>,
+    YELLOW,
+    false,
+    BEAM_ACID,
+    DCHAR_FIRED_ZAP,
+    true,
+    false,
+},
+
+{
+    ZAP_NULLIFYING_BREATH,
+    "nullifying energy",
+    50,
+    new dicedef_calculator<3, 4, 1, 3>,
+    new tohit_calculator<AUTOMATIC_HIT>,
+    nullptr,
+    nullptr,
+    MAGENTA,
+    false,
+    BEAM_MMISSILE,
+    DCHAR_FIRED_ZAP,
+    false,
+    true,
+},
+
+{
+    ZAP_MUD_BREATH,
+    "ball of mud",
+    200,
+    new dicedef_calculator<3, 3, 1, 1>,
+    new tohit_calculator<30>,
+    nullptr,
+    nullptr,
+    BROWN,
+    false,
+    BEAM_MISSILE,
+    DCHAR_FIRED_ZAP,
+    false,
+    false,
+},
+
+{
+    ZAP_GALVANIC_BREATH,
+    "arc of electricity",
+    200,
+    new dicedef_calculator<3, 3, 9, 10>,
+    new tohit_calculator<AUTOMATIC_HIT>,
+    nullptr,
+    nullptr,
+    LIGHTCYAN,
+    false,
+    BEAM_ELECTRICITY,
+    DCHAR_FIRED_ZAP,
+    false,
+    false,
+},
+
+{
+    ZAP_AUTODAZZLE,
+    "blinding ray",
+    50,
+    new dicedef_calculator<0, 1, 0, 1>,
+    new tohit_calculator<10, 1, 25>,
+    nullptr,
+    nullptr,
+    WHITE,
+    false,
+    BEAM_LIGHT,
+    DCHAR_FIRED_ZAP,
+    true,
+    false,
+},
+
+{
+    ZAP_GRAVITAS,
+    "crushing gravity",
+    100,
+    new dicedef_calculator<3, 6, 1, 14>,
+    new tohit_calculator<AUTOMATIC_HIT>,
+    nullptr,
+    nullptr,
+    LIGHTMAGENTA,
+    false,
+    BEAM_MMISSILE,
+    DCHAR_FIRED_ZAP,
+    false,
+    true,
+},
+
+{
+    ZAP_MAGNAVOLT,
+    "arc of electricity",
+    200,
+    new dicedef_calculator<4, 9, 1, 10>,
+    new tohit_calculator<11, 1, 20>,
+    nullptr,
+    nullptr,
+    LIGHTCYAN,
+    false,
+    BEAM_ELECTRICITY,
+    DCHAR_FIRED_ZAP,
+    true,
+    false,
+},
+
+{
+    ZAP_FULSOME_FUSILLADE,
+    "volatile concoction",
+    200,
+    new dicedef_calculator<3, 5, 1, 8>,
+    new tohit_calculator<AUTOMATIC_HIT>,
+    nullptr,
+    nullptr,
+    WHITE,
+    false,
+    BEAM_MMISSILE,
+    DCHAR_FIRED_ZAP,
+    false,
+    false,
+},
+
+{
+    ZAP_RIMEBLIGHT,
+    "",
+    200,
+    nullptr,
+    new tohit_calculator<0, 1, 3>,
+    nullptr,
+    new tohit_calculator<0, 1, 3>,
+    BLUE,
+    true,
+    BEAM_RIMEBLIGHT,
+    NUM_DCHAR_TYPES,
+    false,
+    false,
+},
+
+{
+    ZAP_RIMEBLIGHT_SHARDS,
+    "shards of blighted ice",
+    200,
+    new dicedef_calculator<3, 6, 1, 8>,
+    new tohit_calculator<AUTOMATIC_HIT>,
+    nullptr,
+    nullptr,
+    ETC_ICE,
+    false,
+    BEAM_ICE,
+    DCHAR_FIRED_BURST,
+    false,
+    true,
+},
+
+{
+    ZAP_HOARFROST_BULLET,
+    "shard of alchemical ice",
+    100,
+    nullptr,
+    nullptr,
+    new dicedef_calculator<3, 3, 1, 16>,
+    new tohit_calculator<13, 1, 35>,
+    WHITE,
+    false,
+    BEAM_ICE,
+    DCHAR_FIRED_MISSILE,
+    false,
+    false,
+},
+
+{
+    ZAP_HOARFROST_BULLET_FINALE,
+    "salvo of alchemical ice",
+    100,
+    nullptr,
+    nullptr,
+    new dicedef_calculator<3, 6, 1, 8>,
+    new tohit_calculator<AUTOMATIC_HIT>,
+    WHITE,
+    false,
+    BEAM_ICE,
+    DCHAR_FIRED_MISSILE,
+    false,
+    true
+},
+
+{
+    ZAP_FLASHING_BALESTRA,
+    "flashing steel",
+    200,
+    nullptr,
+    nullptr,
+    new dicedef_calculator<3, 7, 1, 12>,
+    new tohit_calculator<20, 1, 25>,
+    LIGHTCYAN,
+    false,
+    BEAM_MMISSILE,
+    DCHAR_FIRED_MISSILE,
+    false,
+    false,
+},
+
+{
+    ZAP_SHADOW_SHARD,
+    "shard of hardened shadow",
+    200,
+    nullptr,
+    nullptr,
+    new dicedef_calculator<3, 7, 1, 9>,
+    new tohit_calculator<20, 1, 10>,
+    MAGENTA,
+    false,
+    BEAM_MMISSILE,
+    DCHAR_FIRED_ZAP,
+    false,
+    false,
+},
+
+{
+    ZAP_SHADOW_BOLT,
+    "ray of shadow",
+    200,
+    nullptr,
+    nullptr,
+    new dicedef_calculator<3, 6, 1, 13>,
+    new tohit_calculator<20, 1, 15>,
+    MAGENTA,
+    false,
+    BEAM_MMISSILE,
+    DCHAR_FIRED_ZAP,
+    true,
+    false,
+},
+
+{
+    ZAP_SHADOWBALL,
+    "shadowball",
+    200,
+    nullptr,
+    nullptr,
+    new dicedef_calculator<3, 4, 1, 14>,
+    new tohit_calculator<AUTOMATIC_HIT>,
+    MAGENTA,
+    false,
+    BEAM_MMISSILE,
+    DCHAR_FIRED_ZAP,
+    false,
+    true,
+},
+
+{
+    ZAP_CREEPING_SHADOW,
+    "creeping shadow",
+    200,
+    nullptr,
+    nullptr,
+    new dicedef_calculator<3, 4, 1, 14>,
+    new tohit_calculator<AUTOMATIC_HIT>,
+    MAGENTA,
+    false,
+    BEAM_MMISSILE,
+    DCHAR_FIRED_ZAP,
+    false,
+    false,
+},
+
+{
+    ZAP_SHADOW_JOLT,
+    "shadowy lightning",
+    200,
+    nullptr,
+    nullptr,
+    new dicedef_calculator<3, 4, 1, 16>,
+    new tohit_calculator<15, 1, 20>,
+    MAGENTA,
+    false,
+    BEAM_MMISSILE,
+    DCHAR_FIRED_ZAP,
+    false,
+    false,
+},
+
+{
+    ZAP_SHADOW_DRAIN,
+    "wave of devouring shadow",
+    200,
+    nullptr,
+    nullptr,
+    new dicedef_calculator<3, 3, 1, 18>,
+    new tohit_calculator<AUTOMATIC_HIT>,
+    MAGENTA,
+    false,
+    BEAM_MMISSILE,
+    DCHAR_FIRED_ZAP,
+    false,
+    false,
+},
+
+{
+    ZAP_SHADOW_TORPOR,
+    "",
+    200,
+    nullptr,
+    new tohit_calculator<0, 1, 3>,
+    nullptr,
+    new tohit_calculator<0, 1, 3>,
+    BLUE,
+    true,
+    BEAM_SHADOW_TORPOR,
+    NUM_DCHAR_TYPES,
+    true,
+    false,
+},
+
+{
+    ZAP_GRAVE_CLAW,
+    "jagged bones",
+    50,
+    new dicedef_calculator<2, 7, 1, 7>,
+    new tohit_calculator<AUTOMATIC_HIT>,
+    new dicedef_calculator<2, 4, 1, 9>,
+    new tohit_calculator<AUTOMATIC_HIT>,
+    WHITE,
+    false,
+    BEAM_MMISSILE,
+    NUM_DCHAR_TYPES,
+    false,
+    false,
+},
+
+{
+    ZAP_UNLEASH_DESTRUCTION,
+    "gout of destruction",
+    100,
+    new dicedef_calculator<3, 5, 1, 2>,
+    new tohit_calculator<10, 1, 7>,
+    nullptr,
+    nullptr,
+    WHITE,
+    false,
+    BEAM_MMISSILE,
+    DCHAR_FIRED_ZAP,
+    false,
+    false,
+},
+
+{
+    ZAP_UNLEASH_DESTRUCTION_PIERCING,
+    "gout of destruction",
+    100,
+    new dicedef_calculator<3, 5, 1, 2>,
+    new tohit_calculator<10, 1, 7>,
+    nullptr,
+    nullptr,
+    WHITE,
+    false,
+    BEAM_MMISSILE,
+    DCHAR_FIRED_ZAP,
+    true,
+    false,
+},
+
+{
+    ZAP_BLOOD_ARROW,
+    "blood arrow",
+    200,
+    new dicedef_calculator<4, 3, 27, 100>,
+    new tohit_calculator<20, 1, 7>,
+    nullptr,
+    nullptr,
+    LIGHTRED,
+    false,
+    BEAM_BLOODRITE,
+    DCHAR_FIRED_ZAP,
+    false,
+    false,
+},
+
 };
