@@ -35,10 +35,9 @@ enum zap_type
     ZAP_SPIT_POISON,
     ZAP_DEBUGGING_RAY,
     ZAP_BREATHE_FIRE,
-    ZAP_BREATHE_FROST,
+    ZAP_GLACIAL_BREATH,
     ZAP_BREATHE_ACID,
     ZAP_BREATHE_POISON,
-    ZAP_BREATHE_POWER,
     ZAP_AGONY,
     ZAP_MINDBURST,
     ZAP_BREATHE_STEAM,
@@ -53,11 +52,12 @@ enum zap_type
     ZAP_SLEEP,
     ZAP_PRIMAL_WAVE,
     ZAP_IOOD,
-    ZAP_BREATHE_MEPHITIC,
+    ZAP_NOXIOUS_BREATH,
     ZAP_INNER_FLAME,
     ZAP_FORCE_LANCE,
     ZAP_SEARING_RAY,
-    ZAP_CRYSTAL_BOLT,
+    ZAP_REBOUNDING_BLAZE,
+    ZAP_REBOUNDING_CHILL,
     ZAP_TUKIMAS_DANCE,
     ZAP_QUICKSILVER_BOLT,
     ZAP_CORROSIVE_BOLT,
@@ -95,8 +95,8 @@ enum zap_type
     ZAP_MIASMA_BREATH,
     ZAP_PETRIFYING_CLOUD,
     ZAP_GHOSTLY_FIREBALL,
-    ZAP_ICY_FLASH_FREEZE,
     ZAP_FLASH_FREEZE,
+    ZAP_CREEPING_FROST,
     ZAP_SPIT_LAVA,
     ZAP_THROW_BARBS,
     ZAP_HURL_SLUDGE,
@@ -118,7 +118,7 @@ enum zap_type
     ZAP_STUNNING_BURST,
     ZAP_FLAME_WAVE,
     ZAP_ENFEEBLE,
-    ZAP_NECROTISE,
+    ZAP_SOUL_SPLINTER,
     ZAP_BOLT_OF_LIGHT,
     ZAP_FASTROOT,
     ZAP_BLASTMOTE,
@@ -130,5 +130,37 @@ enum zap_type
     ZAP_VITRIFY,
     ZAP_MALIGN_OFFERING,
     ZAP_CURSE_OF_AGONY,
+    ZAP_HURL_TORCHLIGHT,
+    ZAP_CRYSTALLIZING_SHOT,
+    ZAP_WARP_SPACE,
+    ZAP_COMBUSTION_BREATH,
+    ZAP_NULLIFYING_BREATH,
+    ZAP_MUD_BREATH,
+    ZAP_GALVANIC_BREATH,
+    ZAP_CAUSTIC_BREATH,
+    ZAP_AUTODAZZLE,
+    ZAP_PERMAFROST_ERUPTION_COLD,
+    ZAP_PERMAFROST_ERUPTION_EARTH,
+    ZAP_GRAVITAS,
+    ZAP_MAGNAVOLT,
+    ZAP_FULSOME_FUSILLADE,
+    ZAP_RIMEBLIGHT,
+    ZAP_RIMEBLIGHT_SHARDS,
+    ZAP_SEISMIC_SHOCKWAVE,
+    ZAP_HOARFROST_BULLET,
+    ZAP_HOARFROST_BULLET_FINALE,
+    ZAP_FLASHING_BALESTRA,
+    ZAP_HELLFIRE_MORTAR_DIG,
+    ZAP_SHADOW_SHARD,
+    ZAP_SHADOW_BOLT,
+    ZAP_SHADOWBALL,
+    ZAP_CREEPING_SHADOW,
+    ZAP_SHADOW_JOLT,
+    ZAP_SHADOW_DRAIN,
+    ZAP_SHADOW_TORPOR,
+    ZAP_GRAVE_CLAW,
+    ZAP_UNLEASH_DESTRUCTION,
+    ZAP_UNLEASH_DESTRUCTION_PIERCING,
+    ZAP_BLOOD_ARROW,
     NUM_ZAPS
 };
